
#ifndef COMPY_GENERAL_H
#define COMPY_GENERAL_H

// behavior types
enum COMPY_TYPE
{
	COMPY_GROUP_SPLINE,
	COMPY_GROUP_WAVE
};

// behavior states
enum COMPY_STATE
{
	COMPY_NONE,
	COMPY_INIT,
	COMPY_WAIT,				// legacy for spline type
	COMPY_WAIT_TRIGGER,
	COMPY_WAIT_DELAY,
	COMPY_RUN,
	COMPY_ATTACK,
	COMPY_ATTACK1
};

// group states
enum COMPY_GROUP_STATE
{
	COMPY_GROUP_NONE,
	COMPY_GROUP_INIT,
};

// death states
enum COMPY_DEATH
{
	COMPY_DEATH_NONE,
	COMPY_DEATH_START,
	COMPY_DEATH_FLYBACK,
	COMPY_DEATH_GROUND
};

struct CompyGroupData;
typedef struct CompyData
{
	int type;					// type of behavior desired
	int state;					// master state
	int attack_state;
	int death_state;			// state to manage death
	crd hit_dir[3];				// direction bullet hit them from
	float start_time;			// delay timer
	float life;					// how long these guys are active before being removed from scene
	float attack_radius;
	uns32 objkill_timer;		// used to remove dead guys

	crd follow_offset[3];

	float hop_time;				// hopping timers
	float hop_time_min;
	float hop_time_max;

	float squeak_time;			// squeaking timers
	float squeak_time_min;
	float squeak_time_max;

	uns32 track_attack;
	uns32 track_death;			// spinning death animation
	uns32 track_hop;			// hopping animation
	int track_hop_clear;		// flag to clear off hopping and reset timers
	
	crd rot_quat[4];			// used to rotate shot compy to correct direction
	crd last_rot_quat[4];

	uns32 exp_id;				// explosion callback handle

	// generally useful stuff
	ObjKeyAnim* okan;
	ObjVel* ov;
	struct CompyGroupData* cgd;
	int grp_indx;
	Obj* parent;
	ObjTrackAnim* otan;
	SteeringAgent agent;
	SM_TravelFunc_Params travelParams;
}CompyData;

typedef struct CompyGroupData
{
	int type;							// copy of behavior type
	int state;							// group state
	Res* group_spline;
	float spline_time;
	float spline_len;
	crd group_spline_mat[VM_SIZE];

	float shift_min_dist;				// min distance between formation points
	float shift_width;					// offset point box dimensions
	float shift_depth;

	char start_trigger[SM_NAME_SIZE];	// trigger these guys get activated on
	float speed;

	// creation box info
	crd create_pt[3];
	crd create_forward[3];
	crd create_rot[3];
	crd create_right[3];

	// sfx
	uns32 squeak_stick;
	uns32 squeak_next;

	// group info
	int num_objs;
	CompyData* cds[DINO_GROUP_MAX_OBJS];

	// ov scale to apply
	float scale;

	// commonly used stuff
	ObjKeyAnim* cam_okan;

	// probably a better grouping mechanism, and hanging any data as obj
	ObjGroup* obj_grp;


}CompyGroupData;


// prototypes

// helpers
DinoGroup* Compy_CreateGroupMember( Obj* prime, Res* res );
int Compy_NextFrameGroundHit( CompyData* cd, float offset );
void Compy_HandleDeath( CompyData* cd );
int Compy_Travel_ObjVel( CompyData* rd, float deltaTime, float pitch );
void Compy_ExplosionCallback( ExplosionData* exp_data, void* data );

// group callbacks
void Compy_GroupUpdate( void* vgrp, void* vdata );
void Compy_GroupDestroy( void* vgrp );

// vel controller
int Compy_Update( ObjVel* objVel, float deltaTime );
int Compy_Free( ObjVelController* velCtrl );

// type specific
void Compy_GrpSpline_InitPath( DinoGroup* grp, CompyGroupData* cgd );
void Compy_GrpSpline_MakeOffsets( DinoGroup* grp, CompyGroupData* cgd );
void Compy_GrpSpline_GroupUpdate( DinoGroup* grp, CompyGroupData* cgd );
void Compy_GrpSpline_Update( ObjVel* objVel, float deltaTime, CompyData* cd, CompyGroupData* cgd );


void Compy_Wave_GroupUpdate( DinoGroup* grp, CompyGroupData* cgd );
void Compy_Wave_Update( ObjVel* objVel, float deltaTime, CompyData* cd, CompyGroupData* cgd );


#endif


