// game.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//

#ifndef GAME_H
#define GAME_H

/// Pickup stuff
typedef enum PICKUP_TYPE
{
	PICKUP_NONE,
	PICKUP_GRENADE,
	PICKUP_HEALTH,
	PICKUP_AMMO,
	PICKUP_BONUS,
	PICKUP_ELECTROGUN,
	PICKUP_ROCKET,
	PICKUP_SHOTGUN,
	PICKUP_FREEZERAY,
	PICKUP_MINIGUN,
	PICKUP_NUM_TYPES
} PICKUP_TYPE;

extern char* gPickupTypeNames[PICKUP_NUM_TYPES];


typedef enum DamageType
{
	DAMAGE_TYPE_RAPTOR,
	DAMAGE_TYPE_RAPTOR_CHASE,
	DAMAGE_TYPE_RAPTOR_GRAPPLE,
	DAMAGE_TYPE_DILO,
	DAMAGE_TYPE_MRAPTOR,
	DAMAGE_TYPE_WASP,
	DAMAGE_TYPE_PTERA,
	DAMAGE_TYPE_BUG,
	DAMAGE_TYPE_COMPY,
	DAMAGE_TYPE_CIRCLE,
	DAMAGE_TYPE_OBSTACLE,
	DAMAGE_TYPE_OTHER,

	DAMAGE_NUM_TYPES

} DamageType;

extern char* gDamageTypeNames[DAMAGE_NUM_TYPES];


typedef enum EnemyType
{
	ENEMY_TYPE_RAPTOR,
	ENEMY_TYPE_RAPTOR_CHASE,
	ENEMY_TYPE_RAPTOR_GRAPPLE,
	ENEMY_TYPE_DILO,
	ENEMY_TYPE_MRAPTOR,
	ENEMY_TYPE_WASP,
	ENEMY_TYPE_PTERA,
	ENEMY_TYPE_BUG,
	ENEMY_TYPE_COMPY,
	ENEMY_TYPE_CIRCLE,
	ENEMY_TYPE_CIRCLE_SEQUENCE,
	ENEMY_TYPE_OBSTACLE,

	ENEMY_NUM_TYPES
} EnemyType;

char* gEnemyTypeNames[ENEMY_NUM_TYPES];



typedef enum VibrateGuns
{
	VIBRATE_GUNS_OFF,
	VIBRATE_GUNS_SINGLE_SOL,
	VIBRATE_GUNS_DUAL_SOL,
	VIBRATE_GUNS_UNIS,
	VIBRATE_GUNS_NAMCO,

	NUM_VIBRATE_GUN_TYPES
} VibrateGuns;



#include <g5/g5.h>
#include "game_sys.h"
#include "game_rtdt_datatypes.h"
#include "game_statemachine_datatypes.h"
#include "game_statemachines.h"
#include "boot.h"
#include "attract.h"
#include "credit_msg.h"
#include "aud_game.h"
#include "game_dongle.h"
#include "game_flow.h"
#include "game_flow_ggg.h"
#include "gameflow_records.h"
#include "game_score.h"
#include "game_score_ggg.h"
#include "game_results.h"
#include "progress_map.h"
#include "game_hud.h"
#include "game_player.h"
#include "screen_fx.h"
#include "game_diag.h"
#include "game_proc.h"
#include "game_snd.h"
#include "game_gun.h"
#include "game_gun_mg.h"
#include "game_gun_grenade.h"
#include "game_gun_beam.h"
#include "game_gun_electro.h"
#include "game_gun_tracker.h"
#include "game_gun_rocket.h"
#include "game_gun_shotgun.h"
#include "game_gun_freeze.h"
#include "game_gun_minigun.h"

#include "game_2D.h"
#include "bullet_sm.h"
#include "tracker_sm.h"
#include "enemy_statemachine.h"
#include "cera_statemachine.h"
#include "wasp_statemachine.h"
#include "wasp_swarm_statemachine.h"
#include "ptrdon_sm.h"
#include "ptrdon_grp_sm.h"
#include "trex_sm.h"
#include "jeep_statemachine.h"
#include "tricera_statemachine.h"
#include "placeholder_statemachine.h"
#include "mraptor_statemachine.h"
#include "compy_statemachine.h"
#include "compyspline_statemachine.h"
#include "raptor_statemachine.h"
#include "crawlerbug_statemachine.h"
#include "screenbug_statemachine.h"
#include "wrkr_fight.h"
#include "raptor_ggg_statemachine.h"
#include "obs_shoot_statemachine.h"
#include "level_select.h"
#include "level_select2.h"
#include "level_select3.h"
#include "level_select_ggg.h"
#include "level_load.h"
#include "game_dialog.h"
#include "lamps.h"


// special Jurassic Park yellow color
extern float gJPYellow[V3_SIZE];


typedef struct BangupDoorsData
{
	uns32 doorsOID;

	// list of lights created for the bangup screen
	List lights;

	// keep track of whether the guns try to turn on during this screen.
	int gunsState;
	int initDone;

} BangupDoorsData;

typedef enum BangupDoorsGunsState
{
	BANGUP_DOORS_GUNS_ON,
	BANGUP_DOORS_GUNS_OFF,
	BANGUP_DOORS_GUNS_HOLD_FIRE,
};

int BangupDoorsActive();


int GameIsBossActive();
void GameBossOn();
void GameBossOff();

// health in range [0.0, 1.0]
void GameSetBossHealth(float pct);
float GameGetBossHealth();


#endif // GAME_H
