// game_gun.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//

#ifndef GAME_GUN_H
#define GAME_GUN_H


#include "game.h"


extern int gGunsOn;
extern int gGunModelsOn;
extern int gGunUseForceFeedback;
extern int gGunUseForceFeedbackEnabled; // for namco kit to have enable/disable as different adjustment

typedef enum GunType
{
	GUN_TYPE_MG,
	GUN_TYPE_GRENADE,
	GUN_TYPE_BEAM,
	GUN_TYPE_TRACKER, // dino tracking beacon
	GUN_TYPE_ELECTRO,
	GUN_TYPE_ROCKET,
	GUN_TYPE_SHOTGUN,
	GUN_TYPE_FREEZE,
	GUN_TYPE_MINIGUN,

	NUM_GUN_TYPES

} GunType;

#define MAX_GUN_PARTS 5


// gun firing states
enum GUN_STATE
{
	GUN_INACTIVE,
	GUN_STARTING_SHOOT,
	GUN_SHOOTING
};


enum AUTO_RELOAD_STATE
{
	AUTO_RELOAD_NONE,
	AUTO_RELOAD_START
};


typedef enum
{
	GAME_GUNCAL_ONE_POINT=0,
	GAME_GUNCAL_FIVE_POINT,
} gameGuncalType;


// number of triggers per gun that fire a weapon
#define GUN_NUM_SHOOT_TRIGS 2


typedef struct PlayerGunData
{
	GunType curType[GUN_NUM_SHOOT_TRIGS];
	crd modelWorldPos[V3_SIZE];

	// has to be implemented separately for each gun type
	int infinite_ammo;

} PlayerGunData;


extern PlayerGunData gPlayerGun[GAME_MAX_PLAYERS];
extern GunType gGunDfltType[GUN_NUM_SHOOT_TRIGS];

typedef enum FFDUAL_TYPE
{
	FFDUAL_OFF,
	FFDUAL_MG,
	FFDUAL_MINIGUN,
	FFDUAL_FREEZE,
	FFDUAL_ELECTRO,
	FFDUAL_SHOTGUN,
	FFDUAL_SOL1TEST_SINGLE,
	FFDUAL_SOL2TEST_SINGLE,
	FFDUAL_SOL1TEST_REPEAT,
	FFDUAL_SOL2TEST_REPEAT,

	FFDUAL_CNT

} FFDUAL_TYPE;

int GameGunInit(void);

// init/destroy gun stuff for a single scene
void GunCreate();
void GunDestroy();

void GunUpdate();

void GunSwitchAll(uns32 trigger, GunType newType);
void GunSwitch(uns32 trigger, GunType newType, int pnum);

void GunSetConfig();
int GunUseForceFeedback();
void GunFFPulseDual(uns32 pnum, FFDUAL_TYPE type);
void GunFFPulseUnis(uns32 pnum);
void GunFFPulseNamco(uns32 pnum, FFDUAL_TYPE type);
void GunFFPulse(uns32 pnum, int pulse, int period);
// get the gun number for the trigger input
uns32 GunInputToIdx(int input);


int GunShoot( uns32 trig, int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );

// input callbacks
int GunCbTrig0( int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );
int GunCbTrig1( int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );
int GunCbTrig2( int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );
int GunCbStart( int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );
int GunCb_TAP( int input, uns32 state, io_input_analog *ai, io_input_digital *trigger, void *data );


// 
int IsBigDino( ObjDispRes* odr );
int ProjColFuncBasic(int cf,void *p0,void *p1,void *p2);
int ProjColFuncGrenade(int cf,void *p0,void *p1,void *p2);
int ProjColDeathFuncBasic(void *p0);
int GunAddCol(Obj *o, int pnum,  int gnum, uns32 projType, ftype4 *colFunc , Obj* parent, ObjColAct **oca);
int GunAddCol2D(float x, float y, int pnum, int gnum, ftype4 *colFunc, Obj *parent, ObjColAct **oca);

int GunCol_AttachFX(char* fxGroup, ObjColAct* cola);

int GunGetColor(int gnum, float *rgb );

int GunGetAimPos(GunInput* gun, crd* outPos, int pnum, ftype4 *colFunc);

int CalibrateGunFromAuditFile(int gnum, int gunCalType);
int GunCalibrate(int gnum, int gunCalType);

int GunCalProcStart(void *params);
void GunCalProcRun();
void GunCalProcKill(void *params);

void GunModelsOff();
void GunModelsOn();

int Pickup_ReadType(char* pickupName);
void Pickup_Shot(int player, int pickupType, ObjDispRes* odr);
void Pickup_Create(int pickupType);

int CursorGetAimPos(crd* cursorPos, crd* outPos, int pnum, ftype4 *colFunc);

void GunVibrateSetup();

#endif // GAME_GUN_H

