// game_hud.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//


#ifndef GAME_HUD_H
#define GAME_HUD_H

#include "game.h"


// continue time in ms
#define CONT_DISP_TIME			30000

#define RETICLE_BASE_SCALE	8.0f
#define RETICLE_MINIGUN_BASE_SCALE	20.0f

#define HUD_SORT_PRI	0.0f
#define HUD_SORT_PRI_RETICLE	-5.0f
#define HUD_SORT_PRI_CONT	15.0f
#define HUD_SORT_PRI_CONT_TEXT	16.0f

#define OBJCAM_HUD	OBJCAM4
#define OBJCAM_RETICLE	OBJCAM4
// put it on a cam that won't shake and keep it under the reticles
#define OBJCAM_LETTER_BOX	OBJCAM1


// external stuff

enum
{
	HUD_VISIBLE,
	HUD_HIDDEN
};

enum
{
	HUD_PANEL_ACTIVE_START,
	HUD_PANEL_ACTIVE,
	HUD_PANEL_DEAD_START,
	HUD_PANEL_DEAD_ROT_HALF,
	HUD_PANEL_DEAD_ROT_FINISH,
	HUD_PANEL_JOIN
};

enum
{
	HUD_BOSS_NONE,
	HUD_BOSS_ACTIVATE,
	HUD_BOSS_UPDATE,
	HUD_BOSS_DEACTIVATE
};

// reticle types
enum
{
	RETICLE_TYPE_TARGETING,
	RETICLE_TYPE_TARGETING_MINIGUN,
	RETICLE_TYPE_NO_SHOOT,
	RETICLE_TYPE_CNT
};

void HUD_Create();
void HUD_CreateLightGroups();
void HUD_DestroyLightGroups();
void HUD_Destroy();

void HUD_Update();				// update hud and reticles
void HUD_UpdateReticles();		// update only reticles
void HUD_SetReticleColor(int pnum, float* clr);
void HUD_ReticlesEnableLockColRes();
void HUD_ReticlesDisableLockColRes();
void HUD_ReticlesSetType( int type );
void HUD_ReticleSetType( int pnum, int type );
// if guns are on, show active and hide inactive
void HUD_ReticlesShowActive();
// show active even if guns are turned off
void HUD_ReticlesShowActiveForce();
void HUD_ReticlesHide();

// show main panel and health bar
void HUD_HealthPanelsShowActive();
// offset health panel position for one player
void HUD_SetHealthPanelOffset(int pnum, crd* trans);
// offset health panel for all players
void HUD_SetHealthPanelsOffsets(crd* trans);
void HUD_UpdateLifeMeters();

void HUD_SetInitialPlayerState();
void HUD_ContinueInactivePlayers();
void HUD_SetPanelState( int pnum, int state );
void HUD_SetBossState( int state );

void HUD_Hide();
void HUD_Show();
void HUD_SetVisibility(int visEnum);
void HUD_ShowLetterBox();
void HUD_HideLetterBox();
void HUD_HideLetterBoxForce();
void HUD_ShowHoldFireText();
void HUD_HideHoldFireText();

void HUD_StackerAddDinoKill( int pnum, int dino );
void HUD_StackerAddBonusScore( int pnum, int bonus );
void HUD_UpdateScore( int pnum );

void HUD_PlayerHit( int pnum );
void HUD_FriendlyShot( ObjColAct* cola );

void HUD_GunFired( int pnum );
void HUD_GunChanged( int pnum );
void HUD_PickupShot( int pnum, int type, ObjDispRes* odr );

int HUD_IsActive();

int HUD_ContinueCanTrigContinue( int pnum );
int HUD_ContinueIsPopUpEnabled( int pnum );
int HUD_ContinueIsPopUpEnabledAll();
int HUD_ContinueIsPopUpEnabledAny();
int HUD_ContinueAddedMoney( int pnum );

void HUD_SetHealthScaleSmooth(int smooth);

ObjLightRes* HUD_GetLight(char* name);

float HUD_GetLastGunFireTime(int pnum);

void HUD_CreditMsgSetPanelPos(int pnum);

extern int gShowHudInTool;


#endif // GAME_HUD_H