// game_rtdt_datatypes.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//
//
// data types for game use


// NOTES:
// only simple defines are supported by RTDataParser
// only local defines are supported by RTDataParser 
// only 'typdef struct's are supported by RTDataParser
// only single dimension arrays are supported
//
// NOTE:
// use game_statemachine_datatypes.h for structs related to state machines

#ifndef GAMERTDTDATATYPES_H
#define GAMERTDTDATATYPES_H


#pragma pack(4)

// stuff goes here

#define GAME_MAX_PLAYERS		4
#define GAME_MAX_GRENADES		5

#define GAME_LEVEL_TRICERA		0
#define GAME_LEVEL_TREX			1
#define GAME_LEVEL_BRAC			2
#define GAME_NUM_LEVELS			3
#define GAME_ACTS_PER_LEVEL		3
#define GAME_NUM_GEN_SECTIONS	4


// attract page stuff
typedef struct AttractPageInfo
{
	int type;
	char scene_name[32];
	int time;
	int show_coins;
	int preload;
	int noWipeoutAtStart;

	char loadadd_name[32];
	char loadadd_cam[32];
	int  loadadd_camnum;
	
	char movie_obj[32];
	char sfx[32];

	int i_preloaded;


} AttractPageInfo;


#define GAME_HST_MAX_SCORES	12
#define GAME_HST_MAX_TABLE_TYPES	8
#define GAME_HST_NAME_STR_SIZE	32


typedef struct GameScore
{
	wchar_t Initials[GAME_HST_NAME_STR_SIZE];
	// make it a double so it can handle storing ints too
	double score;

} GameScore;


typedef struct GameScoreTable
{
	GameScore scores[GAME_HST_MAX_SCORES];

} GameScoreTable;


typedef struct GameScoreLevel
{
	GameScoreTable tbls[GAME_HST_MAX_TABLE_TYPES];

} GameScoreLevel;


typedef struct GameHighScores
{
	int version;

	// tables for each level, plus global
	GameScoreLevel lvls[GAME_NUM_LEVELS+1];

} GameHighScores;


// max number of general sections per level
#define GAME_AUD_MAX_GENERAL 8


typedef struct GeneralAudit
{
	// seconds spent in this general section
	float sec;
	// credits spent in this general section (based on damage taken)
	float cred;

} GeneralAudit;

typedef struct LevelAudit
{	
	int starts;
	int deaths;
	int continues;
	// total levels beaten
	int finishes;
	// number of times the player beat exactly x levels (0,1,2,3) before quitting.
	int finishes_exact[4];
	// number of times the game was beaten (captured all dinos)
	int gameFinishes;
	int abandons;

	int totalFails;
	// number of times player reached the end of the level (finished or failed at least once)
	int levelEndReached;
	// number of times player finished the level after failing at least once
	int winsAfterFail;
	// number of times player didn't continue after failing at least once
	int lossesAfterFail;

	int grenadesFired;
	// total players who fired at least one grenade
	int firedAnyGrenades;

	// time (sec) in the level (doesn't include select menu, loading, results)
	uns32 levelTime;
	// total play time (sec) of each player added together
	uns32 playerTime;

	uns32 creditsSpent;
	
	// [0] is times obtained, [1] is total times appeared
	int pickupHealth[2];
	int pickupBonus[2];
	int pickupElectro[2];
	int pickupRocket[2];
	int pickupShotgun[2];
	int pickupMinigun[2];
	int pickupFreezeray[2];

	// total damage taken from each enemy type.
	// array size should equal DAMAGE_NUM_TYPES.
	float damageTaken[12];

	// array size should equal ENEMY_NUM_TYPES.
	int attackersSpawned[12];
	int attackersKilled[12];

	GeneralAudit genAud[8]; //[GAME_AUD_MAX_GENERAL]

} LevelAudit;

typedef struct GameplayAudit
{
	int loadedFromDefault;
	// count toward high score reset
	int hsrGames;

	LevelAudit level_tricera[4];
	LevelAudit level_brac[4];
	LevelAudit level_trex[4];

} GameplayAudit;


#endif // GAMERTDTDATATYPES_H
