// game_statemachines_datatypes.h
// Play Mechanix - Video Game System
// Copyright (c) 2010 Play Mechanix Inc. All Rights Reserved
//
// data types for game statemachines

// NOTRS:
// only simple defines are supported by RTDataParser
// only local defines are supported by RTDataParser 
// only 'typdef struct's are supported by RTDataParser

////////////////////
// SM PARMS
//
// Structs used for State Machine Params
// these are the global settings for a state machine
////////////////////

#ifndef GAMESTATEMACHINESDATATYPES_H
#define GAMESTATEMACHINESDATATYPES_H

#pragma pack(4)

// state machine params for our general purpose state machine
typedef struct SM_GameCommon_Params
{
	SM_Common_Params common; // this one should be first item in params struct for all statemachines
} SM_GameCommon_Params;

typedef struct SM_2D_Appear_Params
{
	float e_n_xdist;
	float e_n_ydist;
	float e_n_frames;
} SM_2D_Appear_Params;


typedef struct SM_G2D_UI_HoverScale
{
	float	e_n_minScale;
	float	e_n_maxScale;
	int	e_n_shrink_ms;
	int e_n_grow_ms;
} SM_G2D_UI_HoverScale;

typedef struct SM_G2D_UI_HoverColor
{
	int e_cb_red;
	float e_n_r;

	int e_cb_green;
	float e_n_g;

	int e_cb_blue;
	float e_n_b;

	int e_cb_alpha;
	float e_n_a;

	int e_cb_mult;

	int e_n_newColor_ms;
	int e_n_oldColor_ms;
} SM_G2D_UI_HoverColor;

typedef struct SM_G2D_UI_HoverTrigger
{
	char e_s_Hover_Enter[64];
	char e_s_Hover_Exit[64];
} SM_G2D_UI_HoverTrigger;


typedef struct SM_G2D_CamSeq_Params
{

	char	e_s_Basename[32];
	int e_n_First;
	int e_n_Last;
	int e_n_Start;
	char	e_s_ForwTrigger[32];
	char	e_s_BackTrigger[32];
	char	e_s_ExitTrigger[32];

	int e_cb_Loop;
	int e_cb_DoTransCams;
	int e_cb_SetCamTrigs;
	int e_cb_SetAppear;
	float e_n_xdist;
	float e_n_ydist;
	float e_n_frames;
	int e_cb_CallDisappear;

	int  i_n_curcam;
	int  i_n_range;

} SM_G2D_CamSeq_Params;


typedef struct SM_TimerWatch_Params
{
	int e_n_Time;
	int e_cb_Color;
	float e_n_rgb[3];
	int e_cb_Scale;
	float e_n_ScaleUp;
	float e_n_ScaleSpeed;
	int e_cb_AlertTrigger;
	char	e_s_Trigger[32];

	float i_n_start_rgb[3];
	float i_n_ScaleBase[3];
	uns32 i_n_prevTime;

} SM_TimerWatch_Params;

typedef struct SM_TimerSet_Params
{
	int e_n_Time;
	int e_cb_noTapout;
} SM_TimerSet_Params;














typedef struct SM_ConfItem_Params
{
	char e_s_ConfigItem[32];
} SM_ConfItem_Params;


typedef struct SM_Enemy_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	int i_isInitd;

	float i_Health;
	float i_Health1p;
	float i_Health2p;

	// holds all circles waiting to be added
	uns32 i_addCircleGroupOID;
	float i_circleSleepMS;
	uns32 i_CircleGroupOID;
	uns32 i_FailCircleGroupOID;
	int   i_CircleSuccesses; // number of times BR_ON_CICRLES has branched to the 'you got all the circles' path
	int i_NumCircles; // total number of circles in the current sequence
	int i_CirclesLeft;
	int i_CircleHits[4]; // number of time each player hit a circle in sequence
	uns32 i_CircleNotifyKillOID; // oid of object that gets sent 'circle_killed' event
	int i_CircleMaxPerObj; // if non 0 limits number of circles created on an object
	char e_s_ObjName[32];
	char e_s_ObjGroup[32];
	char e_s_TriggerName[32];
	char e_s_ShowTriggerName[32];

	// explosion handle for circles
	uns32 i_circleExplId;

	// times in seconds
	float delay;
	float groupDelay;
	float groupShowDelay;
	int useCamSpeed;

	int isLaunched;
	int isShown;
	// absolute game time (ms) at time of launch
	uns32 launchTime;

	int32 i_useStinger;
	int32 i_stingerSndId;

	char e_s_CircleKillFX[32];

} SM_Enemy_Params;


typedef struct SM_Bullet_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	uns32 i_pnum;

} SM_Bullet_Params;


typedef struct SM_Tracker_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	uns32 i_tgtOID;
	uns32 i_startTime;
	crd i_dir0[V3_SIZE];

} SM_Tracker_Params;


typedef struct SM_Placeholder_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines


	float e_n_Health;
	float e_n_Speed;
	float e_n_TriggerDist;
	char e_s_StartTrigger[SM_NAME_SIZE];
	char e_s_ObjName[32];

} SM_Placeholder_Params;


typedef struct SM_Microraptor_Params
{
	SM_Enemy_Params enemy;

	float e_n_Speed;
	float e_n_SteerForce;
	float e_n_MinAng;
	float e_n_MaxAng;
	int e_cb_Attack;
	int e_cb_ignorePlayer;
	float e_n_TriggerDist;
	// min players required to activate this mraptor
	int e_cb_minPlayers;
	int e_cb_maxPlayers;
	int e_cb_startRelative;

	// animation tracks
	uns32 idleTrack;
	uns32 jumpTrack;
	uns32 glideTrack;
	uns32 arriveTrack;
	uns32 holdTrack;
	uns32 atkTrack;
	uns32 deathHitTrack;
	uns32 deathFallTrack;
	uns32 deathLandTrack;

	// index for the name of the current idle animation
	uns32 curIdleAnim;
	
	// distance to stop in front of camera for attack hold state
	float holdDist;

	uns32 expId;

	// ggg stuff
	int attack_screen;

} SM_Microraptor_Params;


typedef struct SM_WaspSwarm_Params
{
	SM_Enemy_Params enemy;

	void* swarmData;

} SM_WaspSwarm_Params;


typedef struct SM_PtrdonGrp_Params
{
	SM_Enemy_Params enemy;

	void* data;

} SM_PtrdonGrp_Params;


typedef struct SM_Wasp_Params
{
	SM_Enemy_Params enemy;

	SM_WaspSwarm_Params* swarmParams;

	int startState;

	void* waspData;

} SM_Wasp_Params;


typedef struct SM_Ptrdon_Params
{
	SM_Enemy_Params enemy;

	SM_PtrdonGrp_Params* grpParams;

	void* ptrdonData;
	void* pathRes;

} SM_Ptrdon_Params;





// stuff related to 'shoot the circle attacks'

typedef struct SM_Enemy_AddCircle_Params
{
	char e_s_res[32];

	float e_n_HP_1P;
	float e_n_HP_2P;
	float e_n_Radius;
	char e_n_Joint[32];
	char e_s_Offset_Spline[32];
	float e_n_Offset_xyz[3];
	int e_cbg_BindSpace;
	int e_cbg_JointSpace;
	int e_cbg_PivotSpace;
	char e_s_Anim[32]; // ding animation to do when you die...
	int  e_n_BlendFrames; // frames to blend the ding anim out...
	float e_n_ScaleSpeed;
	// frames to sleep after adding this circle
	float e_n_SleepAfter;
	// absolute time to sleep after the start of the circle sequence
	float i_Sleep;
	// sfx when the circle dies
	char e_n_SFX[32];

} SM_Enemy_AddCircle_Params;

typedef struct SM_CircleData
{
	float radius;
	float init_hp; // initial hp
	float hp;      // current hp
	float hit_rot; // how much to rotate on a hit
	float hit_scale; // scale to pop to on a hit
	int   do_color_blend; // blend color as hp goes down
	float blend_color0[3]; // three colors, start color, 50% dead color, 100% dead color....
	float blend_color1[3];
	float blend_color2[3];
	int   do_fade_out;
	RTDataSmallString  ding_anims[2];  	// we can apply dings to up to two objs (this is done to support the hktank which is two objs in one...)
	uns32 ding_oid[2]; // oid of obj to apply ding to
	int   ding_time;
	int   killCreditGiven;
	uns32 lastHitTime;
	uns32 notify_kill_oid;
	int partNum;
	// set to true while circle scale is above its its radius
	int isScaling;
	// sfx when the circle dies
	char sfx[32];

	uns32 addGrpOid;
	uns32 grpOid;

} SM_CircleData;

typedef struct SM_TimeoutBr_Params
{
	char  e_s_Label[32];
	float	e_n_Time;
	int		e_n_CamFrame;
	int		e_n_AnimFrame;
} SM_TimeoutBr_Params;


typedef struct SM_EnemyWaitCircle_Params
{
	SM_TimeoutBr_Params timeoutParams;	

	int i_circlesFailed;
	float i_circlesFailedTime;
	// save original postfx saturation value
	float i_saturation;

	int i_disableFailCircles;

	char e_s_CircleKillFX[32];
	int e_cb_preventVO;

} SM_EnemyWaitCircle_Params;


typedef struct SM_Enemy_DamagePlayer_Params
{
	float e_n_Damage;
	int e_cb_AllPlayers;
	char e_s_ScreenFX[32];
}SM_Enemy_DamagePlayer_Params;



typedef struct SM_ObsShootDmgPlayer_Params
{
	float e_n_EndFrame;
	int e_cbg_NoDestruct;

} SM_ObsShootDmgPlayer_Params;



// state machine params for our cera state machine
typedef struct SM_Cera_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	// chase speed
	float i_cur_speed_pct;			// the current pct we are using (slow down/speed up)
	float i_last_speed_pct;			// for smoothing

	// hit speed control
	int i_hit_state;				// controls hit behavior
	uns32 i_hit_tick;				// time hit occurred at

	// hit flashing
	int i_hit_flash_state;
	uns32 i_hit_flash_stick;
	int i_hit_flash_cnt;
	
	// attack speed control
	int i_attack_state;
	uns32 i_attack_tick;
	int i_attack_hit_cnt;

	// flinch animation
	uns32 i_flinch_track;			// track animation id
	uns32 i_flinch_stick;

	// attack animation
	uns32 i_attack_track;

	// death animation
	uns32 i_death_track;
	int i_death_hit_cnt;

	// attack interrupt animation
	uns32 i_interrupt_track;

	// group stuff
	int i_group_id;
	int i_group_index;

	// flags
	int i_chase_active;

	// chase direction smoothing
	float i_last_dir_quat[4];

	// spline to chase speed transition
	float i_trans_start_speed;
	float i_trans_end_speed;
	uns32 i_trans_stick;
	int i_trans_state;

	// color shift
	float e_n_ClrR;
	float e_n_ClrG;
	float e_n_ClrB;

	// scale
	float e_n_Scale;
	
} SM_Cera_Params;

typedef struct SM_Cera_Init_Params
{
	// group name actors are in
	char e_s_ActorGroup[32];

	// speed control
	float e_n_SpeedPct;
	float e_n_SpeedPctSmooth;

	// transitioning speed control
	float e_n_TransSec;

	// chase point
	float e_n_PathWidth;
	float e_n_MinPathWidth;
	float e_n_FullPathDist;

	// attacking
	float e_n_AttackDist;
	float e_n_AttackSlowPct;
	float e_n_AttackSlowSec;
	float e_n_AttackRecoverSec;

	// getting hit
	float e_n_HitSlowPct;
	float e_n_HitRecoverSec;
	float e_n_HitMinSpeedPct;
	float e_n_DeathCamFrame;

	// camera control
	float e_n_CameraSmooth;
	float e_n_ReturnCamFrame;
	float e_n_ReturnCamSec;

	// smooth for setting dino rot
	float e_n_TurnSmooth;

} SM_Cera_Init_Params;

typedef struct SM_Cera_Chase_Params
{
	// piggybacking on travel state machine
	SM_Travel_Params travelParams;

	// spline that gives starting position
	char e_s_StartSpline[32];
	float e_n_StartSplineFrame;

	// spline we will fake from
	ObjKeyAnim* i_fake_spline;

	// fake key data built for travel
	char i_fake_okan_res_data[sizeof(ResDataKan)+sizeof(ResDataKanKey)*2];

} SM_Cera_Chase_Params;


typedef struct SM_Compy_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	int i_group_id;
	int i_group_index;
	void* cd;
} SM_Compy_Params;

typedef struct SM_Compy_Attack_Params
{
	// piggybacking on travel state machine
	SM_Travel_Params travelParams;

	// fake key data built for travel
	char i_fake_okan_res_data[sizeof(ResDataKan)+sizeof(ResDataKanKey)*2];
} SM_Compy_Attack_Params;

typedef struct SM_Compy_Spline_Params
{
	SM_Common_Params common;
	void* csd;
}SM_Compy_Spline_Params;

typedef struct SM_CrawlerBug_Params
{
	SM_Common_Params common;
	void* cbd;
}SM_CrawlerBug_Params;

typedef struct SM_ScreenBug_Params
{
	SM_Common_Params common;
	void* data;
}SM_ScreenBug_Params;

typedef struct SM_WrkrFightEnemy_Params
{
	SM_Common_Params common;
	void* fightData;
}SM_WrkrFightEnemy_Params;

typedef struct SM_Raptor_Params
{
	SM_Common_Params common;		// this one should be first item in params struct for all statemachines

	void* rd;
} SM_Raptor_Params;

typedef struct SM_DiloSpit_Params
{
	SM_Common_Params common;
	void* SpitData;
}SM_DiloSpit_Params;


typedef struct SM_ObsShoot_Params
{
	SM_Enemy_Params enemy;

	// game speed altering
	float i_game_speed_start;
	float i_game_speed_dest;
	float i_game_speed_fade_time;
	uns32 i_game_speed_timer;

	// circle kills
	int i_circle_kills;

} SM_ObsShoot_Params;


typedef struct SM_BolderEffects_Params
{
	// fx group
	char e_s_FXGroup[64];

	// cool downs
	unsigned int e_n_ParticleCooldownMS;
	unsigned int e_n_CamShakeCooldownMS;

	// camera spring thresholds
	float e_n_ShakeMinBoundingRadius;
	float e_n_ShakeMinDist;

	SM_Camera_Shake_Params shake;

	float i_LastFrame;		// last animation frame
	float i_LastY;			// last y world position

	// cool down timers
	unsigned int i_OFGCoolDownTimer;
	unsigned int i_ShakeCoolDownTimer;

} SM_BolderEffects_Params;

typedef struct SM_CliffCollapseEffect_Params
{
	// fx group
	char e_s_FXGroup[64];

	// cam min dist stuff
	float i_MinDist_Sq;
	float i_MinDistCrd[3];

	// cliff side movement
	float i_LastTrans[3];

} SM_CliffCollapseEffect_Params;

typedef struct SM_LevelSelectRun_Params
{
	unsigned int i_state;
	unsigned int i_tick;
	int i_sel_panel;
	int i_info_zoom_back;
	unsigned int i_last_info_back_col_tick;
	int i_info_back_hover_state;
	unsigned int i_last_info_start_col_tick;
	int i_info_start_hover_state;
	void* i_cola[4];	
	
	// panel selection stuff
	int i_hover_panel[4];
	int i_last_hover_panel[4];
	unsigned int i_last_hover_tick[4];
	unsigned int i_hover_time_tick[4];
	int i_last_selected_panel;
	int i_last_selected_panel_prev;

	// dino guy data
	int i_dino_state;
	unsigned int i_dino_tick;
	int i_cur_dino;
	int i_dino_rspeed_state;
	unsigned int i_dino_rspeed_tick;
	float i_dino_rot;
	float i_dino_rot_speed;

	// terrain border data
	int i_tb_state;
	int i_tb_panel;

	unsigned int i_rand_flick_time;
	unsigned int i_rand_flick_tick;

	unsigned int i_rand_terrain_flick_time;
	unsigned int i_rand_terrain_flick_tick;
	float i_rand_flick_cnt;
} SM_LevelSelectRun_Params;

typedef struct SM_LevelSelectBntFlash_Params
{
	unsigned int i_FlashTick;
	unsigned int i_FlashTime;
	unsigned int i_FlashState;
} SM_LevelSelectBntFlash_Params;


typedef struct SM_LevelSelect2Run_Params
{
	unsigned int i_state;
	unsigned int i_tick;
	int i_sel_panel;
	int i_info_zoom_back;
	unsigned int i_last_info_back_col_tick;
	int i_info_back_hover_state;
	unsigned int i_last_info_start_col_tick;
	int i_info_start_hover_state;
	void* i_cola[4];	

	// panel selection stuff
	int i_hover_panel[4];
	int i_last_hover_panel[4];
	unsigned int i_last_hover_tick[4];
	unsigned int i_hover_time_tick[4];
	int i_last_selected_panel;
	int i_last_selected_panel_prev;

	// dino guy data
	int i_dino_state;
	unsigned int i_dino_tick;
	int i_cur_dino;
	int i_dino_rspeed_state;
	unsigned int i_dino_rspeed_tick;
	float i_dino_rot;
	float i_dino_rot_speed;

	// terrain border data
	int i_tb_state;
	int i_tb_panel;

	unsigned int i_rand_flick_time;
	unsigned int i_rand_flick_tick;

	unsigned int i_rand_terrain_flick_time;
	unsigned int i_rand_terrain_flick_tick;
	float i_rand_flick_cnt;
} SM_LevelSelect2Run_Params;

typedef struct SM_LevelSelect2BntFlash_Params
{
	unsigned int i_FlashTick;
	unsigned int i_FlashTime;
	unsigned int i_FlashState;
} SM_LevelSelect2BntFlash_Params;

typedef struct SM_Health
{
	unsigned int e_i_1P;
	unsigned int e_i_2P;
	unsigned int e_i_3P;
	unsigned int e_i_4P;
}SM_Health;


typedef struct SM_LightningOnParams
{
	char e_s_NewSkyTexture[64];
	float e_n_FrequencyScale;

} SM_LightningOnParams;


typedef struct SM_GGGRaptorAttPts_Params
{
	int e_n_Screen;
	char e_s_KanPoints[64];
} SM_GGGRaptorAttPts_Params;

typedef struct SM_GGGRaptorSpawnPtActivate_Params
{
	char e_s_Name[64];
	float e_n_SpawnRate;
	float e_n_Time;
} SM_GGGRaptorSpawnPtActivate_Params;



typedef struct SM_SndFootStep_Params
{
	SM_Common_Params common;
	char e_s_ObjName[64];
	char e_s_SFX[64];
	char e_s_RFX[64];
	float e_n_MinMS;
	int e_cb_SkipFoot0;
	int e_cb_SkipFoot1;
	int e_cb_SkipFoot2;
	int e_cb_SkipFoot3;
	int e_cb_3D;
	float e_n_Rolloff;

	unsigned int i_n_oid;
	float i_n_cur_vol;
	unsigned int i_n_last_sfx_tick;
	int i_n_foot_state[4];	// four feet max
} SM_SndFootStep_Params;

typedef struct SM_SndFootStep_Play_Params
{
	float e_n_StartFrame;
	float e_n_StartVol;
	float e_n_EndVol;
	float e_n_BlendFrames;
	float e_n_EndFrame;

	unsigned int i_n_blend_tick;
	int i_n_blend_state;
} SM_SndFootStep_Play_Params;



typedef struct SM_ShakeFootStep_Params
{
	SM_Common_Params common;

	// shake parameters
	int e_cb_Shake;
	float e_n_Intensity;
	float e_n_Seconds;
	int e_n_Period;
	int e_cb_RotOnly;
	int e_cb_TransOnly;

	// spring parameters
	int e_cb_Spring;
	float e_n_DispX;
	float e_n_DispY;
	float e_n_DispZ;
	float e_n_ForceX;
	float e_n_ForceY;
	float e_n_ForceZ;
	float e_n_SmoothX;
	float e_n_SmoothY;
	float e_n_SmoothZ;
	float e_n_DirX;
	int e_cb_RndDirX;
	float e_n_DirY;
	int e_cb_RndDirY;
	float e_n_DirZ;
	int e_cb_RndDirZ;
	float e_n_MaxOscillations;

	// foot step stuff
	char e_s_Name[64];

	// internal stuff
	unsigned int i_n_oid;
	unsigned int i_n_foot_state[4];	// four feet max

	char e_s_CamName[64];
	
	// sound
	char e_s_SFX[64];
	int e_cb_3D;

} SM_ShakeFootStep_Params;

typedef struct SM_ShakeFootStep_Play_Params
{
	float e_n_StartFrame;
	float e_n_EndFrame;
	float e_n_MinDist;
	float e_n_MaxDist;
	float e_n_Intensity;

} SM_ShakeFootStep_Play_Params;

typedef struct SM_AnimSFX_Params
{
	SM_Common_Params common;

	char i_s_Anim[64];
	float i_n_Frame;
	float i_n_Frame1;
	float i_n_Frame2;
	char i_s_SFX[64];	

	unsigned int i_n_SndID;
	float i_n_KanFrame;

} SM_AnimSFX_Params;

typedef struct SM_AnimSFX_Init_Params
{
	char e_s_Anim[64];
	float e_n_Frame;
	char e_s_SFX[64];	
} SM_AnimSFX_Init_Params;

typedef struct SM_AnimSFX_ExtraFrames_Params
{
	float e_n_Frame1;
	float e_n_Frame2;
} SM_AnimSFX_ExtraFrames_Params;

typedef struct SM_AnimSFX_Play_Params
{
	float e_n_CamStartFrame;
	float e_n_CamEndFrame;
	float e_n_Vol;
	int e_n_Pri;
	float e_n_Rolloff;
} SM_AnimSFX_Play_Params;

typedef struct SM_AnimSFX_MuteAnimAction_Params
{
	float e_n_StartFrame;
	float e_n_EndFrame;
} SM_AnimSFX_MuteAnimAction_Params;

typedef struct SM_AutoFF_Params
{
	char e_s_CamName[64];
	float e_n_Frame;
	float e_n_GameSpeed;
	int e_cb_Enabled;
	float i_n_FOV;
	char i_s_StartCam[64];
} SM_AutoFF_Params;

typedef struct SM_WaterDropsRep_Params
{
	unsigned int e_n_RepTimeMS;
	unsigned int e_n_RepDelayMinMS;
	unsigned int e_n_RepDelayMaxMS;
	unsigned int i_n_StartTick;
	unsigned int i_n_NextTick;
} SM_WaterDropsRep_Params;

typedef struct SM_WaterDropsRoar_Params
{
	float e_n_Intensity;
} SM_WaterDropsRoar_Params;

typedef struct SM_WaterDropsRain_Params
{
	float e_n_StartFrame;
	float e_n_StopFrame;
	unsigned int e_n_MinTimeMS;
	unsigned int e_n_MaxTimeMS;
	int e_n_MinSmall;
	int e_n_MaxSmall;
	int e_n_MinBig;
	int e_n_MaxBig;

	unsigned int i_n_Time;
	unsigned int i_n_Tick;

} SM_WaterDropsRain_Params;


typedef struct SM_MidGameBangup_Params
{
	char e_s_Level[64];
	int e_n_Section;
}SM_MidGameBangup_Params;


typedef struct SM_CirclesTutorial_Params
{
	float e_n_Frame;
	char e_s_DinoName[32];

} SM_CirclesTutorial_Params;


typedef struct SM_LvlFailMsg_Params
{
	char e_s_DinoName[32];

} SM_LvlFailMsg_Params;


typedef struct SM_GunsEnable_Params
{
	int e_cbg_on;
	int e_cbg_off;
	int e_cbg_holdFire;
} SM_GunsEnable_Params;


#pragma pack()

#endif // GAMESTATEMACHINES_DATATYPES_H


