#ifndef PTRDON_H_
#define PTRDON_H_


#include "game.h"
#include "grp_path.h"
#include "ptrdon_global.h"


extern const uns32 gPtrdonMinFlockSwitchTime;

extern char* gPtrdonBossTexture;


typedef enum PtrdonStateID
{
	PTRDON_IDLE_HIDDEN,
	// wait for launch delay plus seek delay
	PTRDON_IDLE,
	PTRDON_IDLE_TO_FLY,

	// fly straight forward from initial position, or from the end of the path
	// if there is one.
	PTRDON_FLY_FWD,

	// fly toward spline position
	PTRDON_SEEK_PATH,

	// seek target but don't do an attack, just fly past
	PTRDON_PASS_TGT,

	// fly toward a new position in the attack queue
	PTRDON_ATK_WAIT,

	// fly toward target
	PTRDON_ATK_SEEK,
	PTRDON_ATK_HOLD,
	PTRDON_ATK,
	PTRDON_ATK_FINISH,

	PTRDON_ATK_GRAB_SEEK,
	PTRDON_ATK_GRAB_START,
	PTRDON_ATK_GRAB_IDLE,
	PTRDON_ATK_GRAB,

	// "mount" and attack target
	PTRDON_MOUNT_FLY,
	PTRDON_MOUNT_ATK,
	PTRDON_MOUNT_TOSS,
	PTRDON_MOUNT_DEAD,

	PTRDON_ATK_SWOOP,
	PTRDON_ATK_SWOOP_FINISH,

	PTRDON_FLEE,

	PTRDON_DEAD,

	PTRDON_DONE,

	NUM_PTRDON_STATES,

} PtrdonStateID;


typedef enum PtrdonAnimTrack
{
	PTRDON_TRACK_IDLE,
	PTRDON_TRACK_FLY,

	PTRDON_TRACK_ATK_READY,
	PTRDON_TRACK_ATK,

	PTRDON_TRACK_LOOK_X,
	PTRDON_TRACK_LOOK_Y,

	PTRDON_TRACK_HIT,
	PTRDON_TRACK_FLEE,
	PTRDON_TRACK_DEATH,

	NUM_PTRDON_ANIM_TRACKS,

} PtrdonAnimTrack;


typedef enum PtrdonAnimChan
{
	PTRDON_CHAN_DFLT,
	PTRDON_CHAN_ATK,
	PTRDON_CHAN_HIT,
	PTRDON_CHAN_LOOK,

} PtrdonAnimChan;


typedef enum PtrdonAnimName
{
	PTRDON_ANIM_NAME_IDLE_FLAP1,
	PTRDON_ANIM_NAME_IDLE_FLAP2,
	PTRDON_ANIM_NAME_IDLE1,
	PTRDON_ANIM_NAME_IDLE2,
	PTRDON_ANIM_NAME_IDLE3,

	// for pteras that start out facing up
	PTRDON_ANIM_NAME_IDLE_TO_FLY1,
	// facing down
	PTRDON_ANIM_NAME_IDLE_TO_FLY2,

	PTRDON_ANIM_NAME_GLIDE,
	PTRDON_ANIM_NAME_FLY_ALT1,
	PTRDON_ANIM_NAME_FLY_ALT2,
	PTRDON_ANIM_NAME_FLY_ALT3,
	
	// flap wings (looped)
	PTRDON_ANIM_NAME_FLAP,

	// before attack
	PTRDON_ANIM_NAME_ATK_READY,
	// attack variations
	PTRDON_ANIM_NAME_ATK_BITE,
	PTRDON_ANIM_NAME_ATK_HOVER1,
	PTRDON_ANIM_NAME_ATK_HOVER2,
	PTRDON_ANIM_NAME_ATK_GRAB1,
	PTRDON_ANIM_NAME_ATK_GRAB2,
	PTRDON_ANIM_NAME_ATK_GRAB3,
	PTRDON_ANIM_NAME_ATK_GRAB_RIGHT,
	PTRDON_ANIM_NAME_ATK_GRAB_LEFT,
	PTRDON_ANIM_NAME_ATK_GRAB_RCLAW,
	PTRDON_ANIM_NAME_ATK_GRAB_LCLAW,
//	PTRDON_ANIM_NAME_ATK_SWOOP1,
	PTRDON_ANIM_NAME_ATK_SWOOP2,

	PTRDON_ANIM_NAME_ATK_GRAB1_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB2_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB3_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB_RIGHT_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB_LEFT_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB_RCLAW_IDLE,
	PTRDON_ANIM_NAME_ATK_GRAB_LCLAW_IDLE,

	PTRDON_ANIM_NAME_ATK_SWOOP1_HIT,
	PTRDON_ANIM_NAME_ATK_SWOOP2_HIT,

	PTRDON_ANIM_NAME_MOUNT_FLY1,
	PTRDON_ANIM_NAME_MOUNT_FLY2,
	PTRDON_ANIM_NAME_MOUNT_FLY3,
	PTRDON_ANIM_NAME_MOUNT_FLY4,
	PTRDON_ANIM_NAME_MOUNT_FLY5,
	
	PTRDON_ANIM_NAME_MOUNT_ATK1,
	PTRDON_ANIM_NAME_MOUNT_ATK2,
	PTRDON_ANIM_NAME_MOUNT_ATK3,
	PTRDON_ANIM_NAME_MOUNT_ATK4,
	PTRDON_ANIM_NAME_MOUNT_ATK5,
	
	PTRDON_ANIM_NAME_MOUNT_DEATH1,
	PTRDON_ANIM_NAME_MOUNT_DEATH2,
	PTRDON_ANIM_NAME_MOUNT_DEATH3,
	PTRDON_ANIM_NAME_MOUNT_DEATH4,
	PTRDON_ANIM_NAME_MOUNT_DEATH5,
	
	PTRDON_ANIM_NAME_MOUNT_TOSS1,
	PTRDON_ANIM_NAME_MOUNT_TOSS2,

	PTRDON_ANIM_NAME_HIT_FLY_UP,
	PTRDON_ANIM_NAME_HIT_FLY_DN,
	PTRDON_ANIM_NAME_HIT_FLY_LT,
	PTRDON_ANIM_NAME_HIT_FLY_RT,
	PTRDON_ANIM_NAME_HIT_FLY_UP_LT,
	PTRDON_ANIM_NAME_HIT_FLY_UP_RT,
	PTRDON_ANIM_NAME_HIT_FLY_DN_LT,
	PTRDON_ANIM_NAME_HIT_FLY_DN_RT,

	PTRDON_ANIM_NAME_LOOK_UP_RT,	// +x
	PTRDON_ANIM_NAME_LOOK_DN_LT,	// -x
	PTRDON_ANIM_NAME_LOOK_UP_LT,	// +y
	PTRDON_ANIM_NAME_LOOK_DN_RT,	// -y

	PTRDON_ANIM_NAME_FLEE,
	PTRDON_ANIM_NAME_DEATH_CTR,
	PTRDON_ANIM_NAME_DEATH_LEFT,
	PTRDON_ANIM_NAME_DEATH_RIGHT,

	NUM_PTRDON_ANIM_NAMES,
	FIRST_PTRDON_ATK_ANIM_NAME = PTRDON_ANIM_NAME_ATK_BITE,
	NUM_PTRDON_ATK_ANIMS =
		PTRDON_ANIM_NAME_ATK_SWOOP2 - PTRDON_ANIM_NAME_ATK_BITE + 1,
	NUM_PTRDON_GRAB_ANIMS =
		PTRDON_ANIM_NAME_ATK_GRAB_LCLAW - PTRDON_ANIM_NAME_ATK_GRAB1 + 1,
	NUM_PTRDON_MOUNT_ANIMS = PTRDON_ANIM_NAME_MOUNT_FLY5 - PTRDON_ANIM_NAME_MOUNT_FLY1 + 1,

} PtrdonAnimName;


extern char* gPtrdonAnimNames[NUM_PTRDON_ANIM_NAMES];


typedef enum PtrdonAtkType
{
	PTRDON_ATK_TYPE_GRAB_BITE,
	PTRDON_ATK_TYPE_GRAB_CLAW,
	PTRDON_ATK_TYPE_SWOOP_BITE,
	PTRDON_ATK_TYPE_HOVER_BITE,
	PTRDON_ATK_TYPE_HOVER_CLAW,
	PTRDON_ATK_TYPE_MOUNT_BITE,
	PTRDON_ATK_TYPE_MOUNT_TOSS,

	PTRDON_ATK_TYPE_ANY,

} PtrdonAtkType;


// directions are from the camera's perspective
typedef enum PtrdonSwoopDir
{
	// left to right
	PTRDON_SWOOP_RIGHT,
	// right to left
	PTRDON_SWOOP_LEFT,

} PtrdonSwoopDir;


typedef enum PtrdonAtkSide
{
	PTRDON_ATK_SIDE_ANY,
	// right and center
	PTRDON_ATK_SIDE_RIGHT,
	// left and center
	PTRDON_ATK_SIDE_LEFT,

} PtrdonAtkSide;


typedef struct PtrdonData
{
	uns32 state;
	uns32 lastState;
	uns32 stateStartTime;

	uns32 lastUpdateTime;

	int endFrame;

	// animation tracks
	uns32 animTracks[NUM_PTRDON_ANIM_TRACKS];

	uns32 flyAnimName;
	uns32 atkAnimIdx;
	uns32 mountAnimIdx;
	PtrdonMountSlot mountSlot;

	PtrdonGrabSlot grabSlot;

	uns32 atkSlot;

	float flyAnimSpeed;

	float atkHoldTime;
	
	SteeringAgent steering;
	float pursueWt;
	float gravWt;

	float gravity;

	int isDead;

	// true if ptrdon has been seen by the player in waiting formation
	int waitStarted;
	int mountShakeTgt;
	int dofEnabled;

	// take less time for the hover claw attack 
	int fastHover;

	int flockSwitchDelayOffset;
	// screen-space offset from flock position
	crd flockOffset[V3_SIZE];
	float flockOffsetVelY;

	// how many times a dino can hurt the player while grabbing
	uns32 maxGrabAtks;
	// count how many times dino has damaged player while grabbing
	uns32 grabAtkCount;

	// number of attack animation loops for mount attack pteras. 0 is infinite
	uns32 maxMountLoops;

	// if false, attack immediately from initial position
	int attackAtLaunch;
	int attack;
	int quickAttack;

	PtrdonAtkType atkType;
	PtrdonAtkSide atkSide;

	float maxSpeed;
	float maxSteerForce;

	uns32 tgtOID;
	// defaults to cam obj
	Obj* tgt;

	// offset from target (in target space)
	crd tgtOffset[V3_SIZE];

	crd tgtPos[V3_SIZE];
	crd tgtVel[V3_SIZE];
	// normalized vel
	crd tgtVelDir[V3_SIZE];
	crd tgtRot[V3_SIZE];
	crd tgtRight[V3_SIZE];
	crd tgtUp[V3_SIZE];
	crd tgtFwd[V3_SIZE];

	float tgtSpeed;

	// for converting velocity between the world's reference frame and the target's
	crd velWorld[V3_SIZE];

	// save dino velocity from last frame
	crd velPrev[V3_SIZE];

	// point currently pursuing
	crd seekPos[V3_SIZE];
	float distToSeek;

	float startDist;
	
	crd atkFinalScreenPos[V3_SIZE];

	// local transform
	ObjTransform* otf;

	ObjCam* cam;

	ObjFunc* destructor;

	// initial rotation
	crd heading0[V3_SIZE];
	float rollAng0;

	crd heading[V3_SIZE];
	crd axis[V3_SIZE];
	float axisAng;
	float rollAng;
	float xAng;
	float deathAngVel[V3_SIZE];

	int isFrozen;

	float grabRoll;

	uns32 lastHitTime;
	uns32 lastBounceTime;

	uns32 hitReactCnt;

	float hitBlendOutFrames;

	// accumulate force direction for calculating roll angle
	crd forceDir[V3_SIZE];

	// index in global attack wait queue. -1 if not waiting
	int atkWaitIdx;

	PtrdonSwoopDir swoopDir;

	// initial position for flying forward
	crd flyFwdPos[V3_SIZE];
	// heading
	crd flyFwdDir[V3_SIZE];

	crd atkStartTgt[V3_SIZE];

	int didGrabAtkDmg;

	// true after fly2mount ptera has landed on the target
	int mountHasLanded;

	crd lookDir[V3_SIZE];
	// save previous look dir (after transform)
	crd lastLookDir[V3_SIZE];

	crd screenGrabPos[V2_SIZE];

	GrpPathData path;

	// explosion handle
	uns32 expId;

	// sound ids
	uns32 snd_screech_id;

	uns32 texture;

} PtrdonData;


void Ptrdon_Init(SM_Ptrdon_Params* params);
void Ptrdon_Update(SM_Ptrdon_Params* params);
// call destructor manually
void Ptrdon_Destroy(SM_Ptrdon_Params* params);

void Ptrdon_OnShot(SM_Ptrdon_Params* params, ObjColAct* cola);

void Ptrdon_SetState(SM_Ptrdon_Params* params, uns32 newState);

void Ptrdon_StartAttack(SM_Ptrdon_Params* params);
void Ptrdon_StartQuickAttack(SM_Ptrdon_Params* params);

// return true if cam is in a state that the Ptrdon can attack it from waiting formation
int Ptrdon_CanAttack(SM_Ptrdon_Params* params);
int Ptrdon_TgtFacingForward(SM_Ptrdon_Params* params);
int Ptrdon_IsOnScreen(SM_Ptrdon_Params* params);


#endif