#ifndef PTRDON_GLOBAL_H_
#define PTRDON_GLOBAL_H_


#include "game.h"


#define PTRDON_MAX_ATTACKERS	64

#define PTRDON_MAX_FLY_ANIMS		10
#define PTRDON_MAX_IDLE_STILL_ANIMS	5
#define PTRDON_MAX_IDLE_FLAP_ANIMS	5
#define PTRDON_MAX_IDLE_ANIMS		PTRDON_MAX_IDLE_STILL_ANIMS+PTRDON_MAX_IDLE_FLAP_ANIMS


typedef enum PtrdonGrabSlot
{
	// individual slots
	PTRDON_GRAB_RIGHT,
	PTRDON_GRAB_LEFT,

	NUM_PTRDON_GRAB_SLOTS,

	// covers multiple slots at once
	PTRDON_GRAB_CENTER,

} PtrdonGrabSlot;


typedef enum PtrdonMountSlot
{
	PTRDON_MOUNT_SLOT1,
	PTRDON_MOUNT_SLOT2,
	PTRDON_MOUNT_SLOT3,
	PTRDON_MOUNT_SLOT4,
	PTRDON_MOUNT_SLOT5,

	NUM_PTRDON_MOUNT_SLOTS,

	PTRDON_MOUNT_SLOT_RAND,

} PtrdonMountSlot;


// mount data for each target
typedef struct PtrdonMountData
{
	uns32 tgtOid;

	// total ptrdons currently reserving mount slot on this target
	uns32 numMounting;
	// total for each mount slot
	uns32 slotCounts[NUM_PTRDON_MOUNT_SLOTS];

} PtrdonMountData;


// data shared for all active pteranodons
typedef struct PtrdonGlobalData
{
	// is initialized
	int isInitd;

	// keep track of dinos waiting to start attack sequence
	uns32 numAtkWaiting;
	SM_Ptrdon_Params* atkWaiting[PTRDON_MAX_ATTACKERS];

	uns32 lastAttackTime;
	// minimum time (ms) between attacks
	uns32 attackPeriod;
	// max dinos in attack sequence at one time
	int maxAttackers;
	// number of dinos to send in one attack
	int attackSize;
	// total dinos currently in attack sequence
	int numAttacking;

	// number of ptrdons currently reserving a grab slot
	uns32 grabCount[NUM_PTRDON_GRAB_SLOTS];
	uns32 numFreeGrabSlots;

	List mountDataList;

	int lastAtkName;

	// number of pteras using depth-of-field blur.
	int dofCount;

	// damage each bullet does to a ptera when the maximum number of players are active
	float minBulletDmg;

	float grabHealthMult;

	// determines whether camera is looking forward or backward. pteranodons
	// shouldn't attack when it's facing forward.
	float atkDotThreshold;

	crd atkWaitScrnPos[V3_SIZE];
	crd lastAtkWaitScrnPos[V3_SIZE];
	uns32 waitPosSwitchTime;

	float shotAccel;

	int drawDebug;

	int gndPosIsValid;
	crd gndPos[3];

	ObjFunc* updateFunc;

	// instanced animations (OIDs of OSRs)
	uns32 idleSkelOIDs[PTRDON_MAX_IDLE_STILL_ANIMS];
	uns32 nextIdleSkelIdx;
	uns32 idleFlapSkelOIDs[PTRDON_MAX_IDLE_FLAP_ANIMS];
	uns32 nextIdleFlapSkelIdx;
	uns32 flySkelOIDs[PTRDON_MAX_FLY_ANIMS];
	uns32 nextFlySkelIdx;

	// count number of times an idle animation has been used
	uns32 idleAnimCount;

	uns32 idleOverrideCount;
	uns32 flyOverrideCount;

	// max number of waiting pteras near the camera
	uns32 maxWaitingNear;

	uns32 totalAttacks;

	uns32 nextAtkSlot;

	uns32 lastDeathSFXTime;

	//uns32 wingFlapTimer;
	//uns32 wingFlapSFXLastTick;
	//uns32 windFlapSFXCnt;

} PtrdonGlobalData;


extern PtrdonGlobalData gPtrdonGlobalData;


// create a new pteranodon object with an optional group (pass null for no group)
SM_Ptrdon_Params* CreatePtrdon(Res* res, SM_PtrdonGrp_Params* gParams);


void PtrdonGlobal_StartUp();
void PtrdonGlobal_ShutDown();
void PtrdonGlobal_Reset();

void PtrdonGlobal_AddAtk(SM_Ptrdon_Params* pParams);
void PtrdonGlobal_RemoveAtk(SM_Ptrdon_Params* pParams);
void PtrdonGlobal_AddGrabAtk(SM_Ptrdon_Params* pParams, PtrdonGrabSlot slot);
void PtrdonGlobal_RemoveGrabAtk(SM_Ptrdon_Params* pParams, PtrdonGrabSlot slot);
void PtrdonGlobal_AddMountAtk(SM_Ptrdon_Params* pParams, PtrdonMountSlot slot);
void PtrdonGlobal_RemoveMountAtk(SM_Ptrdon_Params* pParams, PtrdonMountSlot slot);
PtrdonMountSlot PtrdonGlobal_AddMountAtkRand(SM_Ptrdon_Params* pParams);

// add/remove a dino from the attack wait queue
void PtrdonGlobal_AddAtkWait(SM_Ptrdon_Params* pParams);
void PtrdonGlobal_RemoveAtkWait(SM_Ptrdon_Params* pParams);

// for counting pteras enabling depth of field effect
void PtrdonGlobal_AddDoF(SM_Ptrdon_Params* pParams);
void PtrdonGlobal_RemoveDoF(SM_Ptrdon_Params* pParams);

void PtrdonGlobal_RandAtkWaitPos();

ObjSkelRes* PtrdonGlobal_GetNextFlySkel();
ObjSkelRes* PtrdonGlobal_GetNextIdleSkel();

// get attack slot index [0,3].
// order is: bottom left, bottom right, top left, top right.
uns32 PtrdonGlobal_GetNextAtkSlot();
int PtrdonGlobal_IsAtkSlotLeft(uns32 atkSlot);
int PtrdonGlobal_IsAtkSlotBottom(uns32 atkSlot);


#endif