#ifndef WASP_H_
#define WASP_H_


#include "game.h"
#include "grp_path.h"


typedef enum WaspState
{
	// wait for launch; not on a swarm spline
	WASP_IDLE,

	// stay at the start of a swarm spline
	WASP_SPLINE_IDLE,
	// seek a camera-space spline animation
	WASP_SPLINE_SEEK,

	WASP_PURSUE,
	WASP_PURSUE_OFFSET,
	WASP_HOLD,
	WASP_GRAB_START,
	WASP_GRAB_IDLE,
	WASP_GRAB_ATK,
	WASP_GRAB_ATK_DONE,

	// fly out of view after attacking
	WASP_FLEE,
	// scatter from the swarm
	WASP_FLEE_SCATTER,

	WASP_DEAD,

	WASP_DONE,

	NUM_WASP_STATES,

} WaspState;


typedef enum WaspAnim
{
	WASP_ANIM_FLY,
	WASP_ANIM_ATK1,
	WASP_ANIM_ATK2,
	WASP_ANIM_ATK3,
	WASP_ANIM_GRAB_IDLE,
	WASP_ANIM_GRAB_ATK,
	WASP_ANIM_DEATH,

	NUM_WASP_ANIMS,

} WaspAnim;


typedef struct WaspData
{
	uns32 state;
	uns32 stateStartTime;

	uns32 lastUpdateTime;

	// animation tracks
	uns32 animTracks[NUM_WASP_ANIMS];
	
	SteeringAgent agent;
	float pursueWt;
	float arriveWt;
	float gravWt;
	float wanderWt;

	float gravity;

	float wanderRad;
	float wanderDist;
	float wanderJitter;

	int isDead;

	crd camPos[V3_SIZE];
	crd camVel[V3_SIZE];
	// normalized vel
	crd camVelDir[V3_SIZE];
	crd camRot[V3_SIZE];
	crd camRight[V3_SIZE];
	crd camUp[V3_SIZE];
	crd camFwd[V3_SIZE];

	float camSpeed;

	// add to speed based on relative cam speed
	float speedAdjust;
	crd velWorld[V3_SIZE];

	uns32 deathTime;

	crd deathAng;
	crd deathAngSpeed;
	crd deathAxis[3];

	// wasp's destination
	crd tgtPos[V3_SIZE];
	float distToTgt;
	// point wasp is currently pursuing
	crd seekPos[V3_SIZE];
	float distToSeek;
	
	// distance in front of camera to hold before attack
	float holdDist;
	// distance from target when transitioning to arrive state
	float arriveDist;
	// total seconds to spend in current state
	float stateDur;
	
	// local transform
	ObjTransform* otf;

	ObjCam* cam;

	ObjFunc* destructor;

	// initial heading
	crd heading0[V3_SIZE];

	crd heading[V3_SIZE];
	crd axis[V3_SIZE];
	float axisAng;
	float rollAng;

	float grabRollAng;

	// true if has screen slot reserved
	int hasScreenSlot;
	uns32 screenSlotX;
	uns32 screenSlotY;
	
	// camera-space hold position
	crd holdOffsetCam[V3_SIZE];
	// camera-space wasp position 
	crd grabStartCam[V3_SIZE];
	// camera-space wasp position when it died
	crd deathPosCam[V3_SIZE];

	// path when in a swarm
	GrpPathData path;
	// camera-space position of wasp when it leaves the swarm
	crd swarmEndPos[V3_SIZE];
	uns32 swarmFleeTime;

	// explosion handle
	uns32 expId;

} WaspData;


void Wasp_Init(SM_Wasp_Params* params);
void Wasp_Update(SM_Wasp_Params* params);
// call destructor manually
void Wasp_Destroy(SM_Wasp_Params* params);

void Wasp_OnShot(SM_Wasp_Params* params, ObjColAct* cola);

void Wasp_SetState(SM_Wasp_Params* params, uns32 newState);

// zero heading and roll
void Wasp_RotZero(SM_Wasp_Params* params);


#endif