#ifndef WASP_SWARM_H_
#define WASP_SWARM_H_


#include "game.h"
#include "wasp.h"
#include "dino_group.h"


typedef enum WaspSwarmState
{
	// idle until triggered
	WASP_SWARM_WAIT_FOR_TRIG,
	WASP_SWARM_SEEK_SPLINE,
	WASP_SWARM_FLEE,

	WASP_SWARM_DONE,

	NUM_WASP_SWARM_STATES,

} WaspSwarmState;


// data for one swarm object
typedef struct WaspSwarmData
{
	uns32 state;

	uns32 lastUpdateTime;

	// wasps to spawn initially
	uns32 numInitWasps;
	// total wasps spawned
	uns32 numWaspsSpawned;
	uns32 numWaspsKilled;
	uns32 numWaspsDone;

	// number of times a wasp's swarm offset has been switched
	uns32 numWaspOffsetSwitches;

	// wasps currently following the swarm
	SM_Wasp_Params* swarmWasps[DINO_GROUP_MAX_OBJS];
	uns32 numSwarmWasps;
	// target size for number of swarm wasps
	uns32 swarmTgtSize;

	uns32 initTime;
	uns32 launchTime;

	// move the spline into cam coords if true; use world coords if false.
	int useCamCoords;

	// true if some remaining wasps should attack after the spline ends
	int atkAfterSpline;

	// 1 to face camera while seeking spline. 0 to face steer direction
	int faceCam;

	int useRelVel;

	// true if wasps should periodically leave path before it ends.
	int atkFromPath;
	// time (seconds) after path starts to start sending wasps to attack the
	// player.
	float atkStartDelay;
	// time (seconds) between each group of wasps attacking player.
	float atkPeriod;
	int minWaspsPerAtk;
	int maxWaspsPerAtk;
	// number of wasps ready to break from the path to attack player
	uns32 numAtkWaspsReady;
	// number of attack groups sent so far
	uns32 numAtks;

	uns32 switchHoldCounter;

	// percent of dead wasps to respawn back in the swarm
	float respawnPercent;
	float respawnCounter;

	ObjCam* cam;
	crd camPos[V3_SIZE];

	ObjFunc* destructor;

	// ResDataKan
	Res* splineRes;

	crd pos[3];

	int gndPosIsValid;
	crd gndPos[3];

	// swarm sound stuff
	int soundIsStarted;
	crd soundPos[3];
	float soundCnt;

} WaspSwarmData;


void WaspSwarm_Create(SM_WaspSwarm_Params* params);
void WaspSwarm_Update(SM_WaspSwarm_Params* params);
void WaspSwarm_Destroy(SM_WaspSwarm_Params* params);

// count one wasp in this swarm as dead
void WaspSwarm_CountDead(SM_WaspSwarm_Params* params);
void WaspSwarm_CountDone(SM_WaspSwarm_Params* params);

void WaspSwarm_AddAttackWasp
	(SM_WaspSwarm_Params* params, SM_Wasp_Params* wParams);

// create a new wasp and add it to the swarm
SM_Wasp_Params* WaspSwarm_SpawnWasp(SM_WaspSwarm_Params* params);

// respawn this wasp in the swarm, or set it to DONE
void WaspSwarm_TryRespawnWasp
	(SM_WaspSwarm_Params* params, SM_Wasp_Params* wParams);

void WaspSwarm_AddWasp
	(SM_WaspSwarm_Params* params, SM_Wasp_Params* wParams);
void WaspSwarm_RemoveWasp
	(SM_WaspSwarm_Params* params, SM_Wasp_Params* wParams);

void WaspSwarm_UpdateSndPos( WaspSwarmData* sd, ObjVel* ov );

void WaspSwarm_DrawDebug(SM_WaspSwarm_Params* params);


#endif