#ifndef CAM_H
#define CAM_H
// cam.h
// Copyright 2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 19 Oct 04

enum {
	CAM_GMOTION_NONE,
	CAM_GMOTION_GUN0_INPUT,
	CAM_GMOTION_GUN1_INPUT,
	CAM_GMOTION_GUN0_GUN1_INPUT,
	CAM_GMOTION_AUTO,
	CAM_GMOTION_AUTO_SCL // auto with scaling
};

extern int gLockCam;

extern void CamSetMinNearClip( float minDist );

extern void CamBuildLookM3(crd *dstm,ObjCam *cam);
extern void CamBuildVM(crd *dstm,ObjCam *cam);
extern void CamBuildVM2(crd *dstm,crd *trans, crd *rot);
extern void CamInvM3(crd *dstm,ObjCam *cam);
void CamBuildReflectionMatrices(ObjCam *cam, float *trans, float *rot );

float CamGetVerticalAngleOfView(ObjCam* cam); // in degrees

void UpdateCam( ObjCam *o );
int ObjCamResStart(ObjCam *cam,void *vr,uns flags, float speed );
void camPersp(double camx, double camy, double lens, double fovy, double aspect,double znear,double zfar, float *m4out );
void camOrtho(float left, float right, float bottom, float top, float nearv, float farv, float *m4out );

EXTERN_C int CamUnProject(int cam_num, float *in, float *out );   // takes point from camera clip space (0,1) to world space
EXTERN_C int CamProject(int cam_num, float *in, float *out );  // takes point from world space to camera clip space

EXTERN_C int ObjCamUnProject(ObjCam *cam, float *in, float *out );   // takes point from camera clip space (0,1) to world space
EXTERN_C int ObjCamUnProjectVM(crd *iViewVM, crd *iProjM4, float *in, float *out );
EXTERN_C int ObjProjToCamVM(crd *iProjM4, float *in, float *out ); // takes point from camera clip space (0,1) to camera space
EXTERN_C int ObjCamProject(ObjCam *cam, float *in, float *out );  // takes point from world space to camera clip space
EXTERN_C int ObjCamProjectM4(ObjCam *cam, float *proj, float *in, float *out );


int CamInit();
int CamLoop();										// Main loop of the camera system
int CamFinal();
void SetVMFromControl();
void GetCamControlVM( float *vm, float *trans, float *rot);
int  camPos(ObjCam *cam);
int  camLookAt(ObjCam *cam);
void CamStartLookAt(ObjCam * cam, Obj* target, char* bone, float slerpTime, float smoothing);
void CamStopLookAt(ObjCam * cam);
void ObjCamBuildMatrices( ObjCam *cam, int screen );
void CamSetFOV(ObjCam* cam, float fov);

int NextCam(int skip);

void CamGmotionSetState(int state);
void CamGmotionReset();
void CamGmotionSetStillScale( float x, float y, float z ); // scale when camera is still
void CamGmotionSetMovingScale( float x, float y, float z ); // scale when camera is moving
void CamGmotionSetStillSpeed( float x, float y, float z );
void CamGmotionSetMovingSpeed( float x, float y, float z );
void CamGmotionSetGlobalScale( float scale );
void CamGmotionSetTarget( float* target_pos );
// target rotation in XYZ Euler angles
void CamGmotionSetTargetRot( float* target_rot );

void CamGmotionGetTarget( float* target_pos );
void CamGmotionGetTargetRot( float* target_rot );


// camera shakes
#define CAM_SHAKE_SHAKE			0x00000001
#define CAM_SHAKE_SHAKE_ROT		0x00000002
#define CAM_SHAKE_SHAKE_TRANS	0x00000004
#define CAM_SHAKE_SPRING		0x00000008
#define CAM_SHAKE_RAW			0x00000010

void CamShakeReset();

// set global cam shake parameters (randomly move/rotate camera).
// switches target position/rotation every "period" milliseconds.
void CamShake(float intensity, float seconds, uns32 period, uns32 flags);

// set global spring cam shake parameters
void CamShakeSpring( crd* maxDisp, crd* springForce, crd* springSmooth, crd* dir, int* maxOscillations );

// enable cam shake for this cam
void CamShakeOn(int camNum);
void CamShakeOff(int camNum);

int CamShakeIsActive(int camNum);
int ObjCamShakeIsActive(ObjCam* ocam);

#endif // ndef CAM_H
