// core_statemachines.h
// Play Mechanix - Video Game System
// Copyright (c) 2008 Play Mechanix Inc. All Rights Reserved
//
// init routintes and statemachines for programmable statemachine engine 
#ifndef ENEMYSTATEMACHINE_H
#define ENEMYSTATEMACHINE_H


#include "g5.h"


//////////////////////////////////////////////////////////////////////////
// explosion reactions
//////////////////////////////////////////////////////////////////////////

#define MAX_EXPLOSIONS_USERS	1024

enum EXPLOSION_TYPE
{
	EXPLOSION_DESTRUCT,
	EXPLOSION_GRENADE,
	EXPLOSION_KILL_SCREEN_BUGS,
};

typedef struct ExplosionData
{
	crd pos[3];
	// player that triggered the explosion. -1 if none
	int pnum;
	int type;
	float minDist;
	float maxDist;
	float rolloff;
	float force;
} ExplosionData;

typedef void (*Explosion_Callback)( ExplosionData* exp_data, void* data );

void Explosion_Init();
void Explosion_Final();
void Explosion_Loop();
void Explosion_Reset();
uns32 Explosion_Register( Explosion_Callback callback, uns32 oid, void* data );
void Explosion_UnRegister( uns32 id );
void Explosion_Explode( crd* pos, int pnum, int type, float minDist, float maxDist, float rolloff, float force );
float Explosion_ComputeAttenuation( float dist, ExplosionData* exp_data );
// linear
float Explosion_ComputeLinAttenuation( float dist, ExplosionData* exp_data );

//////////////////////////////////////////////////////////////////////////
// destructables
//////////////////////////////////////////////////////////////////////////

// destructible objs state machine
enum {
	SM_DESTRUCT_INIT = NUM_SM_GPSTATES,
	SM_DESTRUCT_INIT_FLYERS,
	SM_DESTRUCT_SHOT,
	SM_DESTRUCT_UPDATE,
	NUM_SM_DESTRUCT_STATES,

};

extern PSM_StateMachine gDestructStateMachine;
extern PSM_State gDestructStateMachineStates[NUM_SM_DESTRUCT_STATES];


enum {
	FLYERTYPE_NORMAL,
	FLYERTYPE_GLASS,
};
enum {
	FLYERSTATE_WAIT,
	FLYERSTATE_FALL
};
// each flyer gets one of these on it's ov controller
typedef struct DestructFlyerData
{
	uns32 update_cnt;
	float start_time;
	float lifetime;
	uns32 state;
	uns32 type;
	ObjDispRes* odr_flyer;
	ObjDispRes* odr_prime;
	crd launch_vel[3];
	int override_launch_vel;

	SM_Destruct_Params *dparams;
} DestructFlyerData;

// instructions
int MakeDestructActor(Res* res, char* name, ObjDispRes** out_odr);
void Destruct_InitFlyers(ObjDispRes* odr_prime, SM_Destruct_Params* dparams);
int SM_Destruct_Init( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Destruct_FlyersToPrimeVM( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Destruct_Shot( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );
int SM_Destruct_Update( PSM_ExeContext *context, void *statemachine_params, void *state_params, uns32 state_updates, float state_elapsed_time, float *elapsed_time );

// helpers
void Destruct_Explosion_Callback( ExplosionData* exp_data, void* data );
void Destruct_FlyerToWorld( ObjVel* ov, ObjDispRes* odr_flyer, ObjDispRes* odr_prime );
void Destruct_MakeFlyerInitialVelocity( crd* right, crd* up, crd* forward, DestructFlyerData* fd, crd* launch_vel );
int Destruct_Flyers_Update( ObjVel* ov, float deltaTime );
int Destruct_Flyers_Free( ObjVelController* velCtrl );
int Destruct_OECFree( void* data );
ObjExeContext* Destruct_FindDestructProg( ObjDispRes* odr );
SM_DestructSM_Params* Destruct_GetDestructParams( ObjExeContext* destruct_oec );
int Destruct_Trigger( ObjExeContext* destruct_oec, SM_DestructSM_Params* dparams );
ObjGroup* Destruct_GetFlyersGroup( SM_DestructSM_Params* dparams, int stage );
void Destruct_SetFlyersColors( ObjGroup* flyers_group, float* rgba );
void Destruct_SetFlyersLightGroup( ObjGroup* flyers_group, uns8 lg );
void Destruct_SetFlyersGlowIntensity( ObjGroup* flyers_group, float intensity );
DestructFlyerData* Destruct_GetFlyerData( ObjDispRes* flyer_odr );
int Destruct_CountFlyers( ObjGroup* flyers_group );
void Destruct_CheckForBonus( int pnum, SM_DestructSM_Params* dparams );


#endif // CORESTATEMACHINES_H
