// core.h
// Play Mechanix - Video Game System
// Copyright (c) 2008 Play Mechanix Inc. All Rights Reserved

// This is a wrapper to include the various core headers in the appropriate order
// to avoid linking errors.

#ifndef G5_H
#define G5_H

#ifdef USE_ENCRYPTED_FILES
#undef USE_ENCRYPTED_FILES
#endif


#ifndef SYS_PC
	#ifndef NTELEMETRY
		#define NTELEMETRY
	#endif
#endif

// #define NO_TOOL

#include <locale.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <math.h> // sqrt, cos, sin
#include <ctype.h>
#ifdef SYS_PC
#include <float.h>
#define isfinite _finite
#endif

#ifdef SYS_BB
 #include <linux/rtc.h>
 #include <sys/time.h>
 #include <time.h>
 #include <sys/ioctl.h>
 #include <sys/utsname.h>
#endif



// PM libs...
#include <pmrt_utils/pmrt_utils.h>
#include <xmlWrapper/xmlWrapper.h>
//#include <psm_engine2/psm_engine2.h> // this system is now in the g5enegine
#include <pmrt_ssl/pmrt_ssl.h>

#ifdef __cplusplus
extern "C" {
#endif
#include <pmrt_io/io.h>
#ifdef __cplusplus
}
#endif

#include <card.h>
#include <pmocl.h>


#ifndef JAMMA_H
#include <jamma.h>
#endif
#include <theora/theora.h>

#define NTELEMETRY_GPU	1
#include <telemetry.h>
#include "tmxgpu_ogl.h"

#ifdef __cplusplus
extern "C" {
#endif
	extern HTELEMETRY gTelemetry;
	extern struct TmxGpuContext gTelemetryGL;
#ifdef __cplusplus
}
#endif

#include "OVR_CWrapper.h"

// GL
#ifdef SYS_PC
	#include <GL/glew.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

#define USE_GL_GET_ERROR 0
#define USE_RIO2_INPUT 0

#ifdef SYS_BB
	#include <GL/glut.h>
	#include <GL/glext.h>
#else
	#include <GL/freeglut.h>
#endif

#ifdef SYS_BB
#ifndef GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT
	#define GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT 0x8C72
#endif
#ifndef GL_COMPRESSED_LUMINANCE_LATC1_EXT
	#define GL_COMPRESSED_LUMINANCE_LATC1_EXT 0x8C71
#endif
#endif // SYS_BB

#ifndef NO_TOOL
#define SCENE_INFO_LOG tool2_infoLog
#else
#define SCENE_INFO_LOG(...)
#endif


#include "qr_code.h"


// another PM lib...
#include <glcgz2.h>




// sdl
#include <SDL.h>
#include <SDL_mixer.h>

#ifdef NEED_FAR_POINTERS
#endif
// this doesn't seem good, but his is what it took to get jpeglib to include on windows vis studio 2008 EE
#undef FAR
#include <jpeglib.h>


// g5 engine headers...

#ifdef __cplusplus
extern "C" {
#endif

#include "psm_engine.h"
#include "pm.h"
#include "paths.h"
#include "file_access.h"
#include "vec.h"
#include "str.h"
#include "mem.h"
#include "sys.h"
#include "sys_dongle.h"
#include "wibu_dongle.h"
#include "sys_time.h"
#include "fs.h"
#include "workerbee.h"
#include "res.h"
#include "nbload.h"
#include "particles.h"
#include "core_rtdt_datatypes.h"
#include "core_input_enums.h"
#include "core_input.h"
#include "motion_platform.h"
#include "wind.h"
#include "Flyers.h"
#include "fur.h"
#include "car_params.h"
#include "obj.h"
#include "anim.h"
#include "cam.h"
#include "adj.h"
#include "adj_dump.h"
#include "aud.h"
#include "currency.h"
#include "cashbox.h"
#include "sound.h"
#include "profanity_filter.h"


// Hongwen Flash
//#include "flash_player.h"

#include "proc.h"
#include "font.h"
#include "wnd.h"
#include "vid.h"
#include "water.h"
#include "noise_tex.h"
#include "render_target.h"
#include "render_list.h"
#include "render_layers.h"
#include "render_state.h"
#include "render_shadmaps.h"
#include "render_loop.h"
#include "render.h"
#include "render_obj.h"
#include "render_debug.h"
#include "draw.h"
#include "shader_gl.h"
#include "postfx.h"
#include "triggers.h"
#include "trigger_func_core.h"
#include "core_scene.h"
#include "scene_manager.h"
#include "PhysX.h"
#include "core_rtdt.h"
#include "core_statemachines_datatypes.h"
#include "core_statemachines.h"
#include "destruct_statemachine.h"
#include "theorafunc.h"
#include "gcon.h"
#include "mcore.h"
#include "localization.h"
#include "tool2.h"
#include "tool2_scene_edit.h"
#include "tool2_anisfx.h"
#include "tool2_anichan.h"
#include "tool2_wind.h"
#include "tool2_envCollide.h"
#include "tool2_text.h"
#include "game_core.h"
#include "cpmgui.h"
#include "collision.h"
#include "keyboard.h"
#include "menu.h"
#include "diag.h"
#include "reboot.h"
#include "vol_dlg.h"
#include "hitFX.h"
#include "aud_diag.h"
#include "adj_diag.h"
#include "system_info.h"
#include "system_info_diag.h"
#include "system_tests.h"
#include "system_tests_diag.h"
#include "resets_diag.h"
#include "utilities_diag.h"
#include "cardreader.h"
#include "res_jpeg.h"
#include "blower.h"
#include "error_message.h"

#include "workerbee_funcs.h"
#include "pmrt_assert.h"

#include "boot_crc.h"

// and our g5 wide functions...



int g5_Init( void );
int g5_Start(int fullscreen, int width, int height, float aspect_ratio, int screen_type );
int g5_Final();

char *g5_GetVersion();
int g5_GetInitTime();
int g5_GetToolMode();
void g5_SetToolMode(int mode);



#ifdef __cplusplus
}
#endif


#endif

