// HitFX.h
// Play Mechanix - Video Game System
// Copyright (c) 2011 Play Mechanix Inc. All Rights Objerved
//

#ifndef HITFX_H
#define HITFX_H

extern HashTable gBoneActionEffectTable;
extern HashTable gObjBoneActionTemplateTable;


enum {
	BA_TRIGGER_ONSTARTTOUCH	= 0x00000001,
	BA_TRIGGER_ONENDTOUCH	= 0x00000002,
	BA_TRIGGER_WHILETOUCHING = 0x00000004,
};

enum {
	BA_EMITTERORIENTATION_SPLINE,
	BA_EMITTERORIENTATION_BONE,
	BA_EMITTERORIENTATION_GROUND,
	BA_EMITTERORIENTATION_NONE,
};

enum {
	BA_ATTACHMODE_NONE,
	BA_ATTACHMODE_BONE,
};

enum {
	BA_EMIT_AT_GROUND,
	BA_EMIT_AT_TESTPOINT,
};

typedef struct BoneActionEffectEntry
{
	uns32	trigger;				// see BA_TRIGGER_ enum
	List	fxGroups;				// stores char arrays
	//char	fxGroup[PM_NAME_SIZE];
	uns32	emitterOrientationMode; // see BA_EMITTERORIENTATION_ enum
	uns32	attachMode;				// see BA_ATTACHMODE_ enum
	uns32	emitLocation;			// see BA_EMIT_ enum
	float	maxSpeed;				// in units per millisec
	float	minSpeed;				// in units per millisec
} BoneActionEffectEntry;

typedef struct BoneActionEffect
{
	char	terrainType[PM_NAME_SIZE];
	char	animal[PM_NAME_SIZE];
	char	type[PM_NAME_SIZE];
	List	effectEntries;
} BoneActionEffect;

typedef struct BoneActionFXGroupRecord
{
	uns32 oid;
	uns32 attachMode;
	uns32 emitLocation;
} BoneActionFXGroupRecord;

char* getHitFX(char* terrain, char* type, char* gunType);

BoneActionEffect* boneActionEffectGetEffect(char* animal, char* type, char* terrain);
int boneActionPopulateObjBoneActions(char* animal, Obj* odr);

int boneActionAddObjBoneActionToPopulateTable(char* model, ObjBoneAction* oba);
int boneActionEffectLoadObjBoneAction(char* filename, char* animal);
int boneActionEffectEntryPopulateFromFile(char* filename, char* animal, char* type, char* terrain);

int hitFXLoadFile(char* filename);

int boneActionEffectLoadAll();
int hitFXLoadAll();

void boneActionClearAll();
void hitFXClearAll();

int HitFX_Init();
int HitFX_Final();
int HitFX_Reload();

#endif
