// ----------------------------------------------------------------------------
// l2list.h - doubly linked list
// ----------------------------------------------------------------------------
#ifndef L2LIST_H_
#define L2LIST_H_


#include "g5.h"


struct L2List
{
	void* next;
	void* prev;
	void* next2;
	void* prev2;
	void* next3;
	void* prev3;
};


struct L2ListHdr
{
	uns8* mem;
	L2List* free;

	int32 nelem;
	int32 size;
	int32 count;
};


void L2List_InitHdr(L2ListHdr* hdr, int32 nelem, int32 size, uns32 id);
void L2List_ShutdownHdr(L2ListHdr* hdr);
void L2List_InitList(L2List* list);

void* L2List_Alloc(L2ListHdr* hdr);
void L2List_Free(L2ListHdr* hdr, void* litem);

void L2List_Insert(L2List* list, void* litem);
void L2List_InsertAfter(void* litem, void* aitem);
void L2List_InsertTail(L2List* list, void* litem);
void L2List_Pull(void* litem);
void L2List_Swap(void* litem);

// supplemental list routines
void L2List_Insert2(L2List* list, void* litem);
void L2List_Pull2(void* litem);
void L2List_Insert3(L2List* list, void* litem);
void L2List_Pull3(void* litem);

//list handlers
void L2List_Reset(L2ListHdr* free, L2List* active);


//////////////////////////////////////////////////////////////////////////

// use these with C++ objects (that can't just be casted to an L2List)

#define MOVD(dest,src) (*(uns32 *)(&dest))=(uns32)(src)

#define L2_INSERT(list,item)\
{\
	ASSERT(item!=NULL && item->m_prev==NULL, "");\
	if(list) list->m_prev = item;\
	item->m_next = list;\
	list = item;\
	MOVD(item->m_prev, &list);\
}

#define L2_INSERTAFTER(item, after)\
{\
	ASSERT(item!=NULL && item->m_prev==NULL, "");\
	ASSERT(after!=NULL, "");\
	item->m_next = after->m_next;\
	after->m_next = item;\
	item->m_prev = after;\
	if(item->m_next)\
		item->m_next->m_prev = item;\
}

#define L2_PULL(item)\
{\
	ASSERT(item->m_prev!=NULL, "");\
	if((*(uns32 *)item->m_prev)==(uns32)item)		/* we are head of list */\
	{\
		if(item->m_next)\
			item->m_next->m_prev = item->m_prev;\
		(*(uns32 *)item->m_prev) = (uns32)item->m_next;\
		item->m_prev = NULL;\
		item->m_next = NULL;\
	}\
	else\
	{\
		if(item->m_next)\
			item->m_next->m_prev = item->m_prev;\
		item->m_prev->m_next = item->m_next;\
		item->m_prev = NULL;\
		item->m_next = NULL;\
	}\
}


#define L2_INSERT2(list,item)\
{\
	ASSERT(item!=NULL && item->m_prev2==NULL, "");\
	if(list)\
		list->m_prev2 = item;\
	item->m_next2 = list;\
	list = item;\
	MOVD(item->m_prev2, &list);\
}

#define L2_PULL2(item)\
{\
	ASSERT(item->m_prev2!=NULL, "");\
	if((*(uns32 *)item->m_prev2)==(uns32)item)		/* we are head of list */\
	{\
		if(item->m_next2)\
			item->m_next2->m_prev2 = item->m_prev2;\
		(*(uns32 *)item->m_prev2) = (uns32)item->m_next2;\
		item->m_prev2 = NULL;\
		item->m_next2 = NULL;\
	}\
	else\
	{\
		if(item->m_next2)\
			item->m_next2->m_prev2 = item->m_prev2;\
		item->m_prev2->m_next2 = item->m_next2;\
		item->m_prev2 = NULL;\
		item->m_next2 = NULL;\
	}\
}

#define L2_INSERT3(list,item)\
{\
	ASSERT(item!=NULL && item->m_prev3==NULL, "");\
	if(list)\
		list->m_prev3 = item;\
	item->m_next3 = list;\
	list = item;\
	MOVD(item->m_prev3, &list);\
}

#define L2_PULL3(item)\
{\
	ASSERT(item->m_prev3!=NULL, "");\
	if((*(uns32 *)item->m_prev3)==(uns32)item)		/* we are head of list */\
	{\
		if(item->m_next3)\
			item->m_next3->m_prev3 = item->m_prev3;\
		(*(uns32 *)item->m_prev3) = (uns32)item->m_next3;\
		item->m_prev3 = NULL;\
		item->m_next3 = NULL;\
	}\
	else\
	{\
		if(item->m_next3)\
			item->m_next3->m_prev3 = item->m_prev3;\
		item->m_prev3->m_next3 = item->m_next3;\
		item->m_prev3 = NULL;\
		item->m_next3 = NULL;\
	}\
}


#endif
