// nbload.h
// Copyright 2008 Play Mechanix Inc. All Rights Reserved
//

#ifndef NBLOAD_H
#define NBLOAD_H

#define NBLOAD_TIME_LOADS 0
#define NBLOAD_CREATE_RES_LOOKUP_HASHES 0  // building these can speed up the create of skel objects...

#include "g5.h"


enum NBLoadErrCodes
{
	NBLOAD_SUCCESS,
	NBLOAD_FILE_ALREADY_LOADED,
	NBLOAD_INVALID_REQUEST,
	NBLOAD_REQUEST_NOT_COMPLETE,
	NBLOAD_MALLOC_ERR,
	NBLOAD_LINK_ERR,

};


enum NBLoadStatus
{
	NBLOAD_STATUS_BUSY,
	NBLOAD_STATUS_COMPLETED,
	NBLOAD_STATUS_ERROR,
};


typedef struct NBLoadRequest
{
	unsigned int id;
#if(NBLOAD_TIME_LOADS==1)
	uns64 start_time;
#endif

	char filename[128];

	int status;
	int state;
	float progress;
	int error;

	void (*callback)(struct NBLoadRequest *);

	// stuff for keeping state as this
	// file is loaded into memory
#ifdef USE_ENCRYPTED_G3S
	ThreadedCryptoZFileIOHandle tczh;
#else
	ThreadedFileIOHandle tfh;
#endif
	int next_state; // what state to go to when current IO operation completes
	int prev_state; // what state we were in before we went to IO state

	/*
	void *fdata;
	int fsize;

	void *headerdata;
	void *resdata;
	*/

	char headerdata[G_STANDARD_HEADER_SIZE];

	G_LBHead blockhead;
	G_LBTail blocktail;

	int splitbuf;
//	int splitpool;
	int splitfree;
//	int splitmallocd;

	int blocks;

	int size;
	// void *splitbuf;

	uns32 res_mark;

	Res *group;
	N2  *kidlist;

	void *mempool;

	
	void *rhMem;
	void *rc;
	void *mem;

	int vbonum;

	ResLinkState linkstate;

	Res* dispRes;

} NBLoadRequest;


int NBLoadInit();

int NBLoadLoop();

int NBLoadFinish(int wipeout); // this one spins on all requests until they are done, if wipeout is set then callbacks are not issued (safety)


int NewNBLoadRequest( char *filename, uns32 mark, void (*callback)(struct NBLoadRequest *), int *id );

int CheckNBLoadRequestStatusByID( unsigned int id, float *progress );
int CheckNBLoadRequestStatusByFile(char *filename, float *progress );

int FinishNBLoadRequestByID( unsigned int id ); 
int FinishNBLoadRequestByFile(char *filename );

int CloseNBLoadRequest( NBLoadRequest *request );
int CloseNBLoadRequestByID( unsigned int id );
int CloseNBLoadRequestByFile(char *filename );

NBLoadRequest *FindNBLoadRequestByID( unsigned int id );

// NBLoadRequest *FindNBLoadRequestByFile( char *filename );
// NBLoadRequest *FindNBLoadRequestByID( unsigned int id );

#endif

