// particles.h
// Copyright 2008 Play Mechanix Inc. All Rights Reserved
//

#ifndef PARTICLES_H
#define PARTICLES_H

#include "g5.h"

// save all fields in particle files
// or just those that have changed from the dflt (0)
#define PARTICLES_FULL_PARTICLE_SAVE 1


enum {
	PARTICLE_DISPERSION_BOX,
	PARTICLE_DISPERSION_CIRCULAR,
	NUM_PARTICLE_DISPERSION_MODES,

};


enum {
	PARTICLE_EJECT_OVER_TIME,    // eject particles over time (irregardless of speed traveling)
	PARTICLE_EJECT_OVER_DIST,    // eject particles over distance traveled (irregardless of time gone by)
	NUM_PARTICLE_EJECT_MODES,
};


enum {
	PARTICLE_COLOR_BLEND_NONE,       // no color blending at all
	PARTICLE_COLOR_BLEND_PARTICLES,  // individual particles blend their over time
	PARTICLE_COLOR_BLEND_EMITTER,    // color of particles emitted blends over time...
	NUM_PARTICLE_COLOR_BLEND_MODES,
};

enum {
	PARTICLE_BLEND_ALPHA,
	PARTICLE_BLEND_ADD,
	PARTICLE_BLEND_SUBTRACT,
	PARTICLE_BLEND_NONE,
	NUM_PARTICLE_BLEND_MODES,

};

enum {
	PARTICLE_IMGMODE_STATIC,
	PARTICLE_IMGMODE_ANIMATING,
};


enum {
	PARTICLE_VMMODE_APPLY_AT_EMISSION  = 0,  // apply vm to particles as they are emitted (use this to leave a 
	PARTICLE_VMMODE_APPLY_EVERY_FRAME,     // apply vm to particles every frame
	PARTICLE_VMMODE_APPLY_TRANS_EVERY_FRAME,     // apply vm translation (but not rotation) to particles every frame
	NUM_PARTICLE_VMMODES,
};


enum {
	PARTICLE_FORCEVELDIR_NONE = 0,
	PARTICLE_FORCEVELDIR_OUTWARD,
	PARTICLE_FORCEVELDIR_INWARD,
	PARTICLE_FORCEVELDIR_RADIAL,
	PARTICLE_FORCEVELDIR_RADIAL_INWARD,
	NUM_PARTICLE_FORCEVELDIR_MODES,
};


enum {
	PARTICLE_SHADERFX_NONE,
	PARTICLE_SHADERFX_FOG,
	PARTICLE_SHADERFX_GLOW,
	PARTICLE_SHADERFX_BLUR,
	PARTICLE_SHADERFX_NMAP,
	NUM_PARTICLE_SHADERFXS,
};

enum {
	PARTICLE_SORT_TYPE_NORMAL,
	PARTICLE_SORT_TYPE_DECAL,
};


typedef struct ParticleRenderData
{
	float verts[3];
	float colors[4];
	float uvs[2];
} ParticleRenderData;





typedef struct Particle
{
	crd ipos[3]; // initial position
	crd pos[3];
	crd vel[3];
	crd acc[3];
	crd irot[3];

	f32 lifetime;
	f32 age;
	f32 ageAnimOffset;
	f32 ageAccum; // age accumulated waiting for an update (used when particle is not updated due to LOD)

	f32 alphaIn;
	f32 alphaOut;

	f32 size;  // initial size
	f32 sizeVel; // size growth rate
	f32 sizeCur; // current size (this is the one actually used to render particle)

	f32 theta;
	f32 thetaVel;

	uns disp2OID;

	f32 u0;
	f32 v0;

	f32 uvDim[2];
	f32 color[4];
	f32 alpha;

//	List posList;

} Particle;

typedef struct ParticleEmitter
{
	int  lod_status;
	char name[PM_NAME_SIZE];
	char group[PM_NAME_SIZE];
	char res_name[PM_NAME_SIZE];
	char normals_res_name[PM_NAME_SIZE];
	char program_res[PM_NAME_SIZE];

	char shader[PM_NAME_SIZE];
	uns32  link_after_oid; // oid of Obj to link disp2 particles created after
	
	// only used for disp2 particles
	int alpha_blend;
	float sort_pri;

	uns32 texGen;
	uns32 normals_texGen;

	uns32 dispersionMode; // eject particles in a box or in a circle?
	uns32 sortType; // normal or decal sorting?

	crd pos[3];
	crd posVar[3];

	crd basePos[3];

	crd vel[3];
	crd velVar[3];

	crd acc[3];
	crd accVar[3];

	f32 size;
	f32 sizeVar;

	f32 sizeVel;
	f32 sizeVelVar;

	f32 sizeVelEnd;

	f32 xsize;
	f32 ysize;

	f32 lifetime;
	f32 lifetimeVar;

	f32 thetaVel;
	f32 thetaVelVar;

	f32 alphaIn, alphaOut;

	uns32 ejectionMode; // eject particles over time or over distance
	f32 ejectionRate; 
	uns32 age;
	
	f32 scale;
	f32 scaleAttack;
	f32 scaleDecay;

	f32 disp2rvel[3];
	f32 disp2rvelvar[3];

	uns blendColor;     // - do we blend from color to colorb or not?
	f32 color[4];
	f32 colorb[4];		// - used as the end color for sparks

	uns maxParticles;

	uns totalParticles;
	uns numParticles;

	uns looping;

	uns32 lastParticleCreateTime; 
	float distAccum; 

	uns blendmode;      // what blend mode to use when rendering particle
	uns imagemode;        // is it a static img, static 2x2, or animating img?

//	uns animating;		// - does this particle emitter animate?
	uns numFrames;		// - how many frames in the animation?
	uns frameDim[2];	// - number of frames in x/y in the image
	uns frameTimeMS;	// - how many milleseconds to display each frame
	uns killondone;		// - is the particle killed when the animation has played?
	int frames[128];     // - list of frames to choose from...

	f32 uvDim[2];		// hieght/width of images in uv space, used for subdividing static imgs

	uns uv_flipx;		// randomly flip image in x
	uns uv_flipy;		// randomly flip image in y



	uns streakLen;		// - how many 'positions' we remember for a streak
	uns streakCam;		// render layer cam to use for streak calculations

	int32 delayMS;		// - how many milleseconds to wait before activating

	uns type;			// - 0 = standard, 1 = yscale, 2 = streak;
	uns orientation;	// - 0 = billboard, 1 = y axis

	uns shaderFX;
	/*
	uns useFog;
	uns useGlow;
	*/
	uns groundCol;
	uns noFaceCull;

	uns forceveldir;    // always make the vel to move away from/toward the center point...

	uns useCameraIVM;	// - use the camera's inverse matrix to rotate the position
						//   (this is so the xyz offset is relative to the player's view)

	uns randtheta;
	uns randthetavelsign;  // - randomly flip sign on thetavel

	uns lightgroup;

	Res *resource;
	Res *normals_resource;
	uns dlist;

	List particles;

	ParticleRenderData *rd;
	int rd_size;
	int rd_active_count;
	// int active_rd_nodes;

	void *parent;

	int vm_mode;
	crd vm[VM_SIZE], vm_prev[VM_SIZE];

	uns selected;

	// uns attachOID;		// - the OID of the object this emitter is "attached" to

	uns fxgroupOID;		// - OID of the fxgroup object this emitter is a part of (if any)


} ParticleEmitter;

enum {
	PEMITTER_TYPE_STANDARD = 0,
	PEMITTER_TYPE_YSCALE,
	PEMITTER_TYPE_STREAK,
	PEMITTER_TYPE_DISP2,
	PEMITTER_TYPE_NUM,
};

enum {
	PEMITTER_ORIENTATION_BILLBOARD = 0,
	PEMITTER_ORIENTATION_YAXIS,
	PEMITTER_ORIENTATION_YZBILLBOARD,
	PEMITTER_ORIENTATION_SURFACE_NORMAL,
	PEMITTER_ORIENTATION_YZSTREAK,
	PEMITTER_ORIENTATION_2DBILLBOARD,
	PEMITTER_ORIENTATION_NUM,
};

/*
#define M_PETYPE_BBOARD		(1 << PEMITTER_ORIENTATION_BILLBOARD)
#define M_PETYPE_YAXIS		(1 << PEMITTER_ORIENTATION_YAXIS)
#define M_PETYPE_YZBBOARD	(1 << PEMITTER_ORIENTATION_YZBILLBOARD)
#define M_PETYPE_DECAL		(1 << PEMITTER_ORIENTATION_SURFACE_NORMAL)
*/

typedef struct
{
	List partEmitters;		// - list of particle emitters
	List fxGroups;			// - list of effects groups (groups of particle emitters and sound calls)

	HashTable partEmittersHash; // - hash of particle emitters by name, for quick finds (points to nodes in partEmitters list)
	HashTable fxGroupsHash;     // - hash of particle fxgroups by name, for quick finds (points to nodes in fxGroups list)

#ifndef NO_TOOL
	int	gui_id_parent;     // - gui id of the main particles edit window
	int gui_id_window;		// - gui id of the sub window for particles/fxgroup list
	int gui_id_emitterbtn;	// - "restart selected" particle emitters
	int gui_id_emitter_sort;
	int gui_id_fxgroups_sort;
	int gui_id_tabs[2];		// - tab buttons for the edit window (emitters, fxgroups)
	int tab_current;		// - which is our active tab?

	// these hash tables are used by the tool gui to track what emitter/fxgroup groups to show...
	HashTable partEmittersShowGroup;
	HashTable fxGroupsShowGroup;
	int dirtyFxGroups;		// - used to tell the fx group window to refresh
	int dirtyEmitters;		// - used to tell the fx group window to refresh
#endif // !NO_TOOL
} ParticleCollection;


typedef struct {
	char emitter[64];
	char soundname[64];
	int soundvolume;
	int soundloop;
	int soundchanmask;
	int soundpri;
	float soundvolswing;
	float sound_dist_modifier;

	int delay;
	float scale;
	float pos[3];
	float pvar[3];
	float vel[3];
	float vvar[3];
	float acc[3];
	float rvel[3];
	int forceveldir;

	int disabled;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int show_details;
	int gui_id_show_details;
	int gui_id_delay;
	int gui_id_scale;
	int gui_id_pos[3];
	int gui_id_pvar[3];
	int gui_id_vel[3];
	int gui_id_vvar[3];
	int gui_id_acc[3];
	int gui_id_rvel[3];
	int gui_id_forceveldir[NUM_PARTICLE_FORCEVELDIR_MODES];

	int gui_id_sound_volume;
	int gui_id_sound_loop;
	int gui_id_sound_chanmask;
	int gui_id_sound_pri;
	int gui_id_sound_volswing;
	float gui_id_sound_dist_modifier;



#endif // NO_TOOL

} PC_FXGroupList;

typedef struct {
	char name[PM_NAME_SIZE];
	char group[PM_NAME_SIZE];
	List effectList;
	void *pc; // particle collection that fxgroup belongs to...
	int dirty;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int gui_id_fxwindow;
	int gui_id_window;
	int gui_id_name;
	int gui_id_group;
	int gui_id_scale;
#endif // NO_TOOL
} PC_FXGroup;

typedef struct {
	
	char name[PM_NAME_SIZE];
	char trigger[PM_NAME_SIZE];
	char offTrigger[PM_NAME_SIZE];
	ParticleEmitter data;
	int dirty;

#ifndef NO_TOOL
	// BEGIN GUI STUFF
	int selected;
	int gui_id_window;
	int gui_id_editwindow;
	int gui_id_tabs[6];
	int tab_current;
	int gui_id_name;
	int gui_id_group;
	int gui_id_res;
	int gui_id_nres;
	int gui_id_prog_res;

	int gui_id_shader;
	int gui_id_trigger, gui_id_trigger_off;
	int gui_id_lightgroup;
	int gui_id_sortpri;
	int gui_id_selected;
	int gui_id_pos[3];
	int gui_id_posvar[3];

	int gui_id_vel[3];
	int gui_id_velvar[3];

	int gui_id_acc[3];
	int gui_id_accvar[3];

	int gui_id_color;
	int gui_id_colorb;

	int gui_id_size, gui_id_sizevar;
	int gui_id_growth, gui_id_growthvar, gui_id_growthend;
	int gui_id_rotvel, gui_id_rotvelvar;
	int gui_id_life, gui_id_lifevar;
	int gui_id_xsize, gui_id_ysize;

	int gui_id_alphain;
	int gui_id_alphaout;

	int gui_id_disp2_rot[3];
	int gui_id_disp2_rotvar[3];

	int gui_id_scale, gui_id_scaleAttack, gui_id_scaleDecay;

	int gui_id_alpha_blend;

	int gui_id_imgmode[2];			// - buttons: 0 = static, 1 = animating
	int gui_id_animating_kill[2];	// - buttons: 0 = loop, 1 = kill on done
	int gui_id_looping[2];			// - buttons: 0 = looping, 1 = non-looping
	int gui_id_particletype[4];		// - buttons: 0 = standard, 1 = yscale, 2 = streak, 3 = disp2
	int gui_id_orientation[PEMITTER_ORIENTATION_NUM]; // - buttons: 0 = billboard, 1 = y axis, 2 = yz billboard, 3 = decal
	int gui_id_flipx;
	int gui_id_flipy;

	int gui_id_dispersion[NUM_PARTICLE_DISPERSION_MODES];
	int gui_id_forceveldir[NUM_PARTICLE_FORCEVELDIR_MODES];
	int gui_id_usecameraivm;

	int gui_id_numframes;
	int gui_id_framedim[2];
	int gui_id_frametime;
	int gui_id_frames;

	int gui_id_eject_over[NUM_PARTICLE_EJECT_MODES];
	int gui_id_rate;
	int gui_id_limit;

	int gui_id_total;
	int gui_id_streakLen;

	int gui_id_randtheta;
	int gui_id_randthetavelsign;

	// int gui_id_gcol[2];
	int gui_id_decal;
	int gui_id_doublesided;

	int gui_id_shaderfx[NUM_PARTICLE_SHADERFXS];

	int gui_id_blend[NUM_PARTICLE_BLEND_MODES];
	int gui_id_blendColor[NUM_PARTICLE_COLOR_BLEND_MODES];

	int gui_id_vm_mode[NUM_PARTICLE_VMMODES];

	uns32 ope_oid;
#endif //  NO_TOOL

} PC_ParticleEmitter;


int ParticleCollectionInit( ParticleCollection *pc );
int ParticleCollectionDeInit( ParticleCollection *pc );
int ParticleCollectionLoad( ParticleCollection *pc, char *filename  );
int ParticleCollectionSave( ParticleCollection *pc );
int ParticleCollectionRebuildHashTables( ParticleCollection *pc );
int ParticleCollectionSetActivePC( ParticleCollection *pc );
ParticleCollection *ParticleCollectionGetActivePC( );


int particle_emitter_init(ParticleEmitter *pe
						  , crd pos[3], crd posVar[3]
						  , crd vel[3], crd velVar[3]
						  , crd acc[3], crd accVar[3]
						  , f32 size, f32 sizeVar
						  , f32 sizeVel, f32 sizeVelVar
						  , f32 thetaVel, f32 thetaVelVar
						  , f32 lifetime, f32 lifetimeVar
						  , f32 alphaIn, f32 alphaOut
						  , f32 rate, uns max, Res *r);
int particle_emitter_create_withdata(void **vpOE, ParticleEmitter *data, int use_parent);
int particle_emitter_setcolor(ParticleEmitter *pe, f32 r, f32 g, f32 b, f32 a);
int particle_emitter_stop(ParticleEmitter *pe);
int particle_emitter_destroy(ParticleEmitter *pe);
int particle_emitter_update(ParticleEmitter *pe, f32 dt);
int particle_emitter_build_renderdata(ParticleEmitter *pe, ParticleRenderData *dst_prd, int dst_prd_size, void *ocam );
int particle_emitter_render(ParticleEmitter *pe);
int particle_emitter_killparticle(ParticleEmitter *pe, Particle *p);
int particle_emitter_kill_all_particles(ParticleEmitter *pe);
int particle_emitter_test(char *resname);
int particle_emitter_create_blank(void **vpOE);

int particle_emitter_scale(ParticleEmitter *pe, float scale);
int particle_emitter_addvel(ParticleEmitter *pe, float *tvel);


EXTERN_C int particle_emitter_instance_create(char *name, crd *pos, void **obj);
EXTERN_C int particle_emitter_instance_create_withparent(char *name, crd *pos, void **obj);
int particle_emitter_instance_create_func(char *name, crd *pos, void **obj, int use_parent);

void *particle_emitter_instance_create_attach(char *name, void *atobj);
int particle_emitter_instance_addparticle(char *name, crd *pos, void **optr);
int particle_emitter_instance_locate(char *name, void **obj);

//int particle_emitter_fxgroup_create(char *name, crd *pos, void *parent);
//int particle_emitter_fxgroup_create_withcola(char *name, void *cola);
//int particle_emitter_fxgroup_create_asgroup(char *name, N2 *lnk, void **ofxgrp, crd *rpos, int rpos_for_sound_only );
//int particle_emitter_fxgroup_create_withveloffset(char *name, crd *pos);
int particle_emitter_fxgroup_launch(void *objfxgrp );

int particle_emitter_fxgroup_delete_byoid(int oid, int typeflags);
int particle_emitter_fxgroup_scale( void *ofxgrp,  float scale);
int particle_emitter_fxgroup_stop( void *ofxgrp );
int particle_emitter_fxgroup_kill_particles( void *ofxgrp );
void particle_emitter_fxgroup_color( void *ofxgrp, crd* rgba );

int particle_emitter_fxgroup_add_emitter_vel( void *ofxgrp, float *tvel );


int particle_emitter_render_strip(ParticleEmitter *pe, Particle *p);

int particle_emitter_fxgroup_util_locate(char *name, void **fxg);

int particle_emitter_fxgroup_create_aniaction(char *name, char *joint, float *offset, uns32 oid);
int particle_emitter_fxrepeat_create_aniaction(char *name, char *joint, float *offset, float repeatTime, float repeatLife, uns32 oid);

// pass ObjColAct which contains the collided object to attach an emitter to.
int particle_emitter_fxgroup_create_on_col(void* obj, char* fxGrp, void** ofxgOut);

int pc_util_xml_build_fxgroup_from_hash(HashTable *table, PC_FXGroup *fxg);

int pc_util_xml_build_fxgrouplist_from_hash(HashTable *table, PC_FXGroupList *fxgl);

// group render buffer stuff
// these are used for batching particles w/ the same texture, blend mode, etc...

typedef struct ParticleRenderBuffer
{
	uns type;
	uns32 texGen;
	uns32 normals_texGen;
	uns blendmode;
	uns shaderFX;

	uns noFaceCull;

	int max_quads;
	int mem_size;

	int active_quads;

	ParticleRenderData *prd_buffer;

} ParticleRenderBuffer;



EXTERN_C ParticleRenderBuffer gParticleRenderBuffer;

#define MAX_GPRD_QUADS 10000

int particle_renderbuffer_render(ParticleRenderBuffer *prb,uns32 renderflags);


int ParticlesInit();
int ParticlesFinal();
int ParticlesReload();
int ParticlesLoop();
int ParticlesLoopEnd();
int ParticlesReset();

int ParticlesSetFXGroupDfltSoundParams( float volume, uns32 channel_mask, int priority, float volswing, float dist_modifier );


typedef void (FXG_GRP_LIST_NAMES_CB)(char *, int);

int ParticleCollectionListFxgGroupNames( FXG_GRP_LIST_NAMES_CB func, ParticleCollection* pc );
int ParticleCollectionListFxgInGroup( FXG_GRP_LIST_NAMES_CB func, ParticleCollection* pc, char* groupName );
int ParticleCollectionNameIsFxgGroup( ParticleCollection* pc, char* name );

int ParticleCollectionListEmitterGroupNames( FXG_GRP_LIST_NAMES_CB func, ParticleCollection* pc );
int ParticleCollectionListEmittersInGroup( FXG_GRP_LIST_NAMES_CB func, ParticleCollection* pc, char* groupName );
int ParticleCollectionNameIsEmitterGroup( ParticleCollection* pc, char* name );


#endif

// EOF

