#ifndef RENDER_H
#define RENDER_H


// render.h
// Copyright 1998-2008 Play Mechanix Inc. All Rights Reserved

#include "g5.h"




typedef struct RenderGlobals
{
	int PLATFORM_ATI;
	int PLATFORM_NVIDIA;
	int MAX_SKEL_MATS;

	int USE_OFFSCREEN_RENDER_BUFFER; // you will lose all postfx if you turn this off

	// postfx settings
	int USE_POST_FX;			// big switch (disables bloom, noise, glow, etc)
	int USE_NOISE_FILTER;
	int USE_BLOOM;
	int USE_QUICK_BLOOM;		// use a cheap/fast vers of bloom filter...
	int USE_GLOW;

	int USE_BLURD_IMAGE;		// enables/disables local contrast, focal distance and 'blur' effects...
	int USE_DEPTH_OF_FIELD;		// focal distance settings for depth of field (depends on USE_BLURD_IMAGE)

	int USE_ODR_OUTLINES;

	int USE_COLOR_CORRECTION;		// brightness, black/white input, and rgb tint...
	int USE_COLOR_CORRECTION_PLUS;	// gamma, black/white output (depends on USE_COLOR_CORRECTION)
	int USE_HSV_CORRECTION;			// saturation

	int USE_CANVAS_FX;			// canvas 'painterly' shader

	int USE_MOTION_BLUR;

	int USE_NOISE_TEX; // this is required if using USE_NOISE_FILTER

	// shadow map settings
	int USE_SHADMAPS;
	int NUM_SHADMAP_SPLITS;    //   2 // max supported is 3
	int NUM_SHADMAP_TEXTURES;  //   2 // should be either 1 or NUM_SHADMAP_SPLITS
	int SHADMAP_TEX_SIZE;		//	 2048

	// reflection maps
	int USE_REFLECTIONS;
	int REFLECTION_MAP_TEX_SIZE;		//	 2048

	// pre-lighting (used for fur and possibly lit alpha)
	int USE_PRELIGHTING;
	int PRELIGHT_FUR;
	int PRELIGHT_ALPHA;
	int PRELIGHTING_MAP_TEX_SIZE;	// set to 0 top get exact match for screen size...

	// deffered shading
	int USE_DEFFERED_SHADING;

	// Shader Error logs
	int SHADER_ERR_LOG_LEVEL;  // 0 only log errors, 1 log loads and errors, 2 spew full log for all loads and errors

	// derived from above...
	int RT_BLOOM;
	int RT_BLOOM2;
	int RT_GLOW_HIGH;
	int RT_GLOW_LOW;
	int RT_LOCAL_CONTRAST;
	int RT_MOTION_BLUR;
	int RT_DYN_CUBE_MAP;

	int BLUR_HI_BUFFERS; // depending on which effects are enabled, each blur RT has many DBFRs
	int BLUR_LOW_BUFFERS; 

	int BLOOM_DBFR_SCRATCH; // 0
	int BLOOM_DBFR_FINAL;   // 1
	int GLOW_DBFR_SCRATCH;  // 0
	int GLOW_DBFR_FINAL;  // 2
	int LCONTRAST_DBFR_SCRATCH; // 0
	int LCONTRAST_DBFR_FINAL; // 3

	// screen-space pixel velocity for motion blur
	int VELOCITY_DBFR_SCRATCH; // 0
	int VELOCITY_DBFR_FINAL; // 1


	int RT_MAIN_CLR_BUFFERS;  // depending on which effects are enabled, we have more or less color attachments on main render target...
	int BLOOM_DBFR_RENDER;
	int GLOW_DBFR_RENDER;
	int BLOOM_CLR_ATTCHMENT; // GL_COLOR_ATTACHMENT1_EXT
	int GLOW_CLR_ATTCHMENT; //  GL_COLOR_ATTACHMENT2_EXT

	// for debugging..
	int DRAW_SHADOW_MAPS;

	// internal, changed only for testing at this point...
	int BLOOM_BEFORE_PARTICLES;  // apply bloom before applying particles (keeps bloom from shining 'over' alpha particles)
	int GLOW_BEFORE_BLUR;  // apply glow before create screen blur
	int SHADMAP_WIDTH;	//		 SHADMAP_TEX_RES
	int SHADMAP_HEIGHT;	//		 SHADMAP_TEX_RES
	float SHADMAP_SPLIT_LAMBDA; // 0.5  // affects how shad map splits are calculated...
	float SHADMAP_FAR_CLIP; // 1000
	int SHADMAP_STABLE_CAM; // shad cam is calculated to minimize 'swimming', default is to make optimal use of shad map texture space...
	int SHADMAP_HW_PCF; // yes!
	int SHADMAP_PCF_X;  // 8 // either 2, 4, or 8
	int NOISE_TEX_WIDTH; // 256
	int NOISE_TEX_HEIGHT; // 128

	int RENDER_PARTICLES_SEPERATELY; // this can lead to better particle batching, but they won't sort right w/ alpha

	int CACHE_SHADER_BINS; // save cached shader binaries, or use existing ones

}RenderGlobals;

extern RenderGlobals RenderG;

extern unsigned int gNoiseTexGen;



extern GLenum gGLColorAttachments[];
extern GLenum gGLDrawBufferNone[];
extern GLenum gGLDrawBufferBack[];
extern GLenum GL_TEX_ENUMS[];


extern float gShadMapSplits[MAX_NUM_SHADMAP_SPLITS][2]; // startz , depth


#define M_RENDERFLAGS_DEPTHONLY			0x00000001
#define M_RENDERFLAGS_SHADMAPPASS		0x00000002
#define M_RENDERFLAGS_SHADDRAWPASS      0x00000004
#define M_RENDERFLAGS_REFLECTIONDRAWPASS 0x00000008
#define M_RENDERFLAGS_PRELIGHTPASS		0x00000010
#define M_RENDERFLAGS_HARDSHELLPASS		0x00000020
#define M_RENDERFLAGS_SOFTSHELLPASS		0x00000040
#define M_RENDERFLAGS_BLOOMPASS			0x00000080
#define M_RENDERFLAGS_GLOWPASS			0x00000100
#define M_RENDERFLAGS_WIREFRAME			0x00000200




// APP MUST CALL THIS FUNC FIRST!!!

// APP MUST FIRST CALL ALL ENABLE FUNCS
// THEN CALL RenderSetup()
void RenderEnableReflections(int REFLECTION_MAP_TEX_SIZE);
void RenderEnableBloom(void);
void RenderEnableQuickBloom(void);
void RenderEnableGlow(void);
void RenderEnableBlurredImage(void);
void RenderEnableOutlines(void);
void RenderEnableColorCorrection(int use_cc_plus);
void RenderEnableHSVCorrection(void);
void RenderEnableHSVCorrection(void);

void RenderEnableMotionBlur(void);
void RenderEnableNoiseFilter(void);
void RenderSetupRenderBuffer( int id,
							 char *name,
							 int colorBuffers,
							 uns32 colorBufferFormat,
							 int hasDepth,
							 int hasStencil,
							 uns32 depthBufferFormat,
							 int width, int height);
void RenderEnableShadowMaps(
								int NUM_SHADMAP_SPLITS,		//   2 // max supported is 3
								int NUM_SHADMAP_TEXTURES,	//   2 // should be either 1 or NUM_SHADMAP_SPLITS
								int SHADMAP_TEX_SIZE		//	 2048
							);
void RenderEnablePreLighting(int PRELIGHT_FUR, int PRELIGHT_ALPHA, int PRELIGHTING_MAP_TEX_SIZE);
void RenderSetMaxSkelMats(int maxSkelMats);
void RenderEnableCanvasFX(void);
int RenderSetup(void);

int RenderInit();

int RenderLoop(uns32 flags);
ObjLightRes *FindShadLight();


int ObjCamCullTest( ObjCam *cam, float *world_pos, float radius, float *out_cam_pos );
int DynCubeCullTest( ObjCam *cam, float *world_pos, float radius, float *out_cam_pos );




void RenderODR( ObjDispRes *o, uns32 renderflags );
void RenderODR_Group( ObjDispRes *odg, uns32 renderflags );


void FrameStart(uns16 FrameStartFlags,uns32 renderflags );


// helper funcs for rendering postfx
void RenderState_SetupBasic2DCamera( float start_x, float start_y, float w, float h );
void DrawTexrdQuad( float x, float y, float z, float w, float h, float u, float v, float tw, float th );


void drawWireSphere(float radius, int slices, int stacks);
void drawWireSphere_clr(float radius, int slices, int stacks, crd* clr);

#ifdef DEBUG
void ShaderCountsInc( Res *shader, int enable, int objs, int tris );
void ShaderUniformDataAttemptCountsInc( char *name );
void ShaderUniformDataSetCountsInc( char *name, int count );
#endif

// Hong cam
// camera setup stuff
//void camPersp(double fovy,double aspect,double znear,double zfar, float *m4out );
//void camOrtho(float left, float right, float bottom, float top, float nearv, float farv, float *m4out );


// int ObjRenderCreateDList( ObjDispRes *odr );

enum
{
	SOLID_COLOR_TEXGEN_WHITE,
	SOLID_COLOR_TEXGEN_BLACK,
	SOLID_COLOR_TEXGEN_RED,
	SOLID_COLOR_TEXGEN_GREEN,
	SOLID_COLOR_TEXGEN_BLUE,
	NUM_SOLID_COLOR_TEXGENS,
};

extern unsigned int gSolidColorTexGens[NUM_SOLID_COLOR_TEXGENS];


#endif

