#ifndef RENDER_LIST_H
#define RENDER_LIST_H


// render_list.h
// Copyright 2011 Play Mechanix Inc. All Rights Reserved

#include "g5.h"



#define M_RLISTNODE_SORTFIRST			0x00000001
#define M_RLISTNODE_SORTLAST			0x00000002
#define M_RLISTNODE_BLEND				0x00000004

typedef struct RenderListNode
{
	Obj *o;
	Res *shader; // what shader to use when rendering this object, may vary from obj's shader depending on use for the list...
	uns32 id;
	uns32 flags;
	float sort_pri;
	int cull; 
	int cull_backoff; // counter for waiting before revisiting a culling decision...
	float camxyz[3]; // pos in camera space
	float facesort_cam_view_vec[3]; // 0,0,-1 transformed into camera space at the moment the node was face sorted
	uns32 facesort_cur_mesh; // save next mesh to sort if this mesh didn't finish sorting before


} RenderListNode;

typedef struct RenderList
{
	int size;
	int active_count;
	ObjCam camera;		// Camera for the current frame	
	ObjCam camera_prev;	// Camera from the previous frame
	ObjCam temp_cam;	// Used to save current camera when manipulating view for multiple screens
	int camfrust_cull; // cull objects using the cam frush or not?
	int cull_holdtime; // how long to hold on to cull info for...

	int compute_bounds; // compute the next two items?
	float obj_mins[3]; // smallest camera space xyz of all objs in list
	float obj_maxs[3]; // largest camera space xyz of all objs in list

	RenderListNode *nodes;

	int (*SortFunc)( const void *, const void *);

	int render_debug_items; // render debug stuff for list (col boxes, cull spheres, etc...)
	int outline_count;  // count of odrs that have the outline flag set, updated during render pass


	uns32 next_id;

#ifdef DEBUG
	int   dbgMinMaxZ;
	float min_z;
	float max_z;
#endif

} RenderList;



int RenderListInit( RenderList *list, int start_size );
int RenderListCopyCamera( RenderList *list, ObjCam *camera );
// int RenderListBuild( RenderList *list, ObjCam *camera, ObjMarker *start, ObjMarker *stop );
int RenderListAddObj( RenderList *list, Obj *o, Res *shader );
int RenderListRemoveObj( RenderList *list, Obj*o );
int RenderListGrow( RenderList *list );
int RenderListClear( RenderList *list );

int RenderListSort( RenderList *list );
int RenderListUpdateNodeCamXYZs( RenderList *list );
int RenderListFaceSort( RenderList *list, int cam_changed );
int RenderListCull( RenderList *list );

int RenderListRender( RenderList *list, uns32 renderflags );



// some different sorting funcs
int RenderListNodeCmpZB2F( const void *a, const void *b );
int RenderListNodeCmpSkelBlendZ( const void *a, const void *b );






#endif

