#ifndef RES_H
#define RES_H
// res.h
// Copyright 2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 08 Oct 04

// note: keep DC up-to-date with these defs


// #include "g5.h"


#define VID_MAXLIGHTS 12


#define M_RES_LOADED   0x0001
#define M_RES_COMPILED 0x0002
#define M_RES_DONTFREE 0x0004
#define M_RES_FREEDATA 0x0008
#define M_RES_PARSE1   0x0080
#define M_RES_LOOP     0x0100
#define M_RES_ANIM     0x0200
#define M_RES_SHARED   0xf000 // these flags are shared (OK to use locally)

enum {
	// -- see a-doc.h for notes on adding types
	// must match DC & resResSubTable[]
	RESSUB_NONE,				// 0
	RESSUB_GROUP,
	RESSUB_IMG1, 
	RESSUB_CAM,  
	RESSUB_KAN,  
	RESSUB_SKEL,				// 5
	RESSUB_LIGHT,
	RESSUB_COL,
	RESSUB_FONT,
	RESSUB_VID, 
	RESSUB_SHADER,				// 10
	RESSUB_SHADERPROGRAM,
	RESSUB_RENDERTEXTURE,
    RESSUB_STATEMACHINE,
    RESSUB_STATEMACHINE_PROGRAM, 
	RESSUB_DAMAGETREE,			// 15
	RESSUB_DISPINSTANCE,
	RESSUB_SHADERUNIFORMBLOCKBUFFERCOLLECTION,  // stores buffer objects for shader uniform data passed as blocks  (see comments in shader_gl.h) 
	RESSUB_MIPMAPIMG, // image with mip map levels...
	RESSUB_LOOKUP_HASH, // data is a hash table, entries are other resources w/ the same name, key is sub_type, value is res ptr...
	RESSUB_DISP5,				// 20
	RESSUB_GROUNDCOL,
	// these ones are just used by the engine for debugging, so keep them at the bottom, don't add them to dc
	RESSUB_SHADERUNIFORMDATA,  // ud is stored w/ shader program, this type is just a pointer for debugging 
	RESSUB_SHADERVERTATTRIB,  // this type is just for debugging 

	RESSUB_MAX					// 23 
	// must match DC & & resResSubTable[]
};

#define M_RESNEW_KID       0x0001
#define M_RESNEW_DONTLINK  0x0002
#define M_RESNEW_DONTCLEAR 0x0004
#define M_RESNEW_NOTABLEADD 0x0008

#define RESMARK_NONE	  0x00000000
#define RESMARK_PERM      0x00000001

#define RESMARK_SCENE0      	0x00010000
#define RESMARK_SCENE1      	0x00020000
#define RESMARK_SCENE2      	0x00040000
#define RESMARK_SCENE3      	0x00080000
#define RESMARK_SCENE4      	0x00100000
#define RESMARK_SCENE5      	0x00200000
#define RESMARK_SCENE6      	0x00400000
#define RESMARK_SCENE7      	0x00800000
#define RESMARK_SCENE8      	0x01000000
#define RESMARK_SCENE9      	0x02000000
#define RESMARK_SCENE10      	0x04000000
#define RESMARK_SCENE11      	0x08000000
#define RESMARK_SCENE12      	0x10000000
#define RESMARK_SCENE13      	0x20000000
#define RESMARK_SCENE14      	0x40000000
#define RESMARK_SCENE15      	0x80000000



#define RES_TAG_SIZE 8 
#define RES_NAME_SIZE PM_NAME_SIZE // must match w/ data compiler!
typedef struct {
	// must match DC
	N2 lnk;
	uns16 flags;
	uns32 mark;

	char name[RES_NAME_SIZE];
	void *data;
	HashTable attributes; // this is constructed at load time from atBuf, sucks but getting hash table loader in g3 would be a disaster...

} Res;

#define RES_LIST_DEPTH 4


extern Res resCompiled[];

//
// Res -- subtype flags
//

//
// DATA
//

// global res struct
typedef struct {
	uns8 flags;
	uns8 dead;
	N2 loadedList;
	N2 deadList;
	memu memubuf[1];

	Res *StateMachines;
	Res *StateMachinePrograms;

} resGlobals;

extern resGlobals resG;

/*
// restag stuff
void ResSetTag( char c1, char c2 );
void ResRestoreTag();
int ResTagKill(char c1, char c2 );
*/

// used by NBLOad
typedef struct {
	Res *rc;
	N2  *list;
	Res *rch;
	N2 *n;
	N2 *kidlist;
	memu kidlist0;
	memu kidlistx;
	void *rhdata;
	Res	 *skelRes;

	int done;

	uns32 mark;

	Res* dispRes;

} ResLinkState;

int resLinkOne(ResLinkState *state); // Res *rc,N2 *list,Res *rch, Res **next_rc, Res **next_rch);



enum ResDispTexTypes { // this one should match RESMTR_TYPES in dc3
	RESDISP_TEX_TYPE_DIFFUSEMAP,
	RESDISP_TEX_TYPE_DIFFUSE_SPECMAP,  // combined diffuse and spec maps (spec=alpha channel)
	RESDISP_TEX_TYPE_NORMALMAP,
	RESDISP_TEX_TYPE_SPECMAP,
	RESDISP_TEX_TYPE_FURDIRMAP,
	RESDISP_TEX_TYPE_FURHGHTMAP,
	RESDISP_TEX_TYPE_FURDIR_FURHGHTMAP, // combined fur dir and fur height map
	RESDISP_TEX_TYPE_ALPHAMAP,

} ; 


// G_RESSUB_DISPINSTANCE

typedef struct {
	char res_name[RES_NAME_SIZE];
//	char name[RES_NAME_SIZE];
//	char parent[RES_NAME_SIZE];
	uns16 atSize;
	float trans[3];
	float rot[3];
	float scale[3];
	char *atBuf;
} ResDataDispInstance;  // this is a copy of another res w/ different transform applied

#define RESDISP5_MAX_TEXTURES 8
#define RESDISP5_MAX_VERTDATA 8

#define RESDISP5_DATA_FIELD_VERTS       0x00000001
#define RESDISP5_DATA_FIELD_UVS0        0x00000002
#define RESDISP5_DATA_FIELD_UVS1        0x00000004
#define RESDISP5_DATA_FIELD_UVS2        0x00000008
#define RESDISP5_DATA_FIELD_UVS3        0x00000010
#define RESDISP5_DATA_FIELD_UVS4        0x00000020
#define RESDISP5_DATA_FIELD_UVS5        0x00000040
#define RESDISP5_DATA_FIELD_UVS6        0x00000080
#define RESDISP5_DATA_FIELD_UVS7        0x00000100
#define RESDISP5_DATA_FIELD_VCOLOR      0x00000200
#define RESDISP5_DATA_FIELD_NORMALS     0x00000400
#define RESDISP5_DATA_FIELD_TANGENTS    0x00000800
#define RESDISP5_DATA_FIELD_JINDICES    0x00001000
#define RESDISP5_DATA_FIELD_JWEIGHTS    0x00002000
#define RESDISP5_DATA_FIELD_VERTDATA0   0x00004000
#define RESDISP5_DATA_FIELD_VERTDATA1   0x00008000
#define RESDISP5_DATA_FIELD_VERTDATA2   0x00010000
#define RESDISP5_DATA_FIELD_VERTDATA3   0x00020000
#define RESDISP5_DATA_FIELD_VERTDATA4   0x00040000
#define RESDISP5_DATA_FIELD_VERTDATA5   0x00080000
#define RESDISP5_DATA_FIELD_VERTDATA6   0x00100000
#define RESDISP5_DATA_FIELD_VERTDATA7   0x00200000

typedef struct
{
	uns32 count;  // how many floats per vert for this data
	uns32 offset; // where in data does this data start
} Disp5_DataPtr;


typedef struct
{
	uns16 type;
	uns16 layer; 
} Disp5_TexInfo;

typedef struct
{
	uns32 flags;
	// uns32 indices[3];
	uns32 *indices; // this is a pointer into the main disp5 indices buffer
	float xyzrCull[4];
	float xyzCam[3];
} Disp5_Face;

typedef struct
{
	char name[RES_NAME_SIZE];
	char parent[RES_NAME_SIZE]; // name of res that this mesh was parented to in the maya file...
	int parent_index; // index of parent mesh in the disp5's list of meshes...
	uns32 depth; // how many parents this mesh has in the disp5's list of meshes...
	uns32 flags;
	uns32 num_faces;
	uns32 num_indices;
	uns32 *indices; // this is a pointer into the main disp5 indices buffer
	Disp5_Face *faces; // this is a pointer into the main disp5 faces buffer
	uns32 num_verts;
	uns8 *data; // this is a pointer into the main disp5 verts/data buffer
	float xyzrCull[4];
	float xyzCam[3];
	// initial transform
	float trans[3];
	float rot[3];
	float scale[3];
	float vm[12];
	// float ivm[12];

} Disp5_Mesh;


typedef struct {
//	uns16 mode; // all ResDataDisp* start with uns16 mode
	char name[RES_NAME_SIZE]; // name of geo from the maya file...
	char parent[RES_NAME_SIZE]; // name of res that this obj was parented to in the maya file...
	uns16 atSize;
	char *atBuf;
	uns8 *data;  // packed data for each vert (verts, uvs, vertdata, etc.)
	uns32 *indices;  // faces by indices...


	// initial transform
	float trans[3];
	float rot[3];
	float scale[3];

	float xyzrCull[4]; // center and radius for culling purposes

	uns32 data_fields; // mask to check which data fields are present...
	uns32 data_stride; // size of data for each vert, also how many bytes between each vert in data

	int num_faces;
	int num_indices;
	int num_verts;

	uns32 indices_size; // num_indices * sizeof(uns32)
	uns32 data_size;   // data_stride * num_verts;
	uns32 faces_size;  // num_faces * sizeof(Disp5_Face)

	Disp5_DataPtr verts;
	Disp5_DataPtr uvs[RESDISP5_MAX_TEXTURES];
	Disp5_DataPtr vcolors;
	Disp5_DataPtr normals;
	Disp5_DataPtr tangents;
	Disp5_DataPtr joint_indices;
	Disp5_DataPtr joint_weights;
	Disp5_DataPtr vert_data[RESDISP5_MAX_VERTDATA];

	uns32 texHash[RESDISP5_MAX_TEXTURES]; // hash of texture's res image name, used to link texture to image

	uns32 vert_data_type[RESDISP5_MAX_VERTDATA];


	uns32 vbo; // vertex buffer object (verts when stored in gl)
	uns32 vio; // vertex index object (indices when stored in gl)
	uns32 texGen[RESDISP5_MAX_TEXTURES]; // texture objects in gl

	Disp5_TexInfo texInfo[RESDISP5_MAX_TEXTURES];

	// these fields are to aid in face/mesh sorting...
	Disp5_Face *faces; // faces w/ cull center (for sorting)

	uns32 num_meshes;
	uns32 meshes_size;
	Disp5_Mesh *meshes; // indices and cull centers for individual meshes that make up the disp5

} ResDataDisp5;

// ground cols
typedef struct {
	float min[3]; // min corner of 3d col box
	float max[3]; // max corner of 3d col box
	float verts[3][3];
	float verts2D[3][3]; // same as verts but y=0 and z mapped to y
	float normal[3];
} GroundColTri;

#define RESDATAGCOL_TRANSFORMED	0x04 // has been transformed from local group to world space

typedef struct {
	uns32 flags;
	float min[3]; // min corner of 3d col box
	float max[3]; // max corner of 3d col box

	uns32 tri_count;
	GroundColTri *tris;
	char *atBuf;
} ResDataGroundCol;



typedef struct {
	PSM_StateMachine *sm;
} ResDataStateMachine;

typedef struct {
	PSM_Program program;
} ResDataProgram;


enum {
	RES_TEX_NONE,
	RES_TEX_RGB8,
	RES_TEX_RGBA8,
	RES_TEX_RGB332,
	RES_TEX_RGB555,
	RES_TEX_DTX1,
	RES_TEX_DTX5,
	RES_TEX_ATI3DC,
	RES_TEX_LATC1,
	RES_TEX_LUM,
	RES_TEX_LUMA,
};

#define M_RESDATAIMG1_MIPMAP   0x0001
#define M_RESDATAIMG1_HASALPHA 0x0002
#define M_RESDATAIMG1_CUBEMAP	0x0004
#define M_RESDATAIMG1_LOADED		 0x0010

typedef struct {
	// must match DC
	uns16 type;
	uns16 flags;
	uns16 dimensions; // 1d, 2d, 3d
	uns16 w;
	uns16 h;
	uns16 d;
	uns32 size;
	uns32 texmem_size; // maybe more than size if using auto mip maps...
	uns32 texHash;
	uns32 texGen;
	void *tex;
} ResDataImg1;

int ResNewImg1( Res **resp, char *name, uns32 mark, int w, int h, int d, int bytesPerPixel );
int ResImg1Upload( Res *res, int mipmap, int compress );
int ResImg1SavePPM( Res *res, char *filename );


#define RES_MIPMAPIMG_MAX_MIPMAP_LEVELS 16
typedef struct {
	// must match DC
	uns16 type;
	uns16 flags;
	uns16 w[RES_MIPMAPIMG_MAX_MIPMAP_LEVELS];
	uns16 h[RES_MIPMAPIMG_MAX_MIPMAP_LEVELS];
	uns32 mipmap_levels;
	uns32 size[RES_MIPMAPIMG_MAX_MIPMAP_LEVELS];
	uns32 texHash;
	uns32 texGen;
	void *tex[RES_MIPMAPIMG_MAX_MIPMAP_LEVELS];
} ResDataMipMapImg;



// Enum order should match enum in res.h in data compiler project
enum {
	RESVID_THEORA,
	RESVID_GLCGZ,
};

// Struct should match G_ResDataVid in the data compiler project
typedef struct {
	uns32 fileSize;
	void *data;
	void *theora;
	uns32 texGen;
	uns8 videoType;
} ResDataVid;


#define M_RESDATACAM_ORTHO 0x0001

typedef struct {
	// must match DC
	uns16 mode;
	uns16 atSize;
	float rot[3];
	float trans[3];
	float scale[3];
	float fov;
	float orthoWidth;
	float clipNear;
	float clipFar;
	float aspectRatio;
	float focalLength;
	float aperture[2]; // aperture width & height
	char *atBuf;
} ResDataCam;

#define M_RESDATAKAN_LOOP 0x0001
#define M_RESDATAKAN_SPLINE 0x0002
#define M_RESDATAKAN_GROUP 0x0004

typedef struct {
	uns16 flags;
	uns16 numk;
	float firstFrame; // as keyed by animator
	float length; // total length (milliseconds)
	char* atBuf;
	uns16 atSize;
	// followed by ResDataKanKey
} ResDataKan;

#define M_RESDATAKANKEY_RX 0x0001
#define M_RESDATAKANKEY_RY 0x0002
#define M_RESDATAKANKEY_RZ 0x0004
#define M_RESDATAKANKEY_TX 0x0008
#define M_RESDATAKANKEY_TY 0x0010
#define M_RESDATAKANKEY_TZ 0x0020
#define M_RESDATAKANKEY_SX 0x0040
#define M_RESDATAKANKEY_SY 0x0080
#define M_RESDATAKANKEY_SZ 0x0100
#define M_RESDATAKANKEY_LINEAR  0x0200
#define M_RESDATAKANKEY_ANGULAR 0x0400

enum
{
	RESDATAKANKEY_CH_RX,
	RESDATAKANKEY_CH_RY,
	RESDATAKANKEY_CH_RZ,
	RESDATAKANKEY_CH_TX,
	RESDATAKANKEY_CH_TY,
	RESDATAKANKEY_CH_TZ,
	RESDATAKANKEY_CH_SX,
	RESDATAKANKEY_CH_SY,
	RESDATAKANKEY_CH_SZ,
};

#define RESDATAKANKEY_CH_NUM 3
typedef struct {
	float time; // duration
	float accumTime; // time from start of first key to start of this key
	uns16 flags;
	uns16 keynum;
    float rts[3*RESDATAKANKEY_CH_NUM]; // rx,ry,rz,tx,ty,tz,sx,sy,sz
} ResDataKanKey;

typedef struct {
	float orient[3];
	float vm[12];
	float bps[12];
	char name[PM_NAME_SIZE];
	char parent[PM_NAME_SIZE];
	int parent_index;
} ResDataJoint;

// every vert has a weight for every joint
// this is how they are stored in the res:
// v0.j0 v0.j1 v0.j2 .. v1.j0 v1.j1 v1.j2 .. v2.j0 v2.j1 v2.j2

#define M_RESDATASKEL_JJJBINDINVERTED 0x0001
typedef struct {
	uns16 flags;
	uns16 numJoints;
	char geoName[PM_NAME_SIZE];
	ResDataJoint *joints;
//	float *weights;
} ResDataSkel;

typedef struct ResVertWeights
{
	int count;  // number of joints vert is weighted to
	int *joints;  // list of joints by index into ResDataSkel joints ptr
	float *weights; // list of weigths for each joint
} ResVertWeights;

// new 'optimized' format for weights
// since most verts are only weighted to one or two joints
// it is faster to just look at the weights that are non-zero
// rather than loop through every vert and every joint
// this is esp. true for complex objects
// so here we store the weights as a ResVertWeights struct per vert
// note that for now the data compiler still produces weights in the 
// 'ResDataSkel' format, then we build this optimized weights list
typedef struct {
	char geoName[PM_NAME_SIZE];
	int count; // number of weights there are
	ResVertWeights *weights;
} ResDataSkelWeights;



#define RESDATALIGHT_NONE		 0x00
#define RESDATALIGHT_AMBIENT	 0x01
#define RESDATALIGHT_DIRECTIONAL 0x02
#define RESDATALIGHT_POSITIONAL  0x04
#define RESDATALIGHT_SPOT		 0x08

/*
enum RESDATALIGHT_TYPES {
	RESDATALIGHT_NONE = 0,
	RESDATALIGHT_AMBIENT = 0x00,
	RESDATALIGHT_DIRECTIONAL,
	RESDATALIGHT_POSITIONAL,
	RESDATALIGHT_SPOT,
};
*/

enum {
		RESDATALIGHTP_COMMON_INTENSITY,
		RESDATALIGHTP_SHADOW_INTENSITY,
		RESDATALIGHTP_SHADOW_SPLITLAMBDA,
		RESDATALIGHTP_SHADOW_FAR_CLIP,
		END_RESDATALIGHT_COMMON,
}; // common light params

enum {
	RESDATALIGHTP_SPOT_CONE = END_RESDATALIGHT_COMMON,
	RESDATALIGHTP_SPOT_EXPONENT,
	RESDATALIGHTP_SPOT_LINEAR_ATTENUATION,
	RESDATALIGHTP_SPOT_QUADRATIC_ATTENUATION,
};

enum {
	RESDATALIGHTP_POINT_LINEAR_ATTENUATION = END_RESDATALIGHT_COMMON,
	RESDATALIGHTP_POINT_QUADRATIC_ATTENUATION,
};




#define M_RESDATALIGHT_SHADDIR		0x01
#define M_RESDATALIGHT_SHADPOS		0x02
#define M_RESDATALIGHT_NOLIGHT		0x04
#define M_RESDATALIGHT_NOSPEC	    0x08
#define M_RESDATALIGHT_NODIFFUSE	0x10
#define M_RESDATALIGHT_SHADINTENSITY	0x20



typedef struct {
	uns8 type;
	uns8 flags;
	uns16 atSize;
	float pos[3];
	float dir[3];
	float rgb[3]; // intensity 'baked' in
	float parameters[16]; // parameters
	char *atBuf;
} ResDataLight;

enum {
	RESDATACOLTYPE_NONE,
	RESDATACOLTYPE_SINGLE,
	RESDATACOLTYPE_SKEL,
	RESDATACOLTYPE_PROP,
};

#define M_RESDATACOL_RTS 0x01 // has ResDataColRTS imm following
#define M_RESDATACOL_GND 0x02 // marked for ground col w/ splines
#define M_RESDATACOL_TRANSFORMED 0x04 // verts have been transformed by parent group


typedef struct {
	uns8 flags;
	uns8 type;
	uns16 atSize;
	char *atBuf;
	void *blocks;
	void *polys;
} ResDataCol;

typedef struct {
	float rts[12];
} ResDataColRTS;

enum {
	RESDATACOLBLOCK_NONE,
	RESDATACOLBLOCK_XZMM,
	RESDATACOLBLOCK_XYZR,
};

#define M_VIDPLAY_LOOP			0x00000001
#define M_VIDPLAY_KILLWHENDONE  0x00000002
#define M_VIDPLAY_IGNORETIME	0x00000004

typedef struct {
	uns8 type; // must be in 1st int
	uns8 flags;
	uns16 xx1;
} ResDataColBlock;

typedef struct {
	ResDataColBlock blk;
	float	minx;
	float	miny;
	float	minz;
	float	maxx;
	float	maxy;
	float	maxz;
	float	xyzr[4];
	int32   off; // offset to list of offsets to G_ResDataGBPoly
} ResDataCBXZMM;

typedef struct {
	ResDataColBlock blk;
	char colName[PM_NAME_SIZE];
	uns16   jointNum;
	uns16   colNum;
	float	xyzr[4]; // xyz of center and r*r
	int32   off; // offset to list of offsets to G_ResDataGBPoly
} ResDataCBXYZR;

#define RESDATACBPOLY_NUMV_END 0xff

typedef struct {
	float vrt[3];
	float nrm[3];
} ResDataCBVN;

typedef struct {
	uns8	numv;
	uns8	xx1;
	uns16	xx2;
	float	nrmk[4]; // normal & plane constant
	ResDataCBVN v[3];
	// followed by other ResDataCBVN needed
} ResDataCBPoly;

typedef struct {
	struct {
		float vrt[3];
	} v[3];
} ResDataCBTri;

typedef struct
{
	char name[PM_NAME_SIZE];

} ResDataDamagePart;


typedef struct 
{
//	ResDataDamagePartName part;  // part name is res name
	ResDataDamagePart parent;
	int num_children;
	ResDataDamagePart *children;
} ResDataDamageTree;



typedef struct {
	uns16 ascii;
//	uns8 xx1;
	uns16 xx2;
	float uvbox[BOX_SIZE]; // ltrb
	float width;
} ResDataFChar;



enum ResDataShaderTypes {
	RESDATA_SHADER_TYPE_VERT_SHADER,
	RESDATA_SHADER_TYPE_FRAG_SHADER,
};

typedef struct {
	uns8 type;
	char filename[64];
	uns32 shader;
} ResDataShader;



typedef struct {
	char descr[PM_NAME_SIZE];

	uns32 shaderprogram;
	uns32 shaderflags;

	uns8  render_priority; // how to prioritize this shader when sorting objs by shader for rendering (lower = render obj's with this shader first)
	uns8  obj_sortorder;  // how to sort objects being rendered by this shader

	void  (*setup_func)(Res *shader, uns32 renderflags);    // run whenever shader is 'turned on' (set gl states here), input is shader res, shader obj, state rec, flags
	void  (*teardown_func)(Res *shader, uns32 renderflags); // run whenever shader is 'turned off', input is shader res, flags
	void  (*prerender_func)(Res *shader, void *odr, uns32 renderflags); // run just before each obj is rendered (use to set obj specific shader uniform vars), input is shader res, obj disp res, state rec, flags
	void  (*render_func)(Res *shader, void *odr, uns32 renderflags);  // used to render an obj disp res w/ shader (glBegin, glVert, glEnd, etc, goes here), input is shader res, obj disp res, state rec, flags
	void  (*postrender_func)(Res *shader, void *odr, uns32 renderflags); // run after each odr is rendered, input is shader res, obj disp res, state rec, flags

	char lightprofile[VID_MAXLIGHTS]; // what light type of light (RESDATALIGHT_TYPES) the shader supports for each light slot ( -1 for any type of light)
	HashTable uniform_data;  // key is uniform var name, val is ShaderUniformData (descr plus values for shader uniform datas)
	HashTable uniform_blocks; // key is uniform block name, val is ShaderUniformBlock (descr w/ current buffer bindings for shader uniform data blocks)
	struct ShaderUniformData* uModelViewProjectionMatrix;
	struct ShaderUniformData* uModelViewMatrix;
	uns32 vertarrays; // mask of what vert attribute arrays are needed by this shader, set dynamically when shader is loaded
	uns32 matrices; // mask of what model/view/proj/tecture matrices are needed by this shader, set dynamically when shader is loaded


} ResDataShaderProgram;


// forward declare some items defined in shader_gl.h
struct ShaderUniformBlockBufferCollection;

struct ShaderUniformData;
struct ShaderUniformBlock;
struct ShaderUniformBlockItem;

typedef struct {
	struct ShaderUniformBlockBufferCollection *subbc; 
} ResDataShaderUniformBlockBufferCollection;


// in debug mode only, a ResDataShaderUniformData is added to for each uniform, just for viewing in gcon
// we also use this struct for block uniforms and for uniform data buffer objects hung off a ShaderUniformBlockBufferCollection
typedef struct {
	struct ShaderUniformData *sud; // ShaderUniformData
	struct ShaderUniformBlock *sub; // ShaderUniformBlock
	uns32 udbo; // ShaderUniformBlock
} ResDataShaderUniformData;

typedef struct {
	int type;
	int location;
} ResDataShaderVertAttribute;


#define M_RESDATAFONT_HASALPHA 0x0001

typedef struct {
	uns16 flags;
	uns16 xx1;
	uns16 asciiFirst;
	uns16 numfcs;
	uns32 texHash;
	uns32 texGen;
	float spaceWidth;
	float bottomY;
	float centerY;
	float height;
	float padX; // any pos or negative padding to apply to each character
	float padY; // any pos or negative padding to apply to each character
	void *fcs; // ResDataFChar
} ResDataFont;

#define RES_FRAME_TIME PM_FRAME_TIME // 16 milliseconds per frame == approx 60 frames-per-second
#define RES_FRAME_TIMEF 16.66667f
#define FRAMES_TO_SECS (RES_FRAME_TIMEF/1000.0f)
#define MS_TO_FRAMES (1.0f/RES_FRAME_TIMEF)


#define RES_CAM2D_Z 100

extern int xResInit(void);
extern void xResFinal(void);
extern int xResStart(void);
extern int xResRelease();

extern int xResLoadGround(Res *res,			void *rhdata);
extern int xResLoadDisp(Res *res,           void *rhdata);
extern int xResLoadVid(Res *res,            void *rhdata);
extern int xResLoadImg(Res *res,            void *rhdata);
extern int xResLoadMipMapImg(Res *res,      void *rhdata);
extern int xResLoadCam(Res *res,            void *rhdata);
extern int xResLoadLight(Res *res,          void *rhdata);
extern int xResLoadSkel(Res *res,           void *rhdata);
extern int xResLoadGroup(Res *res,          void *rhdata);
extern int xResLoadCol(Res *res,            void *rhdata);
extern int xResLoadFont(Res *res,           void *rhdata);
extern int xResLoadDmgTree(Res *res,		void *rhdata);
extern int xResLoadDispInst(Res *res,void *rhdata);
extern int xResLoadKan(Res *res,void *rhdata);


///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// SYSTEM INDEPENDENT


extern int ResInit(void);
extern void ResFinal(void);
extern int ResReset(uns reset);

extern int ResNew(void *resp,uns sub,uns size,uns flags,char *name, uns32 mark);
extern int ResKill(void *r);
extern int ResLevelKill(int level);
extern int ResKillSub(Res *start, uns8 sub, int recurse );

extern void ResLoopEnd(void);
extern int ResIsSub(void *r,uns8 subtype);

EXTERN_C void* ResFindNth(N2 *list,char *name,int *nth,int level);
extern Res* ResFindSubNth(N2 *list,char *name,uns subtype,int *nth);
extern int ResLoad(void *resp,char *name,uns32 mark);
extern int ResAdd(char *name);
extern int ResAddAtCamera(char *name, int cam, int first );
extern int ResAddAtMark(char *name, char *mark );
extern int ResLoadShader(void *resp, char *shaderFile, int force_reload ); // , char *techniqueFile);
extern int ResCmp(char *ngame,char *ndata);
extern int ResUnLoad(char *name,uns32 mark, int kill_objs_using_res );

extern int ResClearMark(char *name,uns32 mark);


extern void* ResLoadedFirst(void);
extern int ResLoadedCount(void);

extern int ResInfoExpand(char *buf,int size,Res *r,uns flags);



extern int ResNFSZeroTime(void *nsr,N2 *n);
extern int ResNFSFlag(void *nsr,N2 *n);
extern int ResNFSImg(void *nsr,N2 *n);


// g5 file stuff
#define G_MAGIC 0xBE136457

#define G_STANDARD_HEADER "LB?? - Copyright (C) ????, PlayMechanix, Inc."
#define G_STANDARD_HEADER_SIZE 128

#define LBHEAD_POOL_SIZE 8
typedef struct {
	// must match DC
	uns32 magic;
	char pool[LBHEAD_POOL_SIZE];
	uns16 flags;
	uns16 rhSize;
	uns32 size;
} G_LBHead;

typedef struct {
	uns32 magic;
} G_LBTail;


#define RESFILE_FLAGS_PERM 0x00000001  

typedef struct ResFileTrackNode
{
	uns32 flags;  //
	int count; // how many load requests have we recieved for this file?
	
} ResFileTrackNode;

EXTERN_C HashTable gResFileTrackTable;



int ResDispBuildVBO( Res *dispres );
int ResDisp5BuildVBO( Res *dispres );

int ResDispGroupBuildDamageTrees( Res *grp );


// int ResSkelBuildOptWeights( ResDataSkel *rds, Res *dispres, void *link );
// int ResSkelResolveParentIndices( ResDataSkel *rds );

Res *FindDispResShaderRes( Res *r1 );

#define	M_COREADDRES_FLAGS_IGNORE_CREATE_TRIGGER	0x00000001
#define	M_COREADDRES_FLAGS_APPEND					0x00000002

int CoreAddResNodes(Res *res,uns32 addflags,N2 *lnk, float* append_vm, float* end_vm);
// int CoreAddRes(char *name,int level,int type,uns32 flags, void **o); 
int CoreAddRes(char *name, N2 *link, uns32 addflags, float* append_vm, void **o );  // o points to the added obj or a group containing all the added objs...
int CorePostAdd(N2* link);
int CorePostAdd_GroundCol(void* gcol, float* group_vm);
int CorePostAdd_Col(void* col, float* group_vm);

int ResNotesGetValue( char *notes, char *key, char *dest, int dest_len );

// use to validate that dispres is ok to render with shader (has all required data, tex maps, etc.)
int ResDispValidateShader( char *shader, Res *dispres );

int ResGroupBuildVBOs( Res *grp, int nth );
int ResGroupBuildLookupHashes( Res *grp, int nth );

int ResImageGetData( Res *res, uns32 *texhash, uns32 *texgen );
int ResFindImageByHash(uns32 hash, Res **res, uns32 *texgen );
int ResFindImageByTexGen(uns32 texGen, Res **res );
int ResFindImageByName(char *name, Res **res, uns32 *texgen );

int FindDispResLightGroup( Res *r );
int FindLightResLightGroup( Res *r );

int ResTableInit(void);
int ResTableReset(void);
int ResTableAdd(Res *r,int add_kids);
int ResTableRemove(Res *r);
int ResTableFind(char *name, uns subtype, Res **rptr);

int ResDumpTGA_BGR(FILE *fh,uns8 *src0,int width,int height,int pixb,int rowb);

// used by ObjResolveDisp
int AddODRUVAnimsFromResAttrs( void *o );


int ResLoadCoreResources();
void ResFindKillObjsUsingRes( Res *r, int do_kids );

int ResNotesToAttributes( char *notes, HashTable *attributes );
int ResAddNotesToAttributes( char *notes, HashTable *attributes );

int KanWalk( char *name );
int ResKanGetDerivs( Res *res, float time, float delta, float max_drot, float *speed, float *acc, float *droty );
int ResKanGetPosAndVel( Res *r, float time, float *trans, float *rot, float *scale, float *tvel, float *rvel );
int ResKanGetPosAndVelHint( Res *r, float time, float *trans, float *rot, float *scale, float *tvel, float *rvel, uns16 keyHint );
int ResKanGetSpeedAndInvTurnRadius( Res *res, float time, 
								    float spd_bkwd_delta, 
								    float spd_fwd_delta, 
								    float turn_bkwd_delta, 
								    float turn_fwd_delta, 
								    float *speed, float *inv_turn_radius );
int ResKanGetKeys( Res *res, float time, float *stime, ResDataKanKey **pkey1, ResDataKanKey **pkey2, ResDataKanKey **pkey3  );

void resPrepBatchCullCenters(Res* res);

int ResDisp5_LineCreate( Res **resp, char *name, uns32 mark, int count, float *points, float *uvs, float *colors, char *img, float line_width );

extern HashTable g_OverrideTable;
void ResOverrideAttributes();

void ResGetLightBlinkParams(Res* res, void* o);

#endif // ndef RES_H

