#ifndef SYS_H
#define SYS_H
// sys.h
// Copyright 1998-2004 Play Mechanix Inc. All Rights Reserved
//
// GNP 05 May 00
// JFL 26 Aug 04
// JFL 20 Jul 05; simple protection

#include "g5.h"


#if PRODUCTION
#define M_SYSG_PROFLAGS_JAMMALIVES1 0x01
#define M_SYSG_PROFLAGS_JAMMALIVES2 0x02
#define M_SYSG_PROFLAGS_DONTFLIP    0x04
#define M_SYSG_PROFLAGS_CFFILEERROR	0x08
#endif // PRODUCTION

#define SYSG_VERSTR_SIZE 64
#define SYSG_TZPATH_SIZE 512
#define SYSG_TZNAME_SIZE 8

#define ENABLE_FRAME_SPIKE_LOGGING 1
#define ENABLE_FRAME_SPIKE_TRACKING 0
#if ENABLE_FRAME_SPIKE_TRACKING
	#define FRAME_SPIKE_COUNT 120
	extern int gFrameSpikes[FRAME_SPIKE_COUNT];
#endif

typedef struct {
	char	tzStr[PM_NAME_SIZE];		// name of timezone, actually the text param id
	int		tzOff;						// offset from GMT
	int		tzDST;						// observe Daylight Savings Time
	char	tzLinuxFile[PM_NAME_SIZE];	// name of linux file
} tzTab_T;


typedef struct {
	uns16 n2FlagsOr;
	int level;
	char versionstr[SYSG_VERSTR_SIZE];
	char tzname[SYSG_TZNAME_SIZE];	// for timezone printing, set only if timezone set successfully
	time_t start_time;

	int AssertOnMissingSound;
	int AssertOnMissingTexture;

	/*
	uns8 quit;

	int tag;

	uns mode;
	uns8 restart;
	uns8 cfg[32];
	uns8 flags[32];
	uns8 info[128];
	char str[8][16]; // 8 16 character strings

	uns forceFrameTime;


	uns loop;
	char log[1024];
	uns8 proflags; // PRODUCTION
	uns8 profailrgb[3]; // PRODUCTION
	int projammasn; // PRODUCTION

	uns64 loopTime;

	memu var[32];
	int gunCalValidated;

	char *scriptStartS1;
	char *scriptStartS2;
	char *progStartS1;
	char *progStartS2;

	char path[1024]; // for all data
	char adjpath[256]; // for adjustments
	char audpath[256]; // for audits
	char cfpath[256]; // for parsed sequences
	char swappath[256]; // for tga swap
	char sndpath[256]; // for sound path
	char logpath[256]; // for log path
	char keymacros[8][16]; // 8 16 character strings
	char *curSeq;
	char *curProc;
	char tzpath[SYSG_TZPATH_SIZE];	// for timezone, set only if timezone set successfully
	*/
} SysGlobals;

enum uTimers {
	UTIMER_FRAME,
	UTIMER_FPS,
	UTIMER_GAMELOOPDELTA,

	UTIMER_OBJLOOP,
	UTIMER_GNDCOL,
	UTIMER_GNDCOL_TESTS,
	UTIMER_WINDUPDATE,
	UTIMER_WINDTRANSFORM,
	UTIMER_ENVCOLCOLLIDE,
	UTIMER_ENVCOLUPDATE,
	UTIMER_INP,
	UTIMER_PROC,
	UTIMER_OBJ,
	UTIMER_OBJNEW,

	UTIMER_OBJFUNC,
	UTIMER_OBJANI,
	UTIMER_SCENELOAD,
	UTIMER_SCENELOOP,
	UTIMER_NBLOAD,
	UTIMER_FILELOAD,
	UTIMER_FILEFIND,
	UTIMER_OBJVEL,
	UTIMER_OBJEXEC,
	UTIMER_OBJBONEFX,
	UTIMER_WAITFOR,
	UTIMER_OBJBUILDVM,

	UTIMER_OBJCOL,
	UTIMER_OBJDISPRES,
	UTIMER_OBJDISPSINGLE,
	UTIMER_OBJDISPGRP,
	UTIMER_OBJDISPCOL,
	UTIMER_OBJRESOLVE,

	UTIMER_SKELMATS,
	UTIMER_SKELVERTS,

	UTIMER_TRIGGERS,

	UTIMER_PHYSXLOOP,
	UTIMER_PHYSXADD,
	UTIMER_PHYSXIO,
	UTIMER_PHYSXSIM,
	UTIMER_PHYSXSTATIC,
	UTIMER_PHYSXDYNAM,

	UTIMER_WORKBEE_MAIN_THREAD,  // when the main thread does workerbee work
	UTIMER_WORKBEE_THREAD_1,     
	// right now, only one workerbee thread, add more timers as we add more threads

	// mark where the CPU side timers end
	NUM_CPU_UTIMERS = UTIMER_WORKBEE_THREAD_1,

	UTIMER_DRAWINIT,
	UTIMER_DRAWFLIP,
	UTIMER_DRAWCAM,

	UTIMER_RENDERLOOP,

	UTIMER_SCENERENDER,

	UTIMER_SHADMAP,
//	UTIMER_SHADMAPDETAIL,
//	UTIMER_SHADMAPAREA,

	UTIMER_POSTFXRENDER,

	UTIMER_CHANGESHADER,
	UTIMER_CHANGELGROUP,
	UTIMER_UNIFORMDATA,
	UTIMER_UNIFORMDATA_GL, // uniform data set made it's way to opengl
	UTIMER_UNIFORMBLOCK,
	UTIMER_UNIFORMBLOCK_GL, // uniform data set made it's way to opengl
	UTIMER_VERTDATA,  // vert data manually sent to gl (no vbo)
	UTIMER_INDEXDATA,  // index data manually sent to gl (no vio)
	UTIMER_FURLAYERS,

	UTIMER_RLISTRENDER,
	UTIMER_OBJRENDER,
	UTIMER_OBJRENDER_SCENE,
	UTIMER_OBJRENDER_SHMAP,
//	UTIMER_OBJRENDER_SHMAP2,
//	UTIMER_OBJRENDER_SHMAP3,
//	UTIMER_OBJRENDER_SHMAP4,

	UTIMER_OBJRENDER_VERTS,
	UTIMER_OBJRENDER_TRIS,
	UTIMER_OBJRENDER_TRISTRIPS,
	UTIMER_OBJRENDER_QUADS,
	UTIMER_OBJRENDER_DLISTS,

	UTIMER_OBJCULL,
	UTIMER_SETFOG,
	UTIMER_DISPSINGLE,
	UTIMER_DISPGROUP,

	UTIMER_RENDERSORT,

	UTIMER_PEMITTER,
	UTIMER_PEMITTER_CREATE,
	UTIMER_PEMITTER_UPDATE,
	UTIMER_PEMITTER_RENDER,
	UTIMER_PEMITTER_RENDER_MEMCPY,
	UTIMER_PEMITTER_QCOUNT,

	UTIMER_GLCGZ_UPDATE,

	UTIMER_CHARACTERS,

	NUM_UTIMERS
};

//
// SysTimer
//	use for any type of system timer used in debugging
//
typedef struct _systimer {
	int		use_telemetry;
	int64	timer;		// the actual timer
	int64	timeacc;	// time accumulator for starts and stops
	char	name[64];	// timer name
	int64	budget_time;
	int64	budget_count;
	int noautoclear; // set to 1 to have this timer skipped by SysMicroTimerClearAll

	int active; // started and not stopped
	int count; // number of times started, easy way to count the number of times something is done

	int64	baseline_timeacc;
	int64	baseline_count; 

	uns64	tmAccum;
} SysTimer;

EXTERN_C SysTimer SysuTimers[NUM_UTIMERS];


extern int SysInit(void);
extern void SysFinal(void);
extern int SysReset(uns reset);
extern int SysLoop(void);

#if ENABLE_FRAME_SPIKE_TRACKING
	extern void SysCheckFrameSpike(void);
#endif

extern int SysLevelSet(int lev);
extern int SysLevelGet(void);

extern void SysWatchdogEnable(void);
extern void SysWatchdogDisable(void);
extern void SysWatchdogBone(void);

extern uns64 SysMicroSec();
extern uns64 SysMilliSec();


EXTERN_C void SysMicroTimerStart(uns32 timerNum);
EXTERN_C void SysMicroTimerStop(uns32 timerNum);
EXTERN_C uns64 SysMicroTimerGetElapsed(uns32 timerNum);
extern void SysMicroTimerClear(uns32 timerNum);
extern void SysMicroTimerClearAll(void);

// 
// this bit of magic removes dbg timers from production code
// NOTE: VisStudio < VisStudio 2005 does not support variodic macros
#define DISABLE_PERF_COUNTERS 0
#if SYS_BB|SYS_PC
	#if PRODUCTION|DISABLE_PERF_COUNTERS
		// replace calls with null op, eating inputs along the way
		#define PerfCounterStart(...)   
		#define PerfCounterStop(...)   
		#define PerfCounterClear(...)   
		#define PerfCounterClearAll(...)   
		#define PerfCounterIncCount(...)
		#define PerfCounterIncTime(...)
		#define PerfCounterCountAdd(...)
		#define PerfCounterTimeAdd(...)
#else // ~PRODUCTION
		#define PerfCounterStart SysMicroTimerStart 
		#define PerfCounterStop  SysMicroTimerStop 
		#define PerfCounterClear SysMicroTimerClear 
		#define PerfCounterClearAll SysMicroTimerClearAll
		#define PerfCounterIncCount(c) SysuTimers[c].count++;
		#define PerfCounterIncTime(c)  SysuTimers[c].timeacc++;
		#define PerfCounterCountAdd(c,n) SysuTimers[c].count+=(n);
		#define PerfCounterTimeAdd(c,n)  SysuTimers[c].timeacc+=(n);
#endif // PRODUCTION
#endif // SYS_BB|SYS_PC

extern SysGlobals SysG;
extern SysTimer SysuTimers[];

// just some useful macros for timing things like loads, etc.
#define SET_START_TIME(a)   (a = SysMilliSec())
#define PRINT_ELAPSED_TIME(start,elapsed,msg)   elapsed = (SysMilliSec() - start); printf( msg, elapsed );

#define SysLogFile "/pm/g5/dino/pmuser/gamelog.txt"
#define ErrorLogFile "/pm/g5/dino/pmuser/errorlog.txt"
#define AutitFile "/pm/g5/dino/pmuser/audits.txt"
#define AuditDir "/pm/g5/dino/pmuser/aud"
#define AdjustmentDir "/pm/g5/dino/pmuser/adj"

EXTERN_C int SysLog(char *fmt,...);
EXTERN_C int SysLogFrame(void* cam, char* fmt,...);
EXTERN_C int ErrorLog(char *fmt,...);
EXTERN_C int FileLog(char* file, char *fmt,...);
extern int SysLogCheck(void);

#ifdef DEBUG
	#define SysLogDbg SysLog
#else
	#define SysLogDbg(...)
#endif

// output formatted string to debug console
extern void dout( const char* msg, ... );

#define MAIN_RESTART_ARGC_MAX 32
extern int MainRestartArgc;
extern char *MainRestartArgv[MAIN_RESTART_ARGC_MAX];

#define SHIFT_NUM	5
#define SHIFT_BITS	28

extern uns32 sysEnc1(uns32 *dstp,uns32 a);
extern uns32 sysChk1(uns32 opseed, uns32 actcode);
extern uns32 sysDecode1(uns32 u,uns32 a);

enum {
	SYSBUILDINFO_NONE,
	SYSBUILDINFO_HARDWARESTRING,
	SYSBUILDINFO_SOFTWARESTRING,
	SYSBUILDINFO_HARDWARESN,
	SYSBUILDINFO_HARDWARECP,
	SYSBUILDINFO_GAMESN,
};
#define M_SYSBUILDINFO 0xff

extern int SysBuildInfo(uns op,char *buf,int bufsize,uns *idp);

enum {
	SYSGETHARDWAREINFO_NONE,
	SYSGETHARDWAREINFO_SN,
};

enum {
	KEYTYPE_NONE=0,
	KEYTYPE_ROCKEY,
	NUM_KEYTYPES,
};


extern int SysGetHardwareInfo(uns op,char *buf,int bufsize);
extern void SysLoadVersionInfo(void);
extern void SysRestart(void);

int LoadConfigFile( char *file, HashTable *dst ); 

void Aud_Increment_System_Boots( void );
void Aud_Increment_System_DiagEntries( void );
void Aud_Increment_System_MaintReboots( void );
void Aud_Increment_System_NoIOBoardInits( void );
void Aud_Increment_System_IODisconnects( void );
void Aud_Increment_System_IOReconnects( void );
void Aud_Increment_System_Excpetions( void );
void Aud_Increment_System_Traps( void );

///////////////////////////////////////////////////////////////////////////////
// SYSTEM SPECIFIC

// SYS_PC|SYS_BB
#if SYS_PC|SYS_BB

#include <time.h>

extern int xSysInit(void);
extern void xSysFinal(void);
extern int xSysReset(uns reset);

#define FLT_MAX         3.402823466e+38F

#endif // SYS_PC|SYS_BB


// these n2 funcs are too tied into obj.h and res.h
// to be in pmrt_utils.h
char *N2Name(N2 *n,char **s2p);
char *N2ResName(N2 *n,char **s2p);

enum {
	// return codes for the scan func
	N2SCANFUNC_MATCH_AND_RETURN,
	N2SCANFUNC_CONTINUE
};

typedef struct {
	N2 *stack[8];
	int stacki;
	N2 *wrapper;
	ftype3 *func3; // int func(N2ScanRec *nsr,N2 *n) return N2SCANFUNC_
	memu fdata; // function data
	char *s1; // match string s1
	char *s2; // match string s2
} N2ScanRec;

#define M_N2SCAN_VAL       0x0000ffff // used to pass values in
#define M_N2SCAN_SUB       0x00010000 // look for subtype
#define M_N2SCAN_FUNC      0x00020000 // function call
#define M_N2SCAN_NAME      0x00040000 // match the name
#define M_N2SCAN_PRIME     0x00100000 // find prime obj
#define M_N2SCAN_INIT      0x00200000 // this is the first call w/n2sr -- init
#define M_N2SCAN_DONTSCAN  0x00400000 // first and don't search (init only)
#define M_N2SCAN_NODEEPER  0x00800000 // dont scan deeper
#define M_N2SCAN_NOKIDS    M_N2SCAN_NODEEPER // -- depricated
#define M_N2SCAN_WRAP      0x01000000 // wrap until start instead of till list end
#define M_N2SCAN_KIDSONLY  0x02000000 // kids only
#define M_N2SCAN_RESNAME   0x04000000 // find objs by the name of the res they are using...
void* N2Scan(N2ScanRec *n2sr,uns find,void *node);
int N2MoveDead(void *vn,void *deadlist);

#ifndef NO_TOOL
extern int GameReLoadConf(void);
extern void GameResetKeyPressed();
#endif

#endif // ndef SYS_H
