#ifndef SYS_DONGLE_H
#define SYS_DONGLE_H
// sys.h
// Copyright 1998-2004 Play Mechanix Inc. All Rights Reserved
//

#include "g5.h"
#include <dongle/dongle.h>

#define USE_HASP_DONGLE_DATA 1

typedef enum _DongleVersion
{
    DONGLE_VERSION_ALADDIN, DONGLE_VERSION_TOURNEY, DONGLE_VERSION_UNKNOWN
}DongleVersion;

typedef enum _game_dongle_cabinet_type_e	// TO DO - change to reflect real cabinet types when defined for BBHD
{
	GAME_CAB_TYPE_UNKNOWN = 0,
	GAME_CAB_TYPE_UPRIGHT,
	GAME_CAB_TYPE_DELUXE,
	GAME_CAB_TYPE_SUPER_DELUXE,
	GAME_CAB_TYPE_HOMEONLY,
} game_cabinet_type_t;

//**************************************************
//*****************Enc/Decryption+Testing Macro*********
//**************************************************
#if DEBUG
#define GameDecrypt(outData, size, functionName)                                                    \
if(1)                                                                                               \
{                                                                                                   \
    int  retVal   = DONGLE_ERR_DECRYPTFAILED;                                                       \
    char testBytes[6];                                                                              \
    memcpy(testBytes, outData, sizeof(testBytes));                                                  \
    gMasterDongle = AcquireAladdinDongle();															\
    DongleDecryptInline(gMasterDongle, outData, size, retVal);                                      \
    if(retVal != DONGLE_OK || memcmp(testBytes, outData, sizeof(testBytes)) == 0)                   \
    {                                                                                               \
        if(retVal != DONGLE_OK)                                                                     \
        {                                                                                           \
            SysLog("WARNING: Dycryption failed in %s : %d\n", functionName, retVal);                \
            prts_debug("WARNING: Dycryption failed in %s : %d\n", functionName, retVal);            \
        }                                                                                           \
        else                                                                                        \
        {                                                                                           \
            SysLog("WARNING: Dycryption failed. Enc/dec bytes are the same %s \n", functionName);   \
            prts_debug("WARNING: Dycryption failed. Enc/dec bytes are the same %s \n", functionName);   \
        }                                                                                           \
        exit(0);                                                                                    \
    }                                                                                               \
	ReleaseAladdinDongle(gMasterDongle);															\
}

#define GameEncrypt(outData, size, functionName)                                                    \
if(1)                                                                                               \
{                                                                                                   \
    int retVal = DONGLE_ERR_ENCRYPTFAILED;                                                          \
    char testBytes[7];                                                                              \
    memcpy(testBytes, outData, sizeof(testBytes));                                                  \
    gMasterDongle = AcquireAladdinDongle();															\
	DongleEncryptInline(gMasterDongle, outData, size, retVal);                                      \
    if(retVal != DONGLE_OK || memcmp(testBytes, outData, sizeof(testBytes)) == 0)                   \
    {                                                                                               \
        if(retVal != DONGLE_OK)                                                                     \
        {                                                                                           \
            SysLog("WARNING: Encryption failed in %s : %d\n", functionName, retVal);                \
            prts_debug("WARNING: Encryption failed in %s : %d\n", functionName, retVal);                \
        }                                                                                           \
        else                                                                                        \
            SysLog("WARNING: Encryption failed. Enc/dec bytes are the same %s \n", functionName);   \
        exit(0);                                                                                    \
    }                                                                                               \
	ReleaseAladdinDongle(gMasterDongle);															\
}

#else //PRODUCTION
#define GameDecrypt(outData, size, functionName)                                                    \
if(1)                                                                                               \
{                                                                                                   \
    int  retVal   = DONGLE_ERR_ENCRYPTFAILED ;                                                      \
    char testBytes[6];                                                                              \
    memcpy(testBytes, outData, sizeof(testBytes));                                                  \
    gMasterDongle = AcquireAladdinDongle();															\
	DongleDecryptInline(gMasterDongle, outData, size, retVal);                                      \
    if(retVal != DONGLE_OK || memcmp(testBytes, outData, sizeof(testBytes)) == 0)                   \
    {                                                                                               \
        if(retVal != DONGLE_OK)                                                                     \
            SysLog("W: DF %s : %d\n", functionName, retVal);                                        \
        else                                                                                        \
            SysLog("W: DFmem. E/D %s \n", functionName);                                               \
        exit(0);                                                                                    \
    }                                                                                               \
	ReleaseAladdinDongle(gMasterDongle);															\
}                                                                                                   

#define GameEncrypt(outData, size, functionName)                                                    \
if(1)                                                                                               \
{                                                                                                   \
    int retVal = DONGLE_ERR_ENCRYPTFAILED;                                                          \
    char testBytes[7];                                                                              \
    memcpy(testBytes, outData, sizeof(testBytes));                                                  \
    gMasterDongle = AcquireAladdinDongle();															\
	DongleEncryptInline(gMasterDongle, outData, size, retVal);                                      \
    if(retVal != DONGLE_OK || memcmp(testBytes, outData, sizeof(testBytes)) == 0)                   \
    {                                                                                               \
        if(retVal != DONGLE_OK)                                                                     \
            SysLog("W: EF %s : %d\n", functionName, retVal);                                        \
        else                                                                                        \
            SysLog("W: EFmem.  E/D %s \n", functionName);                                              \
        exit(0);                                                                                    \
    }                                                                                               \
	ReleaseAladdinDongle(gMasterDongle);															\
}                                                                                                   

#endif //DEBUG

extern int SysGetSecurityInfo(int keyType, void* pKeyData, unsigned int* pHID, int* pKeyErr);
extern int SysGetDongleHID(unsigned int*pHID);
extern int SysGetCryptoSeed(unsigned char *pSeed, int *SeedSize );
extern int DongleInitialize(int dongleType);
extern int UncompressDecrypt(const char *fileName, const char *path, char **outBuffer, int *outSize, int dongleKeyEncrypt);  //used for files which were compressed and encrypted

extern int setDongleParams(int *aladdinPassSeed, int *fileSecret, int *IVSecret, int *aladdinPassHL, int dongleType);

extern unsigned int			gDongleID;
//extern int					gDongleCountry;

//DongleHandle             gAladdinDongle;
//DongleHandle             gNoiseDongle;

extern DongleHandle             gFeiDongle;
// DongleHandle             gFeiNoiseDongle;
extern DongleHandle             *gMasterDongle;
extern DongleVersion            gDongleVersion;
extern ThreadHandle			gDongleNoiseThread;
extern int gCabinetType;
extern unsigned int gLifetimeCoinCount;
extern unsigned int gCabSerialNum;

#define NUM_ALADDIN_DONGLE_HANDLES 2
extern DongleHandle			gAladdinDongleHandles[NUM_ALADDIN_DONGLE_HANDLES];
extern MutexHandle				gAladdingDongleMutexs[NUM_ALADDIN_DONGLE_HANDLES];

// wrapper funcs to grab donlge mutex and release it
DongleHandle	*AcquireAladdinDongle();
int DongleFeatureCheck(uns32 feature_id);
int ReleaseAladdinDongle(DongleHandle *dh);

extern int *gFileSecret;//[128];
extern int *gIVSecret;//[64];

#define PASS_SEED_SIZE		64
#define FILE_SECRET_SIZE	128
#define IV_SECRET_SIZE		64

// Dongle feature IDs.  Since all the dongle interfaces reside in sys, this
// seemed like the most logical place to put them.

#define DONGLE_FEATURE_REVSHARE		1
#define DONGLE_FEATURE_STANDALONE	2
#define DONGLE_FEATURE_DELUXE		3
#define DONGLE_FEATURE_PROJECTOR	4
#define DONGLE_FEATURE_HOMEONLY		5
#define DONGLE_FEATURE_OS_AUTH		12

//-----------------------------------------------------------------------------

EXTERN_C unsigned int SysGetDongleID();
EXTERN_C int SysGetCountryCodeFromDongle();
EXTERN_C int SysIsGameHomeVersion();


#define PROGRAM_ENCRYPT_MOD 5
int Dongle_EncryptPrograms();

#endif // ndef SYS_DONGLE_H
