// -------------------------------------------------------------------
// theorafunc.h - manny najera (11.28.06)

#ifndef _THEORAFUNC_H_
#define _THEORAFUNC_H_

// -------------------------------------------------------------------

#include "g5.h"


// -------------------------------------------------------------------

#define THEORAFUNC_SYNCTIME		1
#define THEORAFUNC_CATCHUP		1

// -------------------------------------------------------------------

enum {

	THEORAFUNC_NOALPHA = 0,
	THEORAFUNC_ALPHA,
	THEORAFUNC_ALPHA_STACKED,
};

enum {
	TF_ENCODE_NOALPHA = 0,
	TF_ENCODE_ALPHA,
};

typedef void (td_callback)(void);

// -------------------------------------------------------------------
// - lookup tables for YUV->RGB conversion

static int sAdjCrr[256];
static int sAdjCrg[256];
static int sAdjCbg[256];
static int sAdjCbb[256];
static int sAdjY[256];
static unsigned char sClampBuff[1024];
static unsigned char *sClamp = sClampBuff + 384;

// -------------------------------------------------------------------
// - lookup tables for RGB->YUV conversion

static int tableRy[256];
static int tableGy[256];
static int tableBy[256];

static int tableRu[256];
static int tableGu[256];
static int tableBu[256];

static int tableRv[256];
static int tableGv[256];
static int tableBv[256];

// -------------------------------------------------------------------
// - video data type

typedef struct {

	ogg_sync_state		oggSyncState;
	ogg_page			oggPage;
	ogg_stream_state	oggStreamState;
	theora_info			theoraInfo;
	theora_comment		theoraComment;
	theora_state		theoraState;

	int theora_p;
	int64 vidStartTicks;
	int x, y, w, h;
	unsigned int glTex;
	int alpha;
	int using_shader;

	unsigned int yuvTex[3];
	unsigned char *yuvBuf[3];

	int lastFrame;

	unsigned int pbo[3];

	int pixelSize;

	int isPlaying;
	int ignoreTime;
	int loop;
	int killWhenDone;

	td_callback *onDoneCallback;

	unsigned char *data;
	int dataPos, dataSize;

	char *linkedObject[128];

	unsigned char *pixelBuff;

	int *offset_x;
	int *offset_y;
	int numOffsets;

} theorafunc_data_t;

// -------------------------------------------------------------------
// - encoding data type

typedef struct {

	ogg_stream_state to; 
	ogg_stream_state vo; 
	ogg_page         og;
	ogg_packet       op;

	theora_state     td;
	theora_info      ti;
	theora_comment   tc;

	FILE *outfile;

	unsigned char *yuvFrame;
	unsigned char *screen, *bScreen;

	int numEncodedFrames;
	int numEncodedBytes;

	int w, h;
	int alphaStack;

} theorafunc_encode_data_t;

// -------------------------------------------------------------------

extern GLuint textureArray[32];

// -------------------------------------------------------------------

extern void theorafunc_buildTables(void);
extern theorafunc_data_t *theorafunc_allocate(void);

//extern void theorafunc_init(theorafunc_data_t *td, char *filename, int glTex);
extern void theorafunc_init(theorafunc_data_t *td, unsigned char *data, int dataSize, int glTex);

extern void theorafunc_openVideo(theorafunc_data_t *td);
extern int theorafunc_readyVideo(theorafunc_data_t *td);
extern void theorafunc_setupYUV(theorafunc_data_t *td);

//extern int theorafunc_bufferData(FILE *in, ogg_sync_state *oy);
extern int theorafunc_bufferData(theorafunc_data_t *td);
extern int theorafunc_queuePage(theorafunc_data_t *td);

extern void theorafunc_videoWrite(theorafunc_data_t *td);
extern void theorafunc_videoWriteRGB(theorafunc_data_t *td);
extern void theorafunc_videoWriteRGBA(theorafunc_data_t *td);
extern void theorafunc_videoWriteStackedRGBA(theorafunc_data_t *td);
extern void theorafunc_videoWriteYUV(theorafunc_data_t *td);

extern void theorafunc_restart(theorafunc_data_t *td);
extern void theorafunc_unload(theorafunc_data_t *td);

extern int64 theorafunc_getTime(void);

extern void encode_unload(theorafunc_encode_data_t *td);
extern void encode_init(theorafunc_encode_data_t *td, char *filename
				, int w, int h, int bit_rate, int quality, int alphaStack);
extern void encode_buildTables(void);
extern void encode_frame(theorafunc_encode_data_t *td, int bpp, int final);
extern void copyFlippedScreen(unsigned char *src, unsigned char *dst
						, int w, int h, int bpp, int alphaStack);

extern void theorafunc_animateG3(Obj *obj);
extern int theorafunc_getFrame(char *name);
extern int disableMipMapping(char *name);
extern theorafunc_data_t *getVideoFromObject(char *name, ObjDispRes *odr );

extern int theorafunc_loadVideo(char *name, ObjDispRes *odr);
extern void theorafunc_unloadVideo(char *name, ObjDispRes *odr);

extern int theorafunc_getBytesAlloced(void);

extern void theorafunc_vidalink(char *vidname, char *objname);
extern void theorafunc_vidstop(char *vidname, ObjDispRes *odr);

void theorafunc_setupShader(void);

extern void theorafunc_shaderOn(int texId);
extern void theorafunc_shaderOff(void);

extern void theorafunc_getOffsets(int texId, int *ox, int *oy, int *ow, int *oh);

void theorafunc_readTexLink(int yt, int *ut, int *vt, int *a);

// -------------------------------------------------------------------

#endif
