// -------------------------------------------------------------------
// tool2.h
//
// Play Mechanix - Video Game System
// Copyright (c) 2004 Play Mechanix Inc. All Rights Reserved
#ifndef _TOOL2_H_
#define _TOOL2_H_

int tool2_gui_visible();

#ifndef NO_TOOL


// -------------------------------------------------------------------

/*
#include "obj.h"
#include "res.h"
#include "core_scene.h"
#include "postfx.h"
*/

// -------------------------------------------------------------------


// - size of memory allocated to the master gui list

#define T2GUI_MEM_SIZE		2048 * 1024

#define T2GUI_MAX_CHILDREN	2048 + 1024 + 256


#define T2SCENE_EDIT_PEMITTER_LIST_UPDATE_DELAY 32

#define T2GUI_MAX_SELECTLISTBOX_CHARS	40

// - gui types

enum {
	T2GUI_NULL = 0,
	T2GUI_WINDOW,
	T2GUI_BUTTON,
	T2GUI_TEXTBOX,
	T2GUI_TEXTINPUT,
	T2GUI_TEXTLABEL,
	T2GUI_CRECT,
	T2GUI_SLIDER,
};

// - action types

enum {
	T2GUI_ACTION_MINUS = 0,
	T2GUI_ACTION_PLUS,
	T2GUI_ACTION_CLOSE,
	T2GUI_ACTION_SENDTEXT,
};

enum {
	T2GUI_KEY_UP = 1,
	T2GUI_KEY_DOWN,
	T2GUI_KEY_LEFT,
	T2GUI_KEY_RIGHT,
	T2GUI_KEY_ALT,
	T2GUI_KEY_SHIFT,
};

// - text justification

enum {
	T2GUI_TEXT_JUST_CENTERED = 0,
	T2GUI_TEXT_JUST_LEFT,
	T2GUI_TEXT_JUST_RIGHT,
};

// - object properties

#define T2GUI_PROP_CLOSABLE		(1 << 0)
#define T2GUI_PROP_MINIMIZABLE	(1 << 1)
#define T2GUI_PROP_MOVEABLE		(1 << 2)
#define T2GUI_PROP_SELECTABLE	(1 << 3)
#define T2GUI_PROP_RESIZABLE	(1 << 4)

typedef void (t2_callback)(void *, void *);
typedef void (td_menucallback)(void);

// **** BASE TYPE ****

typedef struct {

	int type;			// - what object type are we?
	int id;				// - unique object id

	t2_callback *fptr;	// - pointer to a function to run every frame
	void *f_arg;		// - the argument to pass to the above function

	t2_callback *kill_fptr;

	int is_child;		// - are we someone's child?
	int parent_id;		// - object id of the parent (if any)
	int min_immune;		// - immunity to minimizing
	int death;			// - will this object be killed on the next loop?
	void *next;			// - pointer to memory immediately after us
	void *misc;			// - this is a hack: a general-purpose pointer that can
						//   be used for whatever...
} T2GUI_BASE_TYPE;

// **** WINDOW TYPE ****

typedef struct {

	int x, y, w, h;		// - screen position, dimensions
	char text[64];		// - window title

	int no_title;		// - do we have no title text (don't draw a title bar)

	int no_outline;		// - do we want to not display an outline?

	int no_fill;		// - restrict rendering the window background

	int mouse_over;		// - is the mouse over us?
	int mouse_btn_out;	// - did the button press originate outside the window?

	int dragging;		// - are we being dragged?
	int dx, dy;			// - the mouse offset at the start of the drag motion

	int bdragging;		// - are we being bottom-dragged?
	int bdraggable;		// - are we bottom-draggable?

	int sdragging;		// - are we dragging the scroll bar?

	int minimized;		// - are we minimized?
	int minimize_clicks;
	int minimize_prev_lmb;
	uns64 minimize_click_time;  // when was the last time top bar was clicked (used for double click minimizing)

	int child_max_h;	// - the maximum height reached by a child object
	int scrollbar;		// - do we need a scrollbar?
	int scroll_pos;		// - the position of the scroll button
	int scroll_h;		// - the height of the scroll button
	int scroll_total;	// - the total distance of the scroll window
	int scroll_offset;

	// - a window can have a number of child objects
	// TODO: lets use the List object for this!!!11one!
	// NOTE: tried this.  the List object type gets very, very slow while iterating
	// through huge (2000+) child lists.  a very large array wastes memory for every open
	// window, but speed is what we need with the tool!

	int num_children;
	int	child_id[T2GUI_MAX_CHILDREN];

	int kill_on_mouseout;	// - kill the window when the mouse goes away?

	int restrict_scrollbar_check;	// - a scrollbar check is a O(n^2) operation that the
									//   tool will try to do all the time... if we let it
	int autoresizeW;
	int autoresizeH;

} T2GUI_WINDOW_TYPE;

// **** BUTTON TYPE ****

typedef struct {

	int x, y, w, h;		// - screen position, dimensions
	int mouse_over;		// - is the mouse over us?
	int pressed;		// - is the button pressed?
	int mouse_btn_out;	// - did the button press originate outside the button?

	float rgb[3];

	char text[64];		// - title
	int text_just;		// - justifcation (0=centered, 1=left, 2=right)

	int disabled;		// - has this button been deactivated?

	int toggle;			// - is this button "toggleable"?
	int toggleState;	// - (0 = off, 1 = on)
	int toggleGroup;	// - sibling buttons of the same group toggle eachother

	// - ResScript command that is run when the button is pressed

	// char res_script[64];

	// - function pointer for code that is called when pressed

	t2_callback *fptr;
	void *f_arg;		// - argument to pass to the callback

	int fptr_warning;	// - if this flag is set, a warning message is displayed
	//char fptr_warning_text[32];

} T2GUI_BUTTON_TYPE;

// **** TEXTBOX TYPE ****

typedef struct {

	int x, y, w, h;

	char *buf;
	int bufsize;

	int click_state;	// - is the mouse clicked on us?

	int highlight;		// - which line of text to highlight
	int prev_highlight;

	int disable_input;

	int longest_str_len;

} T2GUI_TEXTBOX_TYPE;

// **** TEXT INPUT TYPE ****

typedef struct {

	int x, y, w, h;

	char *buf;
	int bufsize;

	int bufindex;

	int mouse_over;
	int mouse_click;
	int has_key_focus;

	int offset;

	int return_btn_id;	// - the gui id of the button to activate when the user hits enter

	int multi_line;		// 1 if newlines are allowed in this field
	int autoresize;		// 1 if textinput will automatically expand to accomodate text (should be 1 for multi-line)

	int minSizeX;
	int minSizeY;

} T2GUI_TEXTINPUT_TYPE;

// **** TEXT LABEL TYPE ****

typedef struct {

	int x, y;
	char *buf;
	int bufsize;

} T2GUI_TEXTLABEL_TYPE;

// **** COLORED RECTANGLE TYPE ****

typedef struct {
	int x, y, w, h;
	float r, g, b, a;
	int use_outline;
} T2GUI_CRECT_TYPE;

// **** SLIDER BOX TYPE ****

typedef struct {
	int x, y, w, h;
	int slider_w, slider_pos;
	float val_start, val_end;
	float val;
	int mouse_over, mouse_over_slider;
	int mouse_x;
	int using_slider;
	int growable;
} T2GUI_SLIDER_TYPE;

// - although the gui objects are implemented as a simple singly-linked
//   list, a binary tree is maintained which pairs object ids to their
//   corresponding pointers

typedef struct {
	int id;
	T2GUI_BASE_TYPE *ptr;
} T2GUI_LISTDATA_TYPE;

// *******************************************************************
// USEFUL MACROS
// *******************************************************************

// - retrieve the base object of a gui object
#define T2_GETBASE(o) ((T2GUI_BASE_TYPE*)&((T2GUI_BASE_TYPE*)o)[-1])

// - retrieval macros for various object types from the base type
#define T2_GETWINDOW(o) ((T2GUI_WINDOW_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETBUTTON(o) ((T2GUI_BUTTON_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETTEXTBOX(o) ((T2GUI_TEXTBOX_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETTEXTINPUT(o) ((T2GUI_TEXTINPUT_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETTEXTLABEL(o) ((T2GUI_TEXTLABEL_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETCRECT(o) ((T2GUI_CRECT_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])
#define T2_GETSLIDER(o) ((T2GUI_SLIDER_TYPE*)&((T2GUI_BASE_TYPE*)o)[1])

// *******************************************************************
// TOP BAR MENU TYPES
// *******************************************************************

#define T2_MENU_ITEM_MAX	32

typedef struct {
	char text[32];
	td_menucallback *fptr;
	int id;
} T2_MENU_ITEM_TYPE;

typedef struct {
	char text[32];
	int x, y;
	int id;
	T2_MENU_ITEM_TYPE item[T2_MENU_ITEM_MAX];
} T2_MENU_TYPE;

// -------------------------------------------------------------------
// [frameNum][animName][snap][loop][sync][blend][speed][-][+][X]
// [0]       [1]       [2]   [3]   [4]   [5]    [6]	   [7][8][9]

typedef struct {
	char animName[64];
	int frameNum;
	float speed;
	int snap, sync, loop, blend;
	int gui_id[10];					// - the ids of the corresponding gui elements
	void *parent;					// - a link back to the parent swatch object
	int id;							// - each swatch key has a unique global id (so the gui can find it!)
} T2_DATA_SWATCHKEY_TYPE;

typedef struct {
	char name[32];
	char splineName[32];
	int numKeys;
	T2_DATA_SWATCHKEY_TYPE *data;
	int gui_id;						// - the id of the spline watch edit window
} T2_DATA_SWATCH_TYPE;

// *******************************************************************
// PROGRAM INSTRUCTION DATA TYPES

enum {
	T2_DPIT_BUTTON = 0,
	T2_DPIT_NAME,
	T2_DPIT_BAR,
	T2_DPIT_NUM,
};

typedef struct {
	char name[32];
	List glist;		// <- T2_DATA_GUI_TYPE
	int gui_id[T2_DPIT_NUM];
	void *parent;
	int id;
} T2_DATA_PROGRAM_INSTRUCTION_TYPE;

// - event actions are strings that are associated with a program instruction

typedef struct {
	char name[32];			// - the action name
	int gui_id_action;		// - gui id of the text input box containing the action name
	T2_DATA_PROGRAM_INSTRUCTION_TYPE ins;
} T2_DATA_PROGRAM_EVENTACTION_TYPE;

typedef struct {
	char param_name[64];
	char value[64];
	int gui_id_param;
	int gui_id_value;
} T2_DATA_PROGRAM_PARAMETER_TYPE;

typedef struct {
	
	char name[64];			// - program name
	char sm_name[64];		// - state machine name
	char filename[128];

	int numInstructions;
	int insert_id;			// - if nonzero, the id of the instruction to insert a new instruction after
	int add_event;			// - if nonzero, any new instruction will be added as an event action
	int gui_id[3];			// - (0: grandparent window, 1: program name (text input), 2: state machine (text label)
	int gui_id_window;		// - gui id of the instruction window
	int gui_id_eawindow;	// - gui id of the event action window
	int gui_id_pwindow;

	int gui_id_btn_instructions;
	int gui_id_btn_eventactions;
	int gui_id_btn_parameters;
	int gui_max_y;
	T2_DATA_PROGRAM_INSTRUCTION_TYPE *data;

	List eventactions;		// - event actions
	List parameters;
	List params;			// - list of T2_DATA_GUI_TYPES that correspond to each parameter

} T2_DATA_PROGRAM_TYPE;

typedef struct
{
	char name[128];
	char filename[128];
	HashTable Tool_Layouts; // - Tool_Layouts Hash Table
	HashTable Tool_Items; // - Tool_Items Hash Table
	HashTable Tool_UpdateFuncs; // - Tool_UpdateFuncs Hash Table
	HashTable Tool_Programs; // - Tool_Programs Hash Table
	int gui_id_parent;		// - gui id of the outer main layout window
	int gui_id_window;		// - gui id of the inner main layout window
	HashTable layoutsShowGroup;
	HashTable layoutOIDs;
	int dirtyLayouts;		// - used to tell the layout window to refresh
} T2_DATA_LAYOUT_TYPE;

#define T2GUI_INSTRUCTION_YPIXELS	20

typedef struct {

	char name[32];		// - variable name displayed in the gui
	char origName[64];	// - original variable name (with "e_cb_" style encoding entact)
	char type[32];		// - variable type (int, float, char, ect.)
	int gtype;			// - gui object type to use (textbox, button, ect.)
	int gui_id[2];		// - the gui ids of the exact objects that represents us (0: label, 1: data)
	int group_id;		// - if the object is in a group (checkbox groups only for now), this is the group id

	// - the following is the actual data storage

	float fval;
	int ival;
	char *buf;
	int bufSize;

} T2_DATA_GUI_TYPE;


// -------------------------------------------------------------------
// SCENE DATA TYPES

// these have all been moved to core_scene.h


typedef struct {
	char name[32];
	List indices;
	List indices1P2P;
	List indices1P;
	List indices2P;
} T2_DATA_GROUPLIST_TYPE;


typedef struct {
	int on;
	int gui_id;
} T2_DATA_BTN_TRACK_TYPE;

typedef struct {
	int oid;
} T2_DATA_LAYOUT_OID_TYPE;


typedef struct {
	char name[32];
} T2_DATA_LAYOUT_NAME;


typedef struct {

	char name[64];
	int index;

} T2_DATA_SFX_TYPE;

typedef struct {

	char name[64];
	int index;

} T2_DATA_FXG_TYPE;

typedef struct {

	char name[64];
	int index;

} T2_DATA_EMITTER_TYPE;

typedef struct {
	char group[64];
	char anim[64];
	float frame;
	char sfx[64];

	int volume;
	int useDist;

	char joint[64];
	float offset[3];
	int gui_id_jointbtn;
	int gui_id_offsetlabel;
	int gui_id_distbtn;

	int type;
	int global_index;

	void *sobj;
} T2_DATA_ANISFX_NODE_TYPE;

// -------------------------------------------------------------------

typedef struct {
	void *ptr;
	void *ptr2;
	int arg[16];
	t2_callback *fptr;
} T2_F_ARG_TYPE;

// -------------------------------------------------------------------


enum {
	T2_SELTYPE_SOUNDFX = 0,
	T2_SELTYPE_IMAGE,
	T2_SELTYPE_FXG,
	T2_SELTYPE_JOINT,
	T2_SELTYPE_NUM,
};

#define T2_PLAYER1_ACTIVE		0x00000001
#define T2_PLAYER2_ACTIVE		0x00000002
#define T2_PLAYER3_ACTIVE		0x00000004
#define T2_PLAYER4_ACTIVE		0x00000008


// -------------------------------------------------------------------

// - from tool2.c

extern Obj* tool2_curSelObj;
extern int tool2_curSelObjOID;

// - from tool2_data.c
extern List *tool2_data_sfx_open_bank_list;
extern List *tool2_data_fxg_open_group_list;
extern List *tool2_data_emitter_open_group_list;
extern List *tool2_gui_top_instructions;

int tool2_start(void);
int tool2_infoLog(char *buf, ...);
int tool2_menu_loop(void);
int tool2_infoLogDisp(void);
//void tool2_cb_camera_drag(int type, float x, float y, float z);
void tool2_gui_select_object(int mx, int my, int type);
int tool2_auxdraw(void);
void tool2_scene_run(void);


void tool2_gui_objaxis_mousecheck(float mx, float my, int rmb, int lmb, int mmb);

char *tool2_util_strip_path(char *name);

void t2cb_restore(void *ptr, void *ptr2);					// - wipeout, then reload previously loaded files

void t2cb_errorwin_create(void *ptr, void *ptr2);

void t2cb_create_dispgroup_create(void *ptr, void *ptr2);	// - create the "create display group" tool
void t2cb_create_dispgroup_btn(void *ptr, void *ptr2);		// - the "create" button

void t2cb_actorlist_create(void *ptr, void *ptr2);
void t2cb_actorlist_btn(void *ptr, void *ptr2);

void t2cb_featurewin_create(void *ptr, void *ptr2);			// - create the "anim, shader, delete" tool
void t2cb_featurewin_anim(void *ptr, void *ptr2);			// - "anim" button
void t2cb_featurewin_shader(void *ptr, void *ptr2);			// - "shader" button
void t2cb_featurewin_lightgroup(void *ptr, void *ptr2);		// - "l.group" button
void t2cb_featurewin_lightgroup_update(void *ptr, void *ptr2);
void t2cb_featurewin_delete(void *ptr, void *ptr2);			// - "delete" button

void t2cb_filelist_create(void *ptr, void *ptr2);

void t2cb_playanim_create(void *ptr, void *ptr2);			// - create the "play animation" tool
void t2cb_playanim_play(void *ptr, void *ptr2);				// - "play" button
void t2cb_playanim_stop(void *ptr, void *ptr2);				// - "stop" button
void t2cb_playanim_create_func(void *ptr, void *ptr2);

void t2cb_selanim_create(void *ptr, void *ptr2);			// - open the "animation select" (not play!) tool

void t2cb_seldisp2_create(void *ptr, void *ptr2);
void t2cb_seldisp2_create_func(void *ptr, void *ptr2);

void t2cb_selectlistbox_create( char *name, T2GUI_WINDOW_TYPE **pbigwin, T2GUI_WINDOW_TYPE **pwptr, T2GUI_TEXTBOX_TYPE **ptbptr, T2GUI_BUTTON_TYPE **pbtn, T2GUI_TEXTINPUT_TYPE **ptiptr, float width, float height );

void t2cb_selfxgroup_create(void *ptr, void *ptr2);
void t2cb_selfxgroup_create_func(void *ptr, void *ptr2);

void t2cb_selobjgroup_create(void *ptr, void *ptr2);
void t2cb_selobjgroup_create_func(void *ptr, void *ptr2);

void t2cb_selcol_create(void *ptr, void *ptr2);
void t2cb_selcol_create_func(void *ptr, void *ptr2);

void t2cb_seldispgroup_create(void *ptr, void *ptr2);		// - open the "select display group" tool
void t2cb_seldispgroup_select(void *ptr, void *ptr2);		// - 

void t2cb_selshader_create(void *ptr, void *ptr2);
void t2cb_selshader_select(void *ptr, void *ptr2);

void t2cb_selspline_create(void *ptr, void *ptr2);
void t2cb_selspline_create_func(void *ptr, void *ptr2);

void t2cb_copyfxg_create(void *ptr, void *ptr2);

void t2cb_seltrigger_create(void *ptr, void *ptr2);
void t2cb_seltrigger_select(void *ptr, void *ptr2);

void t2cb_selemitter_create(void *ptr, void *ptr2);
void t2cb_selemitter_select(void *ptr, void *ptr2);

void t2cb_selelement_create(void *ptr, void *ptr2, int type, void *scanptr);
void t2cb_selelement_select(void *ptr, void *ptr2);
void t2cb_selelement_sendtogui(void *ptr, void *ptr2);

void t2cb_selsfx_create(void *ptr, void *ptr2);
void t2cb_selsfx_select(void *ptr, void *ptr2);
void t2cb_selsfx_sendtogui(void *ptr, void *ptr2);

void t2cb_selobjtype_create(void *ptr, void *ptr2);
void t2cb_selobjtype_dispgroup(void *ptr, void *ptr2);
void t2cb_selobjtype_disp2(void *ptr, void *ptr2);
void t2cb_selobjtype_fxgroup(void *ptr, void *ptr2);
void t2cb_selobjtype_objgroup(void *ptr, void *ptr2);

void t2cb_selprogparamater_create(void *ptr, void *ptr2);
void t2cb_selprogparamater_create_func(void *ptr, void *ptr2);

void t2cb_state_list_create(void *ptr, void *ptr2);			// - create the program instruction list tool
void t2cb_state_list_select(void *ptr, void *ptr2);			// - called from a button press in the instruction list tool
void t2cb_state_list_update(void *ptr, void *ptr2);			// - called every frame on the window that contains the buttons

void t2cb_swatch_test_create(void);							// - create the spline watch edit tool
void t2cb_swatch_test_add(void *ptr, void *ptr2);			// - add a spline watch key
void t2cb_swatch_test_delete(void *ptr, void *ptr2);		// - delete a key
void t2cb_swatch_test_update(void *ptr, void *ptr2);		// - run every frame, syncs gui data to the spline watch
void t2cb_swatch_test_pickanim(void *ptr, void *ptr2);		// - open the animation select tool
void t2cb_swatch_test_pickanim_btn(void *ptr, void *ptr2);	// - place the selected animation in the key
void t2cb_swatch_test_moveup(void *ptr, void *ptr2);		// - move a key up (later in) the list
void t2cb_swatch_test_movedown(void *ptr, void *ptr2);		// - move a key down (earlier in) the list

void t2cb_splinesel_create(void *ptr, void *ptr2);			// - create a spline select tool
void t2cb_splinesel_select(void *ptr, void *ptr2);			// - select a spline and exit the tool

void t2cb_statemachinesel_create(void *ptr, void *ptr2);	// - create a state machine select tool
void t2cb_statemachinesel_select(void *ptr, void *ptr2);	// - select a state machine and exit the tool

void t2cb_warning_create(void *ptr, void *ptr2);			// - create a warning dialog box
void t2cb_warning_yes(void *ptr, void *ptr2);				// - happens when you press "yes"
void t2cb_warning_no(void *ptr, void *ptr2);				// - happens when you press "no"

void t2cb_program_edit_create(void *ptr, void *ptr2);			// - create the program edit tool
void t2cb_program_edit_addinstruction(void *ptr, void *ptr2);	// - bring up the instruction list tool to add an instruction
void t2cb_program_edit_addeventaction(void *ptr, void *ptr2);	// - same as above, but adds the instruction as an event action
void t2cb_program_edit_addparameter(void *ptr, void *ptr2);		//
void t2cb_program_edit_update(void *ptr, void *ptr2);			// - update function
void t2cb_program_edit_clear(void *ptr, void *ptr2);			// - "clear" button
void t2cb_program_edit_load(void *ptr, void *ptr2);				// - "load" button
void t2cb_program_edit_save(void *ptr, void *ptr2);				// - "save" button
void t2cb_program_edit_saveas(void *ptr, void *ptr2);			// - "save as" button
void t2cb_program_edit_saveas_btn(void *ptr, void *ptr2);
void t2cb_program_edit_miniwin_create(void *ptr, void *ptr2);	// - create the program edit mini action window (up, down, delete, insert)
void t2cb_program_edit_miniwin_btnup(void *ptr, void *ptr2);	// - happens when you press "+" to move the instruction up the list
void t2cb_program_edit_miniwin_btndown(void *ptr, void *ptr2);	// - happens when you press "-" to move the instruction down the list
void t2cb_program_edit_miniwin_delete(void *ptr, void *ptr2);	// - happens when you press "X" to delete the instruction
void t2cb_program_edit_miniwin_insert(void *ptr, void *ptr2);	// - happens when you press "insert" to add a new instruction

void t2cb_program_edit_ea_miniwin_create(void *ptr, void *ptr2);
void t2cb_program_edit_ea_miniwin_delete(void *ptr, void *ptr2);
void t2cb_program_edit_param_miniwin_create(void *ptr, void *ptr2);
void t2cb_program_edit_param_miniwin_delete(void *ptr, void *ptr2);

void t2cb_program_list_create(void *ptr, void *ptr2);		// - create the program select tool
void t2cb_program_list_select(void *ptr, void *ptr2);		// - happens when you select a program from the list
void t2cb_program_list_update(void *ptr, void *ptr2);

void t2cb_newprogram_create(void *ptr, void *ptr2);			// - new program creation tool
void t2cb_newprogram_create_btn(void *ptr, void *ptr2);		//

void t2cb_newprogram_tmpl_create(void *ptr, void *ptr2);	// - new program from template tool
void t2cb_newprogram_tmpl_create_btn(void *ptr, void *ptr2);//

void t2cb_newscene_create(void *ptr, void *ptr2);			// - new scene creation tool
void t2cb_newscene_create_btn(void *ptr, void *ptr2);		// - the "create" button

void t2cb_scene_list_create(void *ptr, void *ptr2);			// - scene selection tool
void t2cb_scene_list_select(void *ptr, void *ptr2);			//

void t2cb_particles_edit_create(void *ptr, void *ptr2);		// particles and fx groups edit tool

void tool2_refreshLayouts(void);

void t2cb_motionBlur_create(void);

void t2cb_reload_gameconf(void);
void t2cb_rebuild_scene_hashtables(void);
void t2cb_reload_sounds(void);
void t2cb_reload_cpmgui(void);
void t2cb_reload_translations(void);
void t2cb_count_translations(void);
void t2cb_set_player_mode(void);
void t2cb_set_player_mode_bnt_hit(void *ptr, void *ptr2);
void t2cb_set_wireframe_mode(void);
void t2cb_show_missing_textures(void);

void t2cb_debugwin_create(void);
void t2cb_debugwin_update(void *ptr, void *ptr2);
void t2cb_debugwin_cb(void *ptr, void *ptr2);

void t2cb_lightswin_create(void);
void t2cb_lightswin_update(void *ptr, void *ptr2);
void t2cb_lightswin_cb(void *ptr, void *ptr2);

void t2cb_waterwin_create(void);
void t2cb_waterwin_update(void *ptr, void *ptr2);
void t2cb_waterwin_load_create(void *ptr, void *ptr2);
void t2cb_waterwin_select(void *ptr, void *ptr2);
void t2cb_waterwin_save(void *ptr, void *ptr2);

void t2cb_fogwin_create(void);
void t2cb_fogwin_update(void *ptr, void *ptr2);
void t2cb_fogwin_load_create(void *ptr, void *ptr2);
void t2cb_fogwin_select(void *ptr, void *ptr2);
void t2cb_fogwin_save(void *ptr, void *ptr2);

void t2cb_furwin_create(void);
void t2cb_furwin_update(void *ptr, void *ptr2);
void t2cb_furwin_load_create(void *ptr, void *ptr2);
void t2cb_furwin_select(void *ptr, void *ptr2);
void t2cb_furwin_save(void *ptr, void *ptr2);
void t2cb_furwin_cb(void *ptr, void *ptr2);

void t2cb_carwin_create(void);
void t2cb_carwin_update(void *ptr, void *ptr2);
void t2cb_carwin_load_create(void *ptr, void *ptr2);
void t2cb_carwin_select(void *ptr, void *ptr2);
void t2cb_carwin_save(void *ptr, void *ptr2);
void t2cb_carwin_cb(void *ptr, void *ptr2);


void t2cb_ccwin_create(void);
void t2cb_ccwin_update(void *ptr, void *ptr2);
void t2cb_ccwin_cb(void *ptr, void *ptr2);
void tool2_util_ccwin_setslider(int gui_id, PostFxSettings *pfx, int index);

int tool2_util_gameplay_toggledifficulty(int dir);

void tool2_runfunclist_add(void *func, void *arg0, void *arg1, void *arg2);
void tool2_runfunclist_execute(void);

void t2cb_ccwin_save(void *ptr, void *ptr2);
void t2cb_ccwin_load_create(void *ptr, void *ptr2);
void t2cb_ccwin_load_create_func(void *ptr, void *ptr2);
void t2cb_ccwin_delete(void *ptr, void *ptr2);

void t2cb_texoverride_create(void *ptr, void *ptr2);
void t2cb_texoverride_update(void *ptr, void *ptr2);

void t2cb_recordmovie_create(void *ptr, void *ptr2);
void t2cb_recordmovie_begin(void *ptr, void *ptr2);
void t2cb_recordmovie_end(void *ptr, void *ptr2);
void tool2_recording_getfilename(char **ptr);

// - from tool2_gui.c

int tool2_gui_meminit(void);
void tool2_gui_toggle_display();
int tool2_gui_func(Obj *obj);

int t2gl_window_scissor_reset(void);
int t2gl_window_scissor_add(int x, int y, int w, int h);
int t2gl_window_scissor_pop(void);
int t2gl_window_scissor_get(int *x, int *y, int *w, int *h);

int tool2_util_objsortedlist_build(void *ptr, void *ptr2);
int tool2_util_objsortedlist_search(int id, void *obj);

int tool2_button_windowkill(void *ptr, void *ptr2);

int tool2_gui_add(int type, void *obj);
int tool2_gui_promote(void *obj);
int tool2_gui_delete(void *obj);
int tool2_gui_delete_add(void *obj);
int tool2_gui_consolitate(void);
int tool2_gui_parent(void *pobj, void *cobj);
int tool2_gui_draw(void);
int tool2_gui_draw_type(void *ptr);
int tool2_gui_action(int action, T2GUI_BASE_TYPE *obj, T2GUI_BASE_TYPE *src);
int tool2_gui_action_loop(void);

int tool2_gui_keyfocus_tab(void);
int tool2_gui_keyfocus_reverse_tab(void);

int tool2_textinput_click(T2GUI_TEXTINPUT_TYPE *tiptr, int clicktype);

int tool2_util_get_parent(T2GUI_BASE_TYPE *bptr, T2GUI_BASE_TYPE **parent);
int tool2_util_kill_parent(T2GUI_BASE_TYPE *bptr);
int tool2_util_kill_children(T2GUI_BASE_TYPE *bptr);
int tool2_util_fix_parent_link(void *base, void *oldptr, void *newptr);
int tool2_util_remove_child(void *obj);
int tool2_util_get_nth_obj(int num, void *obj);
int tool2_util_get_size(void *obj);
int tool2_util_obj_audit(void);
int tool2_util_clear_input(void *src);
int tool2_util_create_window(void *obj, int x, int y, int w, int h, char *text, int flags);
int tool2_util_create_button(T2GUI_BUTTON_TYPE **btn, int x, int y, int w, int h, char *title, int text_just, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_create_textinput(T2GUI_TEXTINPUT_TYPE **tiptr, int x, int y, int w, int h, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_create_multiline_textinput(T2GUI_WINDOW_TYPE **wparent, T2GUI_TEXTINPUT_TYPE **tiptr, int x, int y, int w, int h, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_create_textlabel(T2GUI_TEXTLABEL_TYPE **tlptr, int x, int y, char *text, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_create_crect(T2GUI_CRECT_TYPE **crptr, int x, int y, int w, int h, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_create_slider(T2GUI_SLIDER_TYPE **slptr, int x, int y, int w, int h, T2GUI_WINDOW_TYPE *wptr);
int tool2_util_set_textlabel(T2GUI_TEXTLABEL_TYPE *tlptr, char *text);
int tool2_util_textinput_setint(void *obj, int val);
int tool2_util_textinput_getint(void *obj, int *val);
int tool2_util_textinput_setfloat(void *obj, float val);
int tool2_util_textinput_getfloat(void *obj, float *val);
int tool2_util_textinput_settext(void *obj, char *buf);
int tool2_util_tbox_add(void *obj, char *text);
int tool2_util_tbox_set(void *obj, char *text);
int tool2_util_tbox_clear(void *obj);
int tool2_util_tbox_countlines(void *obj);
int tool2_util_tbox_alphasort(void *obj);
void tool2_util_tbox_copy_highlight(void *obj, char *dst, int dst_len );
int tool2_util_remline(void *obj);
int tool2_util_scrollbar_check(void *obj);
int tool2_util_find_by_name(void *obj, char *name);
int tool2_util_getobjbyid(int id, void *obj);
int tool2_util_get_typechild(void *p, int type, void *obj);
char *tool2_util_compress_filename(char *text, int len);
int tool2_util_toggleGroup(T2GUI_BUTTON_TYPE *btn);
int tool2_util_swatchkey_create(T2_DATA_SWATCH_TYPE *sptr, T2GUI_BASE_TYPE *bptr);
int tool2_util_swatchkey_create_row(T2_DATA_SWATCHKEY_TYPE *skey, T2GUI_BASE_TYPE *bptr, int x, int y);
int tool2_util_eventactionlist_create(T2_DATA_PROGRAM_TYPE *prog, T2GUI_BASE_TYPE *bptr);
int tool2_util_eventactionlist_create_row(T2_DATA_PROGRAM_EVENTACTION_TYPE *eaptr, T2GUI_BASE_TYPE *bptr, int x, int *y, int num);
int tool2_util_instructionlist_create(T2_DATA_PROGRAM_TYPE *prog, T2GUI_BASE_TYPE *bptr);
int tool2_util_instructionlist_create_row(T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins, T2GUI_BASE_TYPE *bptr, int x, int *y, int num);
int tool2_util_getbuttonbytext(T2GUI_BASE_TYPE *bptr, char *text, T2GUI_BASE_TYPE **b);
int tool2_util_statelist_create(T2GUI_WINDOW_TYPE *wptr, T2_DATA_PROGRAM_TYPE *prog );
int tool2_util_programlist_create(T2GUI_WINDOW_TYPE *wptr, int gui_id, int use_scene);
int tool2_util_scenelist_create(T2GUI_WINDOW_TYPE *wptr);
int tool2_util_parameterlist_create(T2_DATA_PROGRAM_TYPE *prog, T2GUI_BASE_TYPE *bptr);
int tool2_util_resize_selectlistbox( T2GUI_WINDOW_TYPE **pbigwin, T2GUI_WINDOW_TYPE **pwptr, T2GUI_TEXTBOX_TYPE **ptbptr, T2GUI_BUTTON_TYPE **pbtn, T2GUI_TEXTINPUT_TYPE **ptiptr);

int tool2_util_paramwin_int_create(void *ptr);

#define T2_COLORPICKER_CREATE_FLAGS_NOALPHA 0x01
#define T2_COLORPICKER_CREATE_FLAGS_GROWABLE 0x02

int tool2_util_colorpicker_create(T2GUI_BASE_TYPE **b, int x, int y, int w, int h, float max, int flags, T2GUI_BASE_TYPE *bptr);
void tool2_util_colorpicker_update(void *ptr, void *ptr2);
int tool2_util_colorpicker_set(void *ptr, float *color);
int tool2_util_colorpicker_get(void *ptr, float *color);

int tool2_util_slider_setval(T2GUI_SLIDER_TYPE *slptr, float val);
int tool2_util_slider_synctoval(void *ptr, void *ptr2);

void tool2_util_window_restore_scrollpos(T2GUI_BASE_TYPE *p, int scrollPos);
void tool2_util_window_killandfill(T2GUI_BASE_TYPE *p, ftype3 *func, void *arg1, void *arg2);

int tool2_util_dgtype_create(T2_DATA_GUI_TYPE *g, T2GUI_BASE_TYPE *bptr, int *x, int *y, int nameLinkID, int num, int xalign );

void t2cb_selpostfx_create(void *ptr, void *ptr2);
void t2cb_selpostfx_select(void *ptr, void *ptr2);

void tool2_cb_camlist(void *ptr, void *ptr2);

// - from tool2_gl.c

void t2gl_testdisp(void);
void t2gl_disp_kan(ResDataKan *resKan);
void t2gl_disp_kan2d(ResDataKan *resKan);
void t2gl_disp_swatch(ResDataKan *resKan, T2_DATA_SWATCH_TYPE *sptr);
void t2gl_disp_pemitter(PC_ParticleEmitter *pe);
void t2gl_disp_objaxis(Obj *o, void *s);
void t2gl_disp_progressbar(float s);
void t2gl_disp_objname(Obj *o, void *s);

void t2gl_util_3dto2d(crd *pos3d, crd *pos2d, int *view);

void t2gl_test(int x, int y);
void t2gl_setMouse(int x, int y);
void t2gl_draw_window(T2GUI_WINDOW_TYPE *wptr);
void t2gl_draw_button(T2GUI_BUTTON_TYPE *btn);
void t2gl_draw_textbox(T2GUI_TEXTBOX_TYPE *tptr);
void t2gl_draw_textinput(T2GUI_TEXTINPUT_TYPE *tiptr);
void t2gl_draw_textlabel(T2GUI_TEXTLABEL_TYPE *tlptr);
void t2gl_draw_crect(T2GUI_CRECT_TYPE *crptr);
void t2gl_draw_slider(T2GUI_SLIDER_TYPE *slptr);

void t2gl_render_text(char *buf, int x, int y);

void t2gl_getViewport(int *view);

void t2gl_matrix_reset(void);
void t2gl_matrix_push();
void t2gl_matrix_push_trans(float x, float y, float z);
void t2gl_matrix_trans(float x, float y, float z );
void t2gl_matrix_pop(void);
float *t2gl_matrix_get(void);
void t2gl_matrix_commit(void);

void t2gl_window_stencil_set(T2GUI_WINDOW_TYPE *wptr);
void t2gl_window_stencil_get(int *x, int *y, int *w, int *h);
void t2gl_window_stencil_on(void);
void t2gl_window_stencil_off(void);
void t2gl_window_offset_off(T2GUI_WINDOW_TYPE *wptr);
void t2gl_window_offset_on(T2GUI_WINDOW_TYPE *wptr);

void t2gl_draw_dbg_texture(int x, int y, int w, int h, int gltex);

// - from tool2_data.c

int tool2_data_init(void);

int tool2_data_programlist_add(T2_DATA_PROGRAM_TYPE *prog);
int tool2_data_programlist_remove(T2_DATA_PROGRAM_TYPE *prog);
int tool2_data_programlist_search(char *progname, T2_DATA_PROGRAM_TYPE **p);

int tool2_data_swatch_create(T2_DATA_SWATCH_TYPE **sbase, char *name);
int tool2_data_swatch_destroy(T2_DATA_SWATCH_TYPE *sptr);
int tool2_data_swatch_addkey(T2_DATA_SWATCH_TYPE *sbase, T2_DATA_SWATCHKEY_TYPE *skey);
int tool2_data_swatch_delkey(T2_DATA_SWATCH_TYPE *sptr, T2_DATA_SWATCHKEY_TYPE *skey);
int tool2_data_swatch_keycount(T2_DATA_SWATCH_TYPE *sptr);
int tool2_data_swatch_getkey(T2_DATA_SWATCH_TYPE *sptr, int num, T2_DATA_SWATCHKEY_TYPE *skey);
int tool2_data_swatch_getkeybyid(T2_DATA_SWATCH_TYPE *sptr, int id, T2_DATA_SWATCHKEY_TYPE **skey);
int tool2_data_swatch_swapkeys(T2_DATA_SWATCH_TYPE *sptr, T2_DATA_SWATCHKEY_TYPE *key1, T2_DATA_SWATCHKEY_TYPE *key2);

void tool2_data_swatch_test(void);
void tool2_data_sm_test(void);

int tool2_data_psm_listinstructions(char *sm_name, char **names, int **rank, int *num);
void tool2_data_psm_listinstructions_listfunc(ListNode *node, void *data);

int tool2_data_program_create(T2_DATA_PROGRAM_TYPE **prog, char *name);
int tool2_data_program_clear(T2_DATA_PROGRAM_TYPE *prog);
int tool2_data_program_destroy(T2_DATA_PROGRAM_TYPE *prog);
int tool2_data_program_destroy_ins(T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_program_swapinstructions(T2_DATA_PROGRAM_TYPE *prog
									  , T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins1
									  , T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins2);
int tool2_data_program_addinstruction(T2_DATA_PROGRAM_TYPE *prog, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_program_delinstruction(T2_DATA_PROGRAM_TYPE *prog, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_program_insertinstruction(T2_DATA_PROGRAM_TYPE *prog, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins1, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins2);
int tool2_data_program_instruction_moveup(T2_DATA_PROGRAM_TYPE *prog, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_program_instruction_movedown(T2_DATA_PROGRAM_TYPE *prog, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_program_getinstructionbyid(T2_DATA_PROGRAM_TYPE *prog, int id, T2_DATA_PROGRAM_INSTRUCTION_TYPE **ins);
int tool2_data_program_writepsm(T2_DATA_PROGRAM_TYPE *prog, PSM_Program *psmprog);
int tool2_data_program_readpsm(T2_DATA_PROGRAM_TYPE *prog, PSM_Program *psmprog);
int tool2_data_program_buildlist(char *dir, List *l);
int tool2_data_program_buildlist_func(char *filename, List *l);
int tool2_data_program_tmpl_buildlist(List *l);
int tool2_data_program_instruction_guisync(T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);

int tool2_data_psm_locate_statemachine(char *sm_name);
int tool2_data_psm_get_statemachine(char *sm_name, Res **sm_res);
int tool2_data_psm_getstate(char *sm_name, char *name, PSM_State **state);
int tool2_data_psm_savetxt(PSM_Program *psmprog, char *filename);
int tool2_data_psm_readtxt(PSM_Program *psmprog, char *filename, char *smname);
int tool2_data_psm_filename_create(char *progname, char **filename);

int tool2_data_psm_guilist_create(char *name, char *smname, List *l);
void tool2_data_psm_guilist_parsenames(char *name, char *type, List *data, char *pname, int alevel);
void tool2_data_psm_guilist_assignguis(List *l);
int tool2_data_psm_ins_to_psm(PSM_ProgramInstruction *pins, T2_DATA_PROGRAM_INSTRUCTION_TYPE *ins);
int tool2_data_psm_smlist_create(char *name, List *l);	// <- produce a list of available state machines


int tool2_data_scene_resource_add(SceneType *scene, SceneResource *res);
int tool2_data_scene_resource_remove(SceneType *scene, SceneResource *res);
int tool2_data_scene_buildlist_func(char *filename, void *data);
int tool2_data_scene_buildtriggerlist(SceneType *scene, char *trigger, List *l);

/*
int tool2_data_scene_create(SceneType **scene);
int tool2_data_scene_destroy(SceneType *scene);
int tool2_data_scene_addguis(SceneType *scene);

int tool2_data_scene_activelist_add(SceneType *scene);
int tool2_data_scene_activelist_remove(SceneType *scene);
int tool2_data_scene_activelist_getlist(List **l);
int tool2_data_scene_activelist_clear(void);
int tool2_data_scene_clear(SceneType *scene);
int tool2_data_scene_savetofile(SceneType *scene);
int tool2_data_scene_savetoxml(SceneType *scene);
int tool2_data_scene_loadfromxml(SceneType *scene, char *name, char *file );
int tool2_data_scene_loadfromfile(SceneType *scene, char *name, int load_pemitters );




int tool2_data_util_xml_listadd_pemitters(SceneType *scene, char *buf );
int tool2_data_util_xml_listadd_fxgroups(SceneType *scene, char *buf );
int tool2_data_util_xml_listadd_pemitters_fromfile(SceneType *scene, char *filename );

int tool2_data_pemitters_savexml(SceneType *scene, char *filename );


*/
int tool2_data_scene_buildlist(List *l);


int tool2_data_obj_findprime(Obj *obj, Obj **prime);

void tool2_data_playanim_animscan(char *name, void *p);		// - scan for anims, and place inside the gui

void tool2_data_seldisp2_scan(char *name, void *p);
void tool2_data_selcol_scan(char *name, void *p);

void tool2_data_selenvcol_scan(void *p);
void tool2_data_selfog_scan(void *p);
void tool2_data_selfur_scan(void *p);
void tool2_data_selcar_scan(void *p);
void tool2_data_selfxg_scan(void *p, char* groupName);
void tool2_data_selfxg_group_scan(void *p);
void tool2_data_selemitter_scan(void *p, char* groupName);
void tool2_data_selemitter_group_scan(void *p);
void tool2_data_selsoundfx_scan(void *p, char* bankName);
void tool2_data_selsoundfxbank_scan(void *p);
void tool2_data_selimage_scan(void *p);
void tool2_data_selobjgroup_scan(void *p);
void tool2_data_seljoint_scan(void *p, Obj *o);
void tool2_data_selwater_scan(void *p);
void tool2_data_selwind_scan(void *p);

void tool2_data_selspline_splinescan(char *name, void *p);	// - scan for splines, and place inside gui
int tool2_data_selspline_isspline(char *name);

void tool2_data_selprogparamater_scan(char *name, void *p, char *var);

int tool2_data_util_xml_betweentag(char *buf, char *tagname, char **result, int *len);
int tool2_data_util_xml_locatetag(char *buf, char *tagname, char **result, int *len);

int tool2_data_util_triggerlist_findtriggers(SceneType *scene);
int tool2_data_util_fxgroup_locateparent(PC_FXGroupList *fxgl, PC_FXGroup **fxg);
int tool2_data_util_emitterlist_nametoindex(ParticleCollection *pc, char *name);
int tool2_data_util_emitterlist_locatebyobj(Obj *o, PC_ParticleEmitter **peptr);
int tool2_data_util_program_findtriggers(char *filename, List *nameList);






int tool2_data_util_dgt_sync(T2_DATA_GUI_TYPE *dgt);

int tool2_data_util_sndplay(char *sndname);
int tool2_data_util_sndplay_vol(char *sndname, int vol);

int tool2_data_postfx_scan(T2GUI_BASE_TYPE *b);


// int tool2_data_alphabetize(SceneType *scene, int type);
int tool2_data_alphabetize_PC(ParticleCollection *pc, int type);


int tool2_data_util_anisfx_nodelist_build(List *l, char *group, char *anim);
int tool2_data_util_anisfx_nodelist_getnthnode(T2_DATA_ANISFX_NODE_TYPE **node, int index);
int tool2_data_util_anisfx_nodelist_removenode(T2_DATA_ANISFX_NODE_TYPE *node);
int tool2_data_anisfx_addnode(char *group, char *anim, float frame, char *sfx, int type, T2_DATA_ANISFX_NODE_TYPE **nobj);
int tool2_data_anisfx_save(void);
int tool2_data_anisfx_load(void);

int tool2_util_plstring_get(char *name, char *buf, int buflen);

int tool2_data_util_texgen_to_imgname(int texGen, char **name);
int tool2_data_util_imgname_to_texgen(char *name, int *texGen);
int tool2_data_image_data_to_ResDataImg1(image_data *image, char* name, Res **res, int mipmap, int compress);

int tool2_data_util_paramlist_dgt_create(T2_DATA_PROGRAM_TYPE *prog);

// - from tool2_particle.c

void tool2_particle_emitter_select(void);
void tool2_particle_emitter_edit_create(void *ptr);
void tool2_particle_emitter_edit_update(void *ptr);
void tool2_particle_emitter_sync(ParticleEmitter *pe_dst, ParticleEmitter *pe_src);

void tool2_particle_emitter_edit_posvel_create(void *ptr, T2GUI_BASE_TYPE *bptr);
void tool2_particle_emitter_edit_color_create(void *ptr, T2GUI_BASE_TYPE *bptr);
void tool2_particle_emitter_edit_dynamics_create(void *ptr, T2GUI_BASE_TYPE *bptr);
void tool2_particle_emitter_edit_creation_create(void *ptr, T2GUI_BASE_TYPE *bptr);
void tool2_particle_emitter_edit_misc_create(void *ptr, T2GUI_BASE_TYPE *bptr);

void tool2_particle_emitter_edit_stop(void *ptr, void *ptr2);
void tool2_particle_emitter_edit_restart(void *ptr, void *ptr2);
void tool2_particle_emitter_edit_restart_selected(void *ptr, void *ptr2);
void tool2_particle_emitter_edit_sort(void *ptr, void *ptr2);


void tool2_particle_fxgroup_edit_create(void *ptr);
void tool2_particle_fxgroup_edit_update(void *ptr);
void tool2_particle_fxgroup_edit_populate(void* ptr, void *ptr2);
void tool2_particle_fxgroup_edit_move_up(void *ptr, void *ptr2);
void tool2_particle_fxgroup_edit_move_down(void *ptr, void *ptr2);

// -------------------------------------------------------------------


// game side init funcs... (for adding game specific callbacks...
void tool2_set_gui_funcs( int (*gui_func_keyboard_input_app)(int), int (*gui_func_each_pass)(void) );

void tool2_set_game_callbacks( int (*game_wipeout)(void),
							   int (*game_wipeout_done)(void),
							   int (*game_loadconfig)(void),
							   int (*game_playermode)(int) 
							   );



// EOF

#endif // #ifndef NO_TOOL 
#endif // TOOL_H

