#ifndef TRIGGERS_H
#define TRIGGERS_H

// #include "g5.h"

enum N2TriggerSubTypes
{
	TRIGSUB_TRIGGER,
	TRIGSUB_EVENT,
};

enum TriggerTypes 
{
	TRIGGER_TYPE_STANDARD,
} ;

#define TRIGGER_FLAGS_SET 0x00000001
#define TRIGGER_FLAGS_UNSET 0x00000002
#define TRIGGER_FLAGS_KILL  0x00000004

#define TRIGGER_NAME_SIZE  PM_NAME_SIZE*2

typedef struct 
{
	int id;
	int type;
	unsigned int flags;
	char trigger_name[TRIGGER_NAME_SIZE];
	int timestamp;
	uns32 GameWaveTime;
	int count; // number of times triggered
	int max_count;	// expire after this man times triggered
} Trigger;

enum TriggerEventTypes 
{
	TRIGGEREVENT_TYPE_STANDARD,
} ;

#define TRIGGEREVENT_FLAGS_FREE_DATA         0x00000001  // free data when freeing trigger event, uses stnd C free()
#define TRIGGEREVENT_FLAGS_REPEAT_ON_CHANGE  0x00000002 // repeat only when trigger goes from unset to set, default is to repeat every frame that trigger is set 




typedef struct TriggerEvent
{
	int id;
	int type;
	unsigned int flags;
	char trigger_name[PM_NAME_SIZE];
	char event_name[PM_NAME_SIZE];
	int (*EventFunc)(Trigger *, struct TriggerEvent *);	// event
	int count;		// number of times triggered
	int max_count;	// expire after this man times triggered
	void *data;
	int last_timestamp;
	uns32 last_GameWaveTime;
	int last_trigger_count;
} TriggerEvent;

typedef struct 
{
	N2 lnk;
	TriggerEvent te;
} TriggerEventNode;

typedef struct 
{
	N2 lnk;
	Trigger t;
} TriggerNode;




int TriggersInit();
int TriggersFinal();
int TriggersLoop();

int TriggersWipeOut();


int RegisterTriggerEvent( int type, unsigned int flags,
						char *trigger_name,
						char *event_name,
						int (*EventFunc)(Trigger *, TriggerEvent *),	// event
						int max_count,	// expire after this man times triggered
						void *data );

int UnRegisterTriggerEvent( int id,
							char *trigger_name,
							char *event_name );

int CreateTrigger( char *trigger_name, int type, unsigned int flags, int max_count );

int SetTrigger( int id, char *trigger_name );

int ClearTrigger( int id, char *trigger_name );
int DeleteTrigger( int id, char *trigger_name );

int CheckTriggerStatus(  int id, char *trigger_name ); // returns 1 if trigger is set, 0 otherwise

int CheckTriggerCount(  int id, char *trigger_name ); // returns number of times trigger has been site
uns32 CheckTriggerGameWaveTime(  int id, char *trigger_name ); // returns GameWaveTime of last time trigger was set


N2 *GetActiveTriggerList();
N2 *GetActiveTriggerEventList();


TriggerEventNode *FindTriggerEvent( int id, char *trigger_name, char *event_name );
int FindTriggerEventByName( char *trigger_name, char *event_name );

TriggerNode *FindTrigger( int id, char *trigger_name );


#endif

