#ifndef VEC_H
#define VEC_H
// vec.h
// Copyright 2004 Play Mechanix Inc. All Rights Reserved
//
// JFL 18 Oct 04

// min and max macros
// note max(a++,b++) won't work so don't use it!
#ifndef MAX
	#define MAX( a, b ) ( ((a) > (b)) ? (a) : (b) )
#endif

#ifndef MIN
	#define MIN( a, b ) ( ((a) < (b)) ? (a) : (b) )
#endif

#ifndef CLAMP // Enforces: a <= x <= b
	#define CLAMP( a, x, b ) ( MAX( (a), MIN( (b), (x) ) ) )
#endif

#define RoundDown( r, n, m ) r = ((int)n/(int)m)*(int)m;
#define RoundUp( r, n, m ) r = (((int)n/(int)m)+ ((n<0)?-1:1))*(int)m;

#define Round(f)	((int)(f+.5))

#define FloatIsValid(f) ( isfinite(f) && !_isnan(f) )


#define BITFLIP(flags, bit) { if(flags & bit) flags &= ~bit; else flags |= bit; }

#ifdef __cplusplus
#define EXTERN_C	extern "C"
#else
#define EXTERN_C	extern
#endif

#define BOX_REL_W 640
#define BOX_REL_H 480

#define BOX_ABS_W 640
#define BOX_ABS_H 480

extern const double _double2fixmagic;     //2^36 * 1.5,  (52-_shiftamt=36) uses limited precisicion to floor
extern const int32 _shiftamt;                    //16.16 fixed point representation,

#if BigEndian_
#define iexp_				0
#define iman_				1
#else
#define iexp_				1
#define iman_				0
#endif //BigEndian_

// ================================================================================================
// Real2Int
// ================================================================================================
extern __inline long Double2Int(double val);

// ================================================================================================
// Real2Int
// ================================================================================================
extern __inline long Float2Int(float val);

//assumes a is between 0 and 255
extern __inline unsigned char flt_to_byte (float a);

/*
//trig lookup tables
#define SINCOS_NUM_VALUES	8192
#define SINCOS_BIT_MASK		8191
extern const float TRIG_BIAS;
float sin_table[SINCOS_NUM_VALUES];
float cos_table[SINCOS_NUM_VALUES];
extern void InitializeTrigTables();
extern __inline float Sin(float radian);
extern __inline float Cos(float radian);
*/
///////////////////////////////////////////////////////////////////////////////
//

typedef struct { crd a[Q_SIZE]; } QuatType;
typedef struct { crd a[V2_SIZE]; } V2Type;
typedef struct { crd a[V3_SIZE]; } V3Type;
typedef struct { crd a[M3_SIZE]; } M3Type;
typedef struct { float a[M3_SIZE]; } M3Typef;
typedef struct { crd a[M4_SIZE]; } M4Type;
typedef struct { crd a[VM_SIZE]; } VMType;



//
// V2
//

#define V2Zero(a) \
	(a)[V2_X]=(a)[V2_Y]=0

#define V2SetXY(a,x,y) \
	(a)[V2_X]=x,(a)[V2_Y]=y


#define V2SetV2(a,b) \
       (a)[V2_X]=(b)[V2_X],(a)[V2_Y]=(b)[V2_Y]

#define V2AddV2(a,b) \
	(a)[V2_X]+=(b)[V2_X],(a)[V2_Y]+=(b)[V3_Y]

#define V2Cpy(a,b) \
	*((V2Type*)a)=*((V2Type*)b)

#define V2Add(dst,a,b) \
	(dst)[V2_X]=(a)[V2_X]+(b)[V2_X], \
	(dst)[V2_Y]=(a)[V2_Y]+(b)[V2_Y]

#define V2Sub(dst,a,b) \
	(dst)[V2_X]=(a)[V2_X]-(b)[V2_X], \
	(dst)[V2_Y]=(a)[V2_Y]-(b)[V2_Y]


#define V2Dot(p,q) \
	(((p)[0])*((q)[0])+((p)[1])*((q)[1]))

#define V2Cross(p,q) \
	(((p)[0])*((q)[1])-((p)[1])*((q)[0]))


//
// V3
//

#define V3Zero(a) \
	(a)[V3_X]=(a)[V3_Y]=(a)[V3_Z]=0

#define V3SetXYZ(a,x,y,z) \
	(a)[V3_X]=x,(a)[V3_Y]=y,(a)[V3_Z]=z

#define V3SetXY(a,x,y) \
	(a)[V3_X]=x,(a)[V3_Y]=y

#define V3SetXZ(a,x,z) \
	(a)[V3_X]=x,(a)[V3_Z]=z

#define V3SetYZ(a,y,z) \
	(a)[V3_Y]=y,(a)[V3_Z]=z


#define V3SetVM(a,b) \
	(a)[V3_X]=(b)[VM_X],(a)[V3_Y]=(b)[VM_Y],(a)[V3_Z]=(b)[VM_Z]

#define V3SetV3(a,b) \
       (a)[V3_X]=(b)[V3_X],(a)[V3_Y]=(b)[V3_Y],(a)[V3_Z]=(b)[V3_Z]


#define V3SetM4(a,b) \
	(a)[V3_X]=(b)[M4_X],(a)[V3_Y]=(b)[M4_Y],(a)[V3_Z]=(b)[M4_Z]

#define V3AddV3(a,b) \
	(a)[V3_X]+=(b)[V3_X],(a)[V3_Y]+=(b)[V3_Y],(a)[V3_Z]+=(b)[V3_Z]

#define V3Cpy(a,b) \
	*((V3Type*)a)=*((V3Type*)b)

#define V3Add(dst,a,b) \
	(dst)[V3_X]=(a)[V3_X]+(b)[V3_X], \
	(dst)[V3_Y]=(a)[V3_Y]+(b)[V3_Y], \
	(dst)[V3_Z]=(a)[V3_Z]+(b)[V3_Z]

#define V3Sub(dst,a,b) \
	(dst)[V3_X]=(a)[V3_X]-(b)[V3_X], \
	(dst)[V3_Y]=(a)[V3_Y]-(b)[V3_Y], \
	(dst)[V3_Z]=(a)[V3_Z]-(b)[V3_Z]

#define V3AddRots(dst,a,b) \
	(dst)[V3_X]= MathAddRot( (b)[V3_X], (a)[V3_X] ), \
	(dst)[V3_Y]= MathAddRot( (b)[V3_Y], (a)[V3_Y] ), \
	(dst)[V3_Z]= MathAddRot( (b)[V3_Z], (a)[V3_Z] ) 

#define V3SubRots(dst,a,b) \
	(dst)[V3_X]= MathSubRot( (b)[V3_X], (a)[V3_X] ), \
	(dst)[V3_Y]= MathSubRot( (b)[V3_Y], (a)[V3_Y] ), \
	(dst)[V3_Z]= MathSubRot( (b)[V3_Z], (a)[V3_Z] ) 


#define V3Min(dst,a,m) \
	(dst)[V3_X]=MIN( (a)[V3_X], m ), \
	(dst)[V3_Y]=MIN( (a)[V3_Y], m ), \
	(dst)[V3_Z]=MIN( (a)[V3_Z], m )

#define V3Max(dst,a,m) \
	(dst)[V3_X]=MAX( (a)[V3_X], m ), \
	(dst)[V3_Y]=MAX( (a)[V3_Y], m ), \
	(dst)[V3_Z]=MAX( (a)[V3_Z], m )

#define V3MinV3(dst,a,b) \
	(dst)[V3_X]=MIN( (a)[V3_X], (b)[V3_X] ), \
	(dst)[V3_Y]=MIN( (a)[V3_Y], (b)[V3_Y] ), \
	(dst)[V3_Z]=MIN( (a)[V3_Z], (b)[V3_Z] )

#define V3MaxV3(dst,a,b) \
	(dst)[V3_X]=MAX( (a)[V3_X], (b)[V3_X] ), \
	(dst)[V3_Y]=MAX( (a)[V3_Y], (b)[V3_Y] ), \
	(dst)[V3_Z]=MAX( (a)[V3_Z], (b)[V3_Z] )


#define V3Inv(dst,a) \
	(dst)[V3_X]=1.0f/(a)[V3_X], \
	(dst)[V3_Y]=1.0f/(a)[V3_Y], \
	(dst)[V3_Z]=1.0f/(a)[V3_Z]



//#define V3Mul(dst,a,m) \
//	(dst)[V3_X]=(a)[V3_X]*(m), \
//	(dst)[V3_Y]=(a)[V3_Y]*(m), \
//	(dst)[V3_Z]=(a)[V3_Z]*(m)
// using a function instead of a macro; the macro causes too many mistakes
void V3Mul(float* dst, float* a, float m);

#define V3MulV3(dst,a,m) \
	(dst)[V3_X]=(a)[V3_X]*(m)[V3_X], \
	(dst)[V3_Y]=(a)[V3_Y]*(m)[V3_Y], \
	(dst)[V3_Z]=(a)[V3_Z]*(m)[V3_Z]



#define V3MulM3(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M3_11]+(v)[1]*(mat)[M3_12]+(v)[2]*(mat)[M3_13],\
	(r)[1]=(v)[0]*(mat)[M3_21]+(v)[1]*(mat)[M3_22]+(v)[2]*(mat)[M3_23],\
	(r)[2]=(v)[0]*(mat)[M3_31]+(v)[1]*(mat)[M3_32]+(v)[2]*(mat)[M3_33]

#define V3RotM3 V3MulM3


#define V3MulVM(r,v,vm) \
	(r)[0]=(v)[0]*(vm)[VM_11]+(v)[1]*(vm)[VM_12]+(v)[2]*(vm)[VM_13]+(vm)[VM_X],\
	(r)[1]=(v)[0]*(vm)[VM_21]+(v)[1]*(vm)[VM_22]+(v)[2]*(vm)[VM_23]+(vm)[VM_Y],\
	(r)[2]=(v)[0]*(vm)[VM_31]+(v)[1]*(vm)[VM_32]+(v)[2]*(vm)[VM_33]+(vm)[VM_Z]

#define V3RotVM(r,v,vm) \
	(r)[0]=(v)[0]*(vm)[VM_11]+(v)[1]*(vm)[VM_12]+(v)[2]*(vm)[VM_13],\
	(r)[1]=(v)[0]*(vm)[VM_21]+(v)[1]*(vm)[VM_22]+(v)[2]*(vm)[VM_23],\
	(r)[2]=(v)[0]*(vm)[VM_31]+(v)[1]*(vm)[VM_32]+(v)[2]*(vm)[VM_33]

#define V3MulM4(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M4_11]+(v)[1]*(mat)[M4_12]+(v)[2]*(mat)[M4_13]+(mat)[M4_14],\
	(r)[1]=(v)[0]*(mat)[M4_21]+(v)[1]*(mat)[M4_22]+(v)[2]*(mat)[M4_23]+(mat)[M4_24],\
	(r)[2]=(v)[0]*(mat)[M4_31]+(v)[1]*(mat)[M4_32]+(v)[2]*(mat)[M4_33]+(mat)[M4_34]

#define V3RotM4(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M4_11]+(v)[1]*(mat)[M4_12]+(v)[2]*(mat)[M4_13],\
	(r)[1]=(v)[0]*(mat)[M4_21]+(v)[1]*(mat)[M4_22]+(v)[2]*(mat)[M4_23],\
	(r)[2]=(v)[0]*(mat)[M4_31]+(v)[1]*(mat)[M4_32]+(v)[2]*(mat)[M4_33]

#define V3Cross(r,p,q) \
	(r)[0] = (p)[1]*(q)[2] - (p)[2]*(q)[1],\
	(r)[1] = (p)[2]*(q)[0] - (p)[0]*(q)[2],\
	(r)[2] = (p)[0]*(q)[1] - (p)[1]*(q)[0]

#define V3Dot(p,q) \
	(((p)[0])*((q)[0])+((p)[1])*((q)[1])+((p)[2])*((q)[2]))

#define V3DotXY(p,q) \
	(((p)[0])*((q)[0])+((p)[1])*((q)[1]))

#define V3DotXZ(p,q) \
	(((p)[0])*((q)[0])+((p)[2])*((q)[2]))

#define V3DotYZ(p,q) \
	(((p)[1])*((q)[1])+((p)[2])*((q)[2]))


#define V3LenSq(v) \
	V3Dot(v,v)

#define V3LenXZSq(v) \
	V3DotXZ(v,v)

#define V3Negate(v) \
	(v)[0] = -(v)[0],\
	(v)[1] = -(v)[1],\
	(v)[2] = -(v)[2]


#define V3IsValid(v) \
	(FloatIsValid((v)[0]) && \
	 FloatIsValid((v)[1]) && \
	 FloatIsValid((v)[2]) )

EXTERN_C crd V3Normalize(crd* v);
extern crd V3Unitize(crd* v,crd len);
EXTERN_C crd V3Len(crd* v);
EXTERN_C crd V3LenXZ(crd* v);
extern int V3NormalCW(crd *normal,crd *v0,crd *v1,crd *v2);
extern crd V3DistFast(crd *v0,crd *v1);
extern crd V3DistSq(crd *v0,crd *v1);
extern crd V3DistXZSq(crd *v0,crd *v1);
extern crd V3DistXYSq(crd *v0,crd *v1);
extern crd V3UnitizeFast(crd* v,crd len);
// extern void V3RotToUnitVec(crd *rot, crd *vec );
extern void V3RotVec(crd *vec, crd *rot );
extern void V3RotY(crd *vec, float yRot );

EXTERN_C void V3CamToWorldFlags(void* ocam, crd* in, crd* out, int ignoreFlags);
EXTERN_C void V3WorldToCamFlags(void* ocam, crd* in, crd* out, int ignoreFlags);
EXTERN_C void V3CamToWorld(void* ocam, crd* in, crd* out);
EXTERN_C void V3WorldToCam(void* ocam, crd* in, crd* out);

EXTERN_C int V3Eq(crd* v0, crd* v1);


//
// M3
//

#define M3Cpy(a,b) \
	*((M3Type*)(a))=*((M3Type*)(b))
#define M3Cpyf(a,b) \
	*((M3Typef*)(a))=*((M3Typef*)(b))

#define M3Id(a) \
	(a)[M3_11]=(a)[M3_22]=(a)[M3_33]=1, \
	(a)[M3_12]=(a)[M3_13]=(a)[M3_21]= \
	(a)[M3_23]=(a)[M3_31]=(a)[M3_32]=0

#define M3BuildScaleXYZ(a,x,y,z) \
	(a)[M3_11]=(x),(a)[M3_22]=(y),(a)[M3_33]=(z), \
	(a)[M3_12]=(a)[M3_13]=(a)[M3_21]= \
	(a)[M3_23]=(a)[M3_31]=(a)[M3_32]=0

#define M3SetM4(a,b) \
	(a)[M3_11]=(b)[M4_11],(a)[M3_12]=(b)[M4_12],(a)[M3_13]=(b)[M4_13], \
	(a)[M3_21]=(b)[M4_21],(a)[M3_22]=(b)[M4_22],(a)[M3_23]=(b)[M4_23], \
	(a)[M3_31]=(b)[M4_31],(a)[M3_32]=(b)[M4_32],(a)[M3_33]=(b)[M4_33]

#define M3VecMul(r,mat,v) \
	(r)[0]=(v)[0]*(mat)[M3_11]+(v)[1]*(mat)[M3_12]+(v)[2]*(mat)[M3_13],\
	(r)[1]=(v)[0]*(mat)[M3_21]+(v)[1]*(mat)[M3_22]+(v)[2]*(mat)[M3_23],\
	(r)[2]=(v)[0]*(mat)[M3_31]+(v)[1]*(mat)[M3_32]+(v)[2]*(mat)[M3_33]

#define M3Mpy M3Mul

extern void		M3FromQuat(crd *m,crd *q);
EXTERN_C void		M3Mul(crd *dst,crd *a,crd *b);
extern void		M3ToQuat(crd* mat, crd* q);
extern int		M3Inv(crd *dst,crd *src);
extern void		M3Transpose(float *dst, float *src);
extern void		M3BuildRot(crd *a,crd rad,int xyz);
extern void		M3BuildRotFast(crd *a, crd rad, int xyz);
extern void		M3BuildObj(crd *dstm,crd *rots);
EXTERN_C void	M3BuildRotZYX(crd *outMat, float rotX, float rotY, float rotZ);
EXTERN_C void	M3BuildRotXYZ(crd *outMat, float rotX, float rotY, float rotZ);
extern void		M3BuildRotXYZ_SlowAccurate(crd *outMat, float rotX, float rotY, float rotZ); //slower but more accurate
extern void		M3BuildRotZYX_SlowAccurate(crd *outMat, float rotX, float rotY, float rotZ);
EXTERN_C void		M3BuildRotAxisAngle(crd *outMat, crd *axis, float angle);
extern void		M3BuildRotQuat(crd *outMat, crd *q);
extern void		M3BuildShear(float* out, float* shearIn, float* shearAlong, float shearAmount);

//
// M4
//

#define M4Cpy(a,b) \
	*((M4Type*)a)=*((M4Type*)b)

#define M4Id(a) \
	(a)[M4_11]=(a)[M4_22]=(a)[M4_33]=(a)[M4_44]=1,\
	(a)[M4_12]=(a)[M4_13]=(a)[M4_14]=(a)[M4_21]=\
	(a)[M4_23]=(a)[M4_24]=(a)[M4_31]=(a)[M4_32]=\
	(a)[M4_34]=(a)[M4_41]=(a)[M4_42]=(a)[M4_43]=0


#define M4IsId(a) \
	( ((a)[M4_11]==1.0f) && ((a)[M4_22]==1.0f) && ((a)[M4_33]==1.0f) && ((a)[M4_44]==1.0f) &&\
	  ((a)[M4_12]==0.0f) && ((a)[M4_13]==0.0f) && ((a)[M4_14]==0.0f) && ((a)[M4_21]==0.0f) &&\
	  ((a)[M4_23]==0.0f) && ((a)[M4_24]==0.0f) && ((a)[M4_31]==0.0f) && ((a)[M4_32]==0.0f) &&\
	  ((a)[M4_34]==0.0f) && ((a)[M4_41]==0.0f) && ((a)[M4_42]==0.0f) && ((a)[M4_43]==0.0f) )


/* this failed for whatever reason...
	( ((a)[M4_11]==(a)[M4_22]==(a)[M4_33]==(a)[M4_44]==1.0f)\
	   &&\
	  ((a)[M4_12]==(a)[M4_13]==(a)[M4_14]==(a)[M4_21]==\
	   (a)[M4_23]==(a)[M4_24]==(a)[M4_31]==(a)[M4_32]==\
	   (a)[M4_34]==(a)[M4_41]==(a)[M4_42]==(a)[M4_43]==0.0f) )
*/


void MathM4ToRot(crd *m4,crd *rot);
int M4IsIdDbg(float *m4);

#define M4ClrVec(a) \
	(a)[M4_44]=1,(a)[M4_X]=(a)[M4_Y]=(a)[M4_Z]=0

#define M4ClrCol(a) \
	(a)[M4_41]=(a)[M4_42]=(a)[M4_43]=0

#define M4SetV3(a,b) \
	(a)[M4_X]=(b)[V3_X],(a)[M4_Y]=(b)[V3_Y],(a)[M4_Z]=(b)[V3_Z]

#define M4AddV3(a,b) \
	(a)[M4_X]+=(b)[V3_X],(a)[M4_Y]+=(b)[V3_Y],(a)[M4_Z]+=(b)[V3_Z]

#define M4SetXYZ(a,x,y,z) \
	(a)[M4_X]=x,(a)[M4_Y]=y,(a)[M4_Z]=z

#define M4AddXYZ(a,x,y,z) \
	(a)[M4_X]+=x,(a)[M4_Y]+=y,(a)[M4_Z]+=z

#define M4SetM3(a,b) \
	(a)[M4_11]=(b)[M3_11],(a)[M4_12]=(b)[M3_12],(a)[M4_13]=(b)[M3_13], \
	(a)[M4_21]=(b)[M3_21],(a)[M4_22]=(b)[M3_22],(a)[M4_23]=(b)[M3_23], \
	(a)[M4_31]=(b)[M3_31],(a)[M4_32]=(b)[M3_32],(a)[M4_33]=(b)[M3_33]

#define M4Set3x3(a,r1,r2,r3,s1,s2,s3,t1,t2,t3) \
	(a)[M4_11]=r1,(a)[M4_12]=r2,(a)[M4_13]=r3, \
	(a)[M4_21]=s1,(a)[M4_22]=s2,(a)[M4_23]=s3, \
	(a)[M4_31]=t1,(a)[M4_32]=t2,(a)[M4_33]=t3
	
#define M4SetVM(a,b) \
	(a)[M4_11]=(b)[VM_11],(a)[M4_12]=(b)[VM_12],(a)[M4_13]=(b)[VM_13], \
	(a)[M4_21]=(b)[VM_21],(a)[M4_22]=(b)[VM_22],(a)[M4_23]=(b)[VM_23], \
	(a)[M4_31]=(b)[VM_31],(a)[M4_32]=(b)[VM_32],(a)[M4_33]=(b)[VM_33], \
	(a)[M4_X]=(b)[VM_X],(a)[M4_Y]=(b)[VM_Y],(a)[M4_Z]=(b)[VM_Z], \
	(a)[M4_41]=(a)[M4_42]=(a)[M4_43]=0,(a)[M4_44]=1

#define M4SetM4(a,b) \
	(a)[M4_11]=(b)[M4_11],(a)[M4_12]=(b)[M4_12],(a)[M4_13]=(b)[M4_13], \
	(a)[M4_21]=(b)[M4_21],(a)[M4_22]=(b)[M4_22],(a)[M4_23]=(b)[M4_23], \
	(a)[M4_31]=(b)[M4_31],(a)[M4_32]=(b)[M4_32],(a)[M4_33]=(b)[M4_33], \
	(a)[M4_X]=(b)[M4_X],(a)[M4_Y]=(b)[M4_Y],(a)[M4_Z]=(b)[M4_Z], \
	(a)[M4_41]=(a)[M4_42]=(a)[M4_43]=0,(a)[M4_44]=1

#define M4VecMpy(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M4_11]+(v)[1]*(mat)[M4_12]+(v)[2]*(mat)[M4_13]+(mat)[M4_14],\
	(r)[1]=(v)[0]*(mat)[M4_21]+(v)[1]*(mat)[M4_22]+(v)[2]*(mat)[M4_23]+(mat)[M4_24],\
	(r)[2]=(v)[0]*(mat)[M4_31]+(v)[1]*(mat)[M4_32]+(v)[2]*(mat)[M4_33]+(mat)[M4_34]

#define M4OGLVecMpy(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M4OGL_11]+(v)[1]*(mat)[M4OGL_12]+(v)[2]*(mat)[M4OGL_13]+(mat)[M4OGL_14],\
	(r)[1]=(v)[0]*(mat)[M4OGL_21]+(v)[1]*(mat)[M4OGL_22]+(v)[2]*(mat)[M4OGL_23]+(mat)[M4OGL_24],\
	(r)[2]=(v)[0]*(mat)[M4OGL_31]+(v)[1]*(mat)[M4OGL_32]+(v)[2]*(mat)[M4OGL_33]+(mat)[M4OGL_34]


#define M4OGLQuatMpy(r,v,mat) \
	(r)[0]=(v)[0]*(mat)[M4OGL_11]+(v)[1]*(mat)[M4OGL_12]+(v)[2]*(mat)[M4OGL_13]+(v)[3]*(mat)[M4OGL_14],\
	(r)[1]=(v)[0]*(mat)[M4OGL_21]+(v)[1]*(mat)[M4OGL_22]+(v)[2]*(mat)[M4OGL_23]+(v)[3]*(mat)[M4OGL_24],\
	(r)[2]=(v)[0]*(mat)[M4OGL_31]+(v)[1]*(mat)[M4OGL_32]+(v)[2]*(mat)[M4OGL_33]+(v)[3]*(mat)[M4OGL_34], \
	(r)[3]=(v)[0]*(mat)[M4OGL_41]+(v)[1]*(mat)[M4OGL_42]+(v)[2]*(mat)[M4OGL_43]+(v)[3]*(mat)[M4OGL_44]



#define M4OGLV3Project(r,v,mat,w) \
	(r)[0]=(v)[0]*(mat)[M4OGL_11]+(v)[1]*(mat)[M4OGL_12]+(v)[2]*(mat)[M4OGL_13]+(mat)[M4OGL_14],\
	(r)[1]=(v)[0]*(mat)[M4OGL_21]+(v)[1]*(mat)[M4OGL_22]+(v)[2]*(mat)[M4OGL_23]+(mat)[M4OGL_24],\
	(r)[2]=(v)[0]*(mat)[M4OGL_31]+(v)[1]*(mat)[M4OGL_32]+(v)[2]*(mat)[M4OGL_33]+(mat)[M4OGL_34],\
	 w    =(v)[0]*(mat)[M4OGL_41]+(v)[1]*(mat)[M4OGL_42]+(v)[2]*(mat)[M4OGL_43]+(mat)[M4OGL_44],\
	(r)[0]/=w,(r)[1]/=w,(r)[2]/=w




#define VMSetM4OGL(a,b) \
	(a)[VM_11]=(b)[M4OGL_11],(a)[VM_12]=(b)[M4OGL_12],(a)[VM_13]=(b)[M4OGL_13], \
	(a)[VM_21]=(b)[M4OGL_21],(a)[VM_22]=(b)[M4OGL_22],(a)[VM_23]=(b)[M4OGL_23], \
	(a)[VM_31]=(b)[M4OGL_31],(a)[VM_32]=(b)[M4OGL_32],(a)[VM_33]=(b)[M4OGL_33], \
	(a)[VM_X]=(b)[M4OGL_X],(a)[VM_Y]=(b)[M4OGL_Y],(a)[VM_Z]=(b)[M4OGL_Z]

#define M4OGLSetVM(a,b) \
	(a)[M4OGL_11]=(b)[VM_11],(a)[M4OGL_12]=(b)[VM_12],(a)[M4OGL_13]=(b)[VM_13], \
	(a)[M4OGL_21]=(b)[VM_21],(a)[M4OGL_22]=(b)[VM_22],(a)[M4OGL_23]=(b)[VM_23], \
	(a)[M4OGL_31]=(b)[VM_31],(a)[M4OGL_32]=(b)[VM_32],(a)[M4OGL_33]=(b)[VM_33], \
	(a)[M4OGL_X]=(b)[VM_X],(a)[M4OGL_Y]=(b)[VM_Y],(a)[M4OGL_Z]=(b)[VM_Z], \
	(a)[M4OGL_41]=(a)[M4OGL_42]=(a)[M4OGL_43]=0,(a)[M4OGL_44]=1

extern void M4Mul(crd *dst,crd *a,const crd *b);
extern int M4InvGeneral(crd *inv,crd *mat);

extern void M4BuildScale(crd *outMat, float scaleX, float scaleY, float scaleZ);
extern void M4BuildTrans(crd *outMat, float X, float Y, float Z);
extern void M4BuildRotZYX(crd *outMat, float rotX, float rotY, float rotZ);
extern void M4BuildRotXYZ(crd *outMat, float rotX, float rotY, float rotZ);
extern void M4BuildRot(crd *a,crd rad,int xyz);

//
// VM
//

#define VMCpy(a,b) \
	*((VMType*)a)=*((VMType*)b)

#define VMId(a) \
	(a)[VM_11]=(a)[VM_22]=(a)[VM_33]=1, \
	(a)[VM_12]=(a)[VM_13]=(a)[VM_21]= \
	(a)[VM_23]=(a)[VM_31]=(a)[VM_32]=0, \
	(a)[VM_X]=(a)[VM_Y]=(a)[VM_Z]=0 


#define VMIsId(a) \
	( ((a)[VM_11]==(a)[VM_22]==(a)[VM_33]==1)\
	&&((a)[VM_12]==(a)[VM_13]==(a)[VM_21]==\
	   (a)[VM_23]==(a)[VM_31]==(a)[VM_32]==\
	   (a)[VM_X]==(a)[VM_Y]==(a)[VM_Z]==0) )



#define VMVecMul(r,v,vm) \
	(r)[0]=(v)[0]*(vm)[VM_11]+(v)[1]*(vm)[VM_12]+(v)[2]*(vm)[VM_13]+(vm)[VM_X],\
	(r)[1]=(v)[0]*(vm)[VM_21]+(v)[1]*(vm)[VM_22]+(v)[2]*(vm)[VM_23]+(vm)[VM_Y],\
	(r)[2]=(v)[0]*(vm)[VM_31]+(v)[1]*(vm)[VM_32]+(v)[2]*(vm)[VM_33]+(vm)[VM_Z]

#define VMVecRot(r,v,vm) \
	(r)[0]=(v)[0]*(vm)[VM_11]+(v)[1]*(vm)[VM_12]+(v)[2]*(vm)[VM_13],\
	(r)[1]=(v)[0]*(vm)[VM_21]+(v)[1]*(vm)[VM_22]+(v)[2]*(vm)[VM_23],\
	(r)[2]=(v)[0]*(vm)[VM_31]+(v)[1]*(vm)[VM_32]+(v)[2]*(vm)[VM_33]

#define VMSetM4(a,b) \
	(a)[VM_11]=(b)[M4_11],(a)[VM_12]=(b)[M4_12],(a)[VM_13]=(b)[M4_13], \
	(a)[VM_21]=(b)[M4_21],(a)[VM_22]=(b)[M4_22],(a)[VM_23]=(b)[M4_23], \
	(a)[VM_31]=(b)[M4_31],(a)[VM_32]=(b)[M4_32],(a)[VM_33]=(b)[M4_33], \
	(a)[VM_X]=(b)[M4_X],(a)[VM_Y]=(b)[M4_Y],(a)[VM_Z]=(b)[M4_Z]

#define VMSetXYZ(a,x,y,z) \
	(a)[VM_X]=(x),(a)[VM_Y]=(y),(a)[VM_Z]=(z)
																							
#define VMAddXYZ(a,x,y,z) \
	(a)[VM_X]+=x,(a)[VM_Y]+=y,(a)[VM_Z]+=z

#define VMSetV3(a,b) \
	(a)[VM_X]=(b)[V3_X],(a)[VM_Y]=(b)[V3_Y],(a)[VM_Z]=(b)[V3_Z]

#define VMAddV3(a,b) \
	(a)[VM_X]+=(b)[V3_X],(a)[VM_Y]+=(b)[V3_Y],(a)[VM_Z]+=(b)[V3_Z]

#define VMSubV3(a,b) \
	(a)[VM_X]-=(b)[V3_X],(a)[VM_Y]-=(b)[V3_Y],(a)[VM_Z]-=(b)[V3_Z]

#define VMBuildRot(a,rad,xyz) \
	(a)[VM_X]=(a)[VM_Y]=(a)[VM_Z]=0,M3BuildRot(&((crd*)(a))[3],rad,xyz)

#define VMIsValid(a) \
	(FloatIsValid((a)[VM_X ]) && \
	 FloatIsValid((a)[VM_Y ]) && \
	 FloatIsValid((a)[VM_Z ]) && \
	 FloatIsValid((a)[VM_11]) && \
	 FloatIsValid((a)[VM_21]) && \
	 FloatIsValid((a)[VM_31]) && \
	 FloatIsValid((a)[VM_12]) && \
	 FloatIsValid((a)[VM_22]) && \
	 FloatIsValid((a)[VM_32]) && \
	 FloatIsValid((a)[VM_13]) && \
	 FloatIsValid((a)[VM_23]) && \
	 FloatIsValid((a)[VM_33]) )
	 			



#define VMMul VMMpy

extern void VMInv(crd *dst,crd *a);
extern void VMInvFastRotsOnly(crd *dst,crd *src);
extern void VMMpy(crd *dst,crd *a,crd *b);
extern void VMCat(crd *dst,crd *a,crd *b);
extern void VMMlt(crd *dst,crd *cur,crd *add);
extern void VMClean(crd *dst);
EXTERN_C void VMBuildObj(crd *dst,crd *rots,crd *trans,crd *scale);

//
// QUAT
//

#define QuatId(a) \
	(a)[Q_X]=(a)[Q_Y]=(a)[Q_Z]=0,(a)[Q_W]=1

#define QuatCpy(a,b) \
	*((QuatType*)a)=*((QuatType*)b)

#define QuatSetXYZW(a,x,y,z,w) \
	(a)[Q_X]=x,(a)[Q_Y]=y,(a)[Q_Z]=z,(a)[Q_W]=w


extern void QuatToM3(crd* q, crd* mat);
extern void QuatNormalize(crd *q);
extern void QuatMul(crd *d,crd *a,crd *b);
extern void QuatAxisToAxis(crd *qdst,crd *a,crd *b);
extern void QuatSlerp(crd *qdst, crd *q1, crd *q2, float t);
extern void QuatNlerp(crd *qdst, crd *q1, crd *q2, float t);

// input/output euler angles. converts to quaternions internally for interpolation.
extern void QuatInterpEuler( crd* e1, crd* e2, crd* out, float t );
extern void QuatInterpM4( crd* m4a, crd* m4b, crd* m4Out, float t );

//
// BOX
//

#define BOXSetXY(a,x,y) \
	(a)[BOX_X]=x,(a)[BOX_Y]=y

#define BOXRelXY(a,x,y) \
	(a)[BOX_X]=(x*BOX_REL_W/BOX_ABS_W),(a)[BOX_Y]=y*BOX_REL_H/BOX_ABS_H

#define BOXSetWH(a,w,h) \
	(a)[BOX_W]=w,(a)[BOX_H]=h

//
// INTERPOLATION
//

extern float InterpLinear(float a, float b, float x);
extern void InterpVec3Linear(float* a, float* b, float* out, float percent);
extern void InterpVec3Linear_NoClamp(float* a, float* b, float* out, float percent);
extern float InterpCosine(float a, float b, float x);

//
// OTHER
//
extern void MathV3AlignRot(crd* rotOut, crd* vecOrig, crd* vecDest);
extern float MathBlendRot(float rot0,float rot1,float blend0,float blend1);
extern float MathAddRot(float rot0,float rot1 );
extern float MathSubRot(float rot0,float rot1 );
EXTERN_C float MathSqrt(float a);
extern float MathFastSqrt(float a);
extern double MathCosSind(double a,double *sinp);
extern crd MathCosSin(crd a,crd *sinp);
extern int MathPDUToCamM4(crd *m4,crd *pos,crd *dir,crd *up);
extern int MathCamToPDUM4(crd *m4,crd *pos,crd *dir,crd *up);
EXTERN_C void MathNrmToRot(crd *nrm,crd *rot);
extern void MathRotToNrm(crd *rot, crd *nrm);
extern void MathQuatToRotXYZ(crd* rotOut, crd* qIn);
extern void MathQuatToRotXYZ_NoCheck(crd* rotOut, crd* qIn);
extern void MathRotXYZToQuat(crd* qOut, crd* rotIn);
extern void MathRotConcat(crd* rotOut, crd* rot1, crd* rot2);
EXTERN_C void MathM3ToRot(crd *m3,crd *rot);
EXTERN_C void MathM3ToRot2(crd *m3,crd *rot);
extern void MathLookAt(crd * dstRot, crd* pos, crd* target);
extern void CameraLookAt(crd * dstRot, crd* pos, crd* target);
extern void MathLookAtUp(crd* dstRot, crd* pos, crd* tgt, crd* up);
extern void MathLookAtXZ(crd * dstRot, crd* pos, crd* target);
// separate XYZ rotation into heading vector and roll angle (in radians)
extern void MathRotToHeadingAndRoll(crd* rotIn, crd* headingOut, float* rollOut);

// extern void MathM3ToRotNew(crd *mat,crd *rot);
extern void MathM3ToScale(crd *m,crd *scale);
extern void MathM3Unscale(crd *m,crd *scale);
extern crd MathACos(crd a);
extern crd MathASin(crd a);
extern crd MathATan(crd a);
extern crd MathTan(crd a);

extern void RandUSeed(uns seed);
extern uns RandU(uns top);
extern float RandFOne(void);
EXTERN_C float RandFRange(float low,float high);
extern uns32 RandUGameType(uns32 n, int gameTourn);
extern float RandFGameType(float n, int gameTourn);
extern void RandInitSeedTourn(void);
extern int32 RandSeedTourn;	// special tournament random seed
extern uns32 RandUTourn(uns32 n);
extern int32 RandRangeTourn(int32 min, int32 max);
extern int32 RandSignTourn(int32 n);
extern float RandFTourn(float n);
extern float RandFSTourn(float n);
extern float RandFRangeTourn(float min, float max);
extern float RandSign();

extern int IsPowerOf2(int x);
extern int NearestPow2(int num);

extern int MathPointTriangleColTest2D( float *point, float *tv1, float *tv2, float *tv3 );


// this kind of funny place to stick these, but where else to put them?
extern uns32 rgbaFloatToUns32( float *rgba_in, int argb_format );
extern void rgbaUns32ToFloat( uns32 rgba_in, float *rgba_out, int argb_format );
void hsv2rgb( float* hsv_in, float* rgb_out );




float CalcBlendPos( float cur, float min, float max );

// given a target spread for blending three animation layers
// where in[0], in[1], in[2] are the desired final blend values for
// layers 0,1, and 2 such that in[0] + in[1] + in[2] = 1.0f
// ouputs three blend values for blending in order:
// layer 0 with bind pos (1.0f)
// layer 0 and layer 1
// combined layer 0+1, and layer 2
// so if you have an Otan w/ three layers
// this func will give you blend values for each layer
// in order to achieve the desired final blend combination....
void Calc3WayBlendValues( float *in, float *out );
// same as above but handles n layers...
void CalcMulitWayBlendValues( int count, float *in, float *out );


typedef struct
{
	float time; // time since start of blend
	float initial; // initial value
	float target; // target value
	float blend_time; // how long to spend blending
	float velocity; // (target - initial) / blend_time
	float blend;  // current blend value
	float current;  // current value
} timed_blend;

int timed_blend_init( timed_blend *tb, float initial, float target,  float blend_time );
int timed_blend_steptime( timed_blend *tb, float time, int relative );
float timed_blend_get_current_value( timed_blend *tb );


// math and trig lookup tables
// 
typedef struct 
{
	float (*func)(float);  // function that lookup table corresponds to
	float min;
	float max;
	float cmult; // multiplier for converting from input floats to table indices
	int num_values;
	float *values;
} math_lookup_table;

int mlut_init( math_lookup_table *mlut, float (*func)(float), float min, float max, int num_values );

#define mlut_get_val( table, f )  (table.values[ (int)(((f)*table.cmult)-min) % table.num_values ])
// same the above, but the range is assumed to start at 0
#define mlut_get_val_min0( table, f )  (table.values[ (int)((f)*table.cmult) % table.num_values ])
// same the above, but f is assumed to be in valid range
#define mlut_get_val_min0_unsafe( table, f )  (table.values[ (int)((f)*table.cmult) ])


extern math_lookup_table gSinTable;
extern math_lookup_table gCosTable;
extern math_lookup_table gTanTable;

#define sin_fast(f) ( gSinTable.values != NULL ? mlut_get_val_min0(gSinTable,f) : sinf(f) )
#define cos_fast(f) ( gCosTable.values != NULL ? mlut_get_val_min0(gCosTable,f) : cosf(f) )
#define tan_fast(f) ( gTanTable.values != NULL ? mlut_get_val_min0(gTanTable,f) : tanf(f) )

#define sin_fast_unsafe(f) ( gSinTable.values != NULL ? mlut_get_val_min0(gSinTable,f) : sinf(f) )
#define cos_fast_unsafe(f) ( gCosTable.values != NULL ? mlut_get_val_min0(gCosTable,f) : cosf(f) )
#define tan_fast_unsafe(f) ( gTanTable.values != NULL ? mlut_get_val_min0(gTanTable,f) : tanf(f) )

float Make8bitFloat( float in );

// return true if ray intersects sphere. vectors are in world space.
int raySphereIntersect( crd* vRayStart, crd* vRayDir, crd* spherePos, float sphereRad, float* tOut );


#define MATRIX_STACK_MAX_SIZE 8

typedef struct MatrixStack
{
	int depth;
	float stack[MATRIX_STACK_MAX_SIZE][M4_SIZE];
	float matrix[M4_SIZE];
} MatrixStack;

EXTERN_C int    MatrixStack_Init(MatrixStack *mstack);
EXTERN_C void   MatrixStack_Compute(MatrixStack *mstack);
EXTERN_C int    MatrixStack_Push(MatrixStack *mstack);
EXTERN_C int    MatrixStack_PushM4(MatrixStack *mstack, float *m4);
EXTERN_C int    MatrixStack_PushTrans(MatrixStack *mstack, float x, float y, float z );
EXTERN_C int    MatrixStack_Trans(MatrixStack *mstack, float x, float y, float z );
EXTERN_C int    MatrixStack_MultM4(MatrixStack *mstack, float *m4 );

EXTERN_C int    MatrixStack_Pop(MatrixStack *mstack);
EXTERN_C float *MatrixStack_Get(MatrixStack *mstack);


#define MODELVIEWPROJ_FLAGS_PROJ_IS_ID      0x00000001
#define MODELVIEWPROJ_FLAGS_VIEW_IS_ID      0x00000002
#define MODELVIEWPROJ_FLAGS_MODEL_IS_ID     0x00000004



typedef struct ModelViewProjectionMatrix
{
	uns32 flags;
	// matrices
	float Projection[M4_SIZE];
	float View[M4_SIZE];
	float Model[M4_SIZE];
	// computed from above
	float ModelViewProjection[M4_SIZE];
	float ViewProjection[M4_SIZE];
	float ModelView[M4_SIZE];
	float Normal[M3_SIZE];  // rotation only vers of modelview, useful when working with normals and tangents, in format ready to hand to gl as mat3x3 (gl promotes to 3x4)

	// inverses, these are only computed on request
	float iProjection[M4_SIZE];
	float iView[M4_SIZE];
	float iModel[M4_SIZE];
	float iModelView[M4_SIZE];
	float iViewProjection[M4_SIZE];
	float iModelViewProjection[M4_SIZE];

} ModelViewProjectionMatrix;

// setting a matrix at a higher level of the stack, will load identity matrix into lower levels (eg. loading a proj matrix causes view and model matrices to be set to id)
EXTERN_C void ModelViewProjectionMatrix_SetProjectionMatrix( ModelViewProjectionMatrix *mvp, float *m4 );
EXTERN_C void ModelViewProjectionMatrix_SetViewMatrix( ModelViewProjectionMatrix *mvp, float *m4 );
EXTERN_C void ModelViewProjectionMatrix_SetViewMatrixByVM( ModelViewProjectionMatrix *mvp, float *vm );
EXTERN_C void ModelViewProjectionMatrix_SetModelMatrix( ModelViewProjectionMatrix *mvp, float *m4 );
EXTERN_C void ModelViewProjectionMatrix_SetModelMatrixByVM( ModelViewProjectionMatrix *mvp, float *vm );
EXTERN_C void ModelViewProjectionMatrix_SetProjectionMatrixProj( ModelViewProjectionMatrix *mvp, double camx, double camy, double lens, double fovy,double aspect,double znear,double zfar );
EXTERN_C void ModelViewProjectionMatrix_SetProjectionMatrixOrtho( ModelViewProjectionMatrix *mvp, float left, float right, float bottom, float top, float nearv, float farv );

EXTERN_C void ModelViewProjectionMatrix_ComputeDerived( ModelViewProjectionMatrix *mvp, int ModelViewProj, int ViewProj, int ModelView );
EXTERN_C void ModelViewProjectionMatrix_ComputeInverses( ModelViewProjectionMatrix *mvp, int iProj, int iView, int iModel, int iModelView, int iViewProj, int iModelViewProj );



int VecInit();
int VecFinal();



#endif // ndef VEC_H
