// Wind.h
// Copyright 2010 Play Mechanix Inc. All Rights Reserved
//

#ifndef WIND_H
#define WIND_H

#include "g5.h"


// this definition moved to core_rtdt_datatypes.h
/*typedef struct WindParams
{
	float dir[3];
	float speed;
	float strength;
	float noiseCoordMult;
	float tangentialRatio;
	float tangentialCoordMult;
	float currentOffset[3];
} WindParams;*/

#define WINDPARAMS_DIRTY		0x00000001 // params have changed since last frame, need to recompute dir and rot axis
#define WINDPARAMS_CLEAR_DIRTY  0x00000002 // clear the dirty bit at start of next update...

// global instance
extern WindParamList gWindParams;

extern int gEnvColUseGlobalParams;
extern EnvColParams currentEnvColParams;

typedef struct WindedElements
{
	uns32 elapsed_frames;
	uns32 update_frame;
	float strength_start;
	float strength_end;
	float strength;
	float pos[3];
	float shearDir[3];
	float rotAxis[3];
	float stiffness[2];
	float transform[9];

	// collision leaning
	EnvColParams* colParams;
	int useLean;
	int isLeaning;
	int wasLeaning;
	float currentLean[3];
	float leanDir[3]; // store it away so we can wobble later

	// post-lean wobble
	int isWobbling;
	int wobbleTimeLeft;
	float minRadius;
} WindedElements;

typedef struct GrassEffector
{
	uns32 oid;
	float radiusMult;
} GrassEffector;
	
typedef struct Wind
{
	WindedElements* elements;
	int numElements;
	int numTransformFloats;
	float offset;
} Wind;

// init the wind system
int	EnvironAnimInit();
int	EnvironAnimFinal();
// reload env anim config files
int EnvironAnimReload();
// destroy a wind object
void	WindDestroy(Wind* wind);

// add wind-effected elements
void WindAddElements(Wind* wind, float* positions, float* stiffnesses, float* shearDir, int count, int lean, void* obj, char* envColPreset);
// set the positions of the wind-effected elements
void	WindSetPositions(Wind* wind, float* positions, int count);
// get the transforms (and stride [in # of floats] between transforms) of wind elements
int		WindGetTransforms(Wind* wind, float** transforms, uns32* stride);

int		EnvironmentAddLeanEffector(uns32 oid, float radiusMult);
EnvColParams* EnvColGetParams(char* name);
int		EnvColLoadFile( char *file, void *data);
int		EnvColSave( EnvColParams *params );

int		WindGetNumElements(Wind* wind);

void	WindUpdate(Wind* wind, uns32 deltaTime);
int		saveWind(char *name);
int		loadWind(char *name);
int		resetWind();



int WindLoop(uns32 deltaTime);



#endif

